package com.skybeacon.sdk.config;

import java.io.Serializable;

import com.skybeacon.sdk.utils.DefaultStaticValues;

/**
 * 多id的格子使用情况类
 * 
 * @author frank
 * 
 */
public class SKYBeaconCellsSituation implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	// 格子使用数量
	private int usedCellsNum = DefaultStaticValues.DEFAULT_SKY_BEACON_CELLS_USED_NUM;
	// 9个格子
	private SKYBeaconBroadcastID[] usedCellsSituation = new SKYBeaconBroadcastID[] {
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
			SKYBeaconBroadcastID.BROADCAST_ID_FALSE };

	/**
	 * 构造函数
	 * 
	 * @param cellsSitution
	 *            格子使用情况
	 */
	public SKYBeaconCellsSituation(SKYBeaconCellsSituation cellsSitution) {
		this.usedCellsSituation[0] = cellsSitution.getUsedCellsSituation()[0];
		this.usedCellsSituation[1] = cellsSitution.getUsedCellsSituation()[1];
		this.usedCellsSituation[2] = cellsSitution.getUsedCellsSituation()[2];
		this.usedCellsSituation[3] = cellsSitution.getUsedCellsSituation()[3];
		this.usedCellsSituation[4] = cellsSitution.getUsedCellsSituation()[4];
		this.usedCellsSituation[5] = cellsSitution.getUsedCellsSituation()[5];
		this.usedCellsSituation[6] = cellsSitution.getUsedCellsSituation()[6];
		this.usedCellsSituation[7] = cellsSitution.getUsedCellsSituation()[7];
		this.usedCellsSituation[8] = cellsSitution.getUsedCellsSituation()[8];
		this.usedCellsNum = cellsSitution.getUsedCellsNum();
		processCellsSort();
	}

	/**
	 * 构造函数，只使用一个格子的情况
	 * 
	 * @param id1
	 *            这个格子对应的id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1) {
		usedCellsSituation[0] = id1;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用两个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用三个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 * @param id3
	 *            格子3的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2, SKYBeaconBroadcastID id3) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsSituation[2] = id3;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id3 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用9个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 * @param id3
	 *            格子3的广播id
	 * @param id4
	 *            格子4的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2, SKYBeaconBroadcastID id3,
			SKYBeaconBroadcastID id4) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsSituation[2] = id3;
		usedCellsSituation[3] = id4;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id3 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id4 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用9个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 * @param id3
	 *            格子3的广播id
	 * @param id4
	 *            格子4的广播id
	 * @param id5
	 *            格子5的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2, SKYBeaconBroadcastID id3,
			SKYBeaconBroadcastID id4, SKYBeaconBroadcastID id5) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsSituation[2] = id3;
		usedCellsSituation[3] = id4;
		usedCellsSituation[4] = id5;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id3 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id4 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id5 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用9个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 * @param id3
	 *            格子3的广播id
	 * @param id4
	 *            格子4的广播id
	 * @param id5
	 *            格子5的广播id
	 * @param id6
	 *            格子6的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2, SKYBeaconBroadcastID id3,
			SKYBeaconBroadcastID id4, SKYBeaconBroadcastID id5,
			SKYBeaconBroadcastID id6) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsSituation[2] = id3;
		usedCellsSituation[3] = id4;
		usedCellsSituation[4] = id5;
		usedCellsSituation[5] = id6;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id3 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id4 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id5 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id6 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用9个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 * @param id3
	 *            格子3的广播id
	 * @param id4
	 *            格子4的广播id
	 * @param id5
	 *            格子5的广播id
	 * @param id6
	 *            格子6的广播id
	 * @param id7
	 *            格子7的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2, SKYBeaconBroadcastID id3,
			SKYBeaconBroadcastID id4, SKYBeaconBroadcastID id5,
			SKYBeaconBroadcastID id6, SKYBeaconBroadcastID id7) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsSituation[2] = id3;
		usedCellsSituation[3] = id4;
		usedCellsSituation[4] = id5;
		usedCellsSituation[5] = id6;
		usedCellsSituation[6] = id7;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id3 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id4 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id5 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id6 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id7 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用9个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 * @param id3
	 *            格子3的广播id
	 * @param id4
	 *            格子4的广播id
	 * @param id5
	 *            格子5的广播id
	 * @param id6
	 *            格子6的广播id
	 * @param id7
	 *            格子7的广播id
	 * @param id8
	 *            格子8的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2, SKYBeaconBroadcastID id3,
			SKYBeaconBroadcastID id4, SKYBeaconBroadcastID id5,
			SKYBeaconBroadcastID id6, SKYBeaconBroadcastID id7,
			SKYBeaconBroadcastID id8) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsSituation[2] = id3;
		usedCellsSituation[3] = id4;
		usedCellsSituation[4] = id5;
		usedCellsSituation[5] = id6;
		usedCellsSituation[6] = id7;
		usedCellsSituation[7] = id8;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id3 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id4 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id5 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id6 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id7 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id8 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	/**
	 * 构造函数，使用9个格子的情况
	 * 
	 * @param id1
	 *            格子1的广播id
	 * @param id2
	 *            格子2的广播id
	 * @param id3
	 *            格子3的广播id
	 * @param id4
	 *            格子4的广播id
	 * @param id5
	 *            格子5的广播id
	 * @param id6
	 *            格子6的广播id
	 * @param id7
	 *            格子7的广播id
	 * @param id8
	 *            格子8的广播id
	 * @param id9
	 *            格子9的广播id
	 */
	public SKYBeaconCellsSituation(SKYBeaconBroadcastID id1,
			SKYBeaconBroadcastID id2, SKYBeaconBroadcastID id3,
			SKYBeaconBroadcastID id4, SKYBeaconBroadcastID id5,
			SKYBeaconBroadcastID id6, SKYBeaconBroadcastID id7,
			SKYBeaconBroadcastID id8, SKYBeaconBroadcastID id9) {
		usedCellsSituation[0] = id1;
		usedCellsSituation[1] = id2;
		usedCellsSituation[2] = id3;
		usedCellsSituation[3] = id4;
		usedCellsSituation[4] = id5;
		usedCellsSituation[5] = id6;
		usedCellsSituation[6] = id7;
		usedCellsSituation[7] = id8;
		usedCellsSituation[8] = id9;
		usedCellsNum = 0;
		if (id1 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id2 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id3 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id4 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id5 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id6 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id7 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id8 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		if (id9 != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
			usedCellsNum++;
		}
		processCellsSort();
	}

	public SKYBeaconBroadcastID[] getUsedCellsSituation() {
		return usedCellsSituation;
	}

	public int getUsedCellsNum() {
		return usedCellsNum;
	}

	public void setUsedCellsNum(int num) {
		this.usedCellsNum = num;
		for (int i = 8; i > (num - 1); i--) {
			usedCellsSituation[i] = SKYBeaconBroadcastID.BROADCAST_ID_FALSE;
		}
	}

	private void processCellsSort() {
		SKYBeaconBroadcastID[] usedCellsSituationTmp = new SKYBeaconBroadcastID[] {
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE,
				SKYBeaconBroadcastID.BROADCAST_ID_FALSE };

		int j = 0;
		for (int i = 0; i < 9; i++) {
			if (this.usedCellsSituation[i] != SKYBeaconBroadcastID.BROADCAST_ID_FALSE) {
				usedCellsSituationTmp[j] = this.usedCellsSituation[i];
				j++;
			}
		}

		for (int i = 0; i < 9; i++) {
			this.usedCellsSituation[i] = usedCellsSituationTmp[i];
		}
	}
}
