package com.skybeacon.sdk.config;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import com.skybeacon.sdk.config.SKYBeaconVersionUtil.WriteFlags;
import com.skybeacon.sdk.locate.SKYBeacon;
import com.skybeacon.sdk.locate.SKYBeaconMultiIDs;
import com.skybeacon.sdk.utils.DatasCheck;
import com.skybeacon.sdk.utils.DefaultStaticValues;

import android.app.Activity;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

/**
 * SeekcyBeacon通信封装类
 * 
 * @author frank
 * 
 */
public class SKYBeaconCommunicateProcess {

	/**
	 * 配置中每个步骤的超时时间
	 */
	private static final int CONFIG_STEP_TIMEOUT_MILLISECOND = 15000;
	/**
	 * 默认的key
	 */
	private static final byte[] DEFAULT_CONFIG_KEY_BYTES = new byte[] {
			(byte) 0xD3, (byte) 0xC5, (byte) 0xC5, (byte) 0xCB, (byte) 0xC3,
			(byte) 0xD9 };

	public static class GattAttributes {
		static public String SEEYCY_BEACON_COMMUNICATE_SERVICE = "0000fff0-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_1_READ_UUID = "0000fff1-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_2_READ_UUID = "0000fff2-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_3_READ_UUID = "0000fff3-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_4_READ_UUID = "0000fff4-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_5_READ_UUID = "0000fff5-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_6_READ_UUID = "0000fff6-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_7_READ_UUID = "0000fff7-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_9_READ_UUID = "0000fff9-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_10_READ_UUID = "0000fffa-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_12_READ_UUID = "0000fffc-0000-1000-8000-00805f9b34fb";
	}

	public class GattAttributesTI {
		static public final String SEEYCY_BEACON_COMMUNICATE_SERVICE = "0000fff0-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_1_READ_UUID = "0000fff1-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_2_READ_UUID = "0000fff2-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_3_READ_UUID = "0000fff3-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_4_READ_UUID = "0000fff4-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_5_READ_UUID = "0000fff5-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_6_READ_UUID = "0000fff6-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_7_READ_UUID = "0000fff7-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_9_READ_UUID = "0000fff9-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_10_READ_UUID = "0000fffa-0000-1000-8000-00805f9b34fb";
		static public final String CHARACTERISTIC_12_READ_UUID = "0000fffc-0000-1000-8000-00805f9b34fb";
	}

	// 0783b03e-8535-b5a0-7140-a304d2495cb0
	public class GattAttributesDialog {
		static public final String SEEYCY_BEACON_COMMUNICATE_SERVICE = "0783b03e-8535-b5a0-7140-a304d2495cb0";
		static public final String CHARACTERISTIC_1_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb1";
		static public final String CHARACTERISTIC_2_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb2";
		static public final String CHARACTERISTIC_3_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb3";
		static public final String CHARACTERISTIC_4_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb4";
		static public final String CHARACTERISTIC_5_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb5";
		static public final String CHARACTERISTIC_6_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb6";
		static public final String CHARACTERISTIC_7_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb7";
		static public final String CHARACTERISTIC_9_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cb9";
		static public final String CHARACTERISTIC_10_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cba";
		static public final String CHARACTERISTIC_12_READ_UUID = "0783b03e-8535-b5a0-7140-a304d2495cbc";
	}

	public class ConfigReadStates {
		static public final int READ_STATE_INIT = -1;
		static public final int READ_STATE_FAILED = 11;
		static public final int READ_STATE_TIMEOUT = 12;
		static public final int READ_STATE_FALSE_PROTOCOL = 21;
		static public final int READ_STATE_EMPTY = 22;
		static public final int READ_STATE_RETURN_KEY_SUCCESS = 31;
		static public final int READ_STATE_RETURN_CONFIG_SUCCESS = 32;
		static public final int READ_STATE_RETURN_CONFIG_PAUSE = 33;
		static public final int READ_STATE_RETURN_CONFIG_PASSCODE_ERROR = 34;
		static public final int READ_STATE_RETURN_CONFIG_HEADER_ERROR = 35;
		static public final int READ_STATE_GATT_SERVICE_NOT_FOUND = 41;
		static public final int READ_STATE_CHARACTERISTIC_NOT_FOUND = 42;
		static public final int READ_STATE_CHARACTERISTIC_1_INIT = 101;
		static public final int READ_STATE_CHARACTERISTIC_2_INIT = 102;
		static public final int READ_STATE_CHARACTERISTIC_3_INIT = 103;
		static public final int READ_STATE_CHARACTERISTIC_4_INIT = 104;
		static public final int READ_STATE_CHARACTERISTIC_5_INIT = 105;
		static public final int READ_STATE_CHARACTERISTIC_6_INIT = 106;
		static public final int READ_STATE_CHARACTERISTIC_7_INIT = 107;
		static public final int READ_STATE_CHARACTERISTIC_9_INIT = 109;
		static public final int READ_STATE_CHARACTERISTIC_10_INIT = 110;
		static public final int READ_STATE_CHARACTERISTIC_12_INIT = 112;

		static public final int READ_STATE_CHARACTERISTIC_1_END = 201;
		static public final int READ_STATE_CHARACTERISTIC_2_END = 202;
		static public final int READ_STATE_CHARACTERISTIC_3_END = 203;
		static public final int READ_STATE_CHARACTERISTIC_4_END = 204;
		static public final int READ_STATE_CHARACTERISTIC_5_END = 205;
		static public final int READ_STATE_CHARACTERISTIC_6_END = 206;
		static public final int READ_STATE_CHARACTERISTIC_7_END = 207;
		static public final int READ_STATE_CHARACTERISTIC_9_END = 209;
		static public final int READ_STATE_CHARACTERISTIC_10_END = 210;
		static public final int READ_STATE_CHARACTERISTIC_12_END = 212;
	}

	private static final String TAG = "SKYBeaconCommunicateProcess";
	private Context context;
	private boolean isConfiging = false;

	private BleServiceConnection serviceConnection = null;
	private BluetoothLeService bluetoothLeService = null;
	private boolean isBind = false;
	private boolean isRegistered = false;

	private Object lockedWrite = new Object();
	private Object lockedRead = new Object();
	private boolean isDeviceConnected = false;
	private boolean isGattServicesDiscover = false;
	private int readState = ConfigReadStates.READ_STATE_INIT;
	private boolean writeState = false;
	private byte randomKey = 0x00;

	private byte[] readDatas = new byte[20];
	private int readDatasLength = 0;
	private boolean isMultiIDsBeacon = false;
	private SKYBeacon skyBeacon;
	private SKYBeaconMultiIDs skyBeaconMultiIDs;

	private Handler handler;

	/**
	 * 监听器接口
	 * 
	 * @author frank
	 * 
	 */
	public interface HandlerMessageListener {
		void handlerMessage(final Message msg);
	}

	private HandlerMessageListener handlerMessageListener = null;

	/**
	 * 添加配置监听器
	 * 
	 * @param handlerMessageListener
	 *            listener
	 */
	public void setOnHandlerMessageListener(
			HandlerMessageListener handlerMessageListener) {
		this.handlerMessageListener = handlerMessageListener;
	}

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public SKYBeaconCommunicateProcess(Context context, Handler handler) {
		this.context = context;
		this.handler = handler;
	}

	/**
	 * 打断配置，在任何异常退出连接、配置情况下，需要调用此函数
	 */
	public void interruptConfig() {
		try {
			skyBeacon = null;
			skyBeaconMultiIDs = null;
			if (isConfiging) {
				isConfiging = false;
				synchronized (lockedWrite) {
					lockedWrite.notifyAll();
				}
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
				if (isRegistered) {
					context.unregisterReceiver(gattUpdateReceiver);
					isRegistered = false;
				}
				if (bluetoothLeService != null) {
					bluetoothLeService.disconnect();
					// bluetoothLeService.close();
				}
				serviceConnection.onServiceDisconnected(((Activity) context)
						.getComponentName());
				if (isBind) {
					context.unbindService(serviceConnection);
				}
			}
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	/**
	 * 连接设备，包括数据读取后才算成功，单id的iBeacon 阻塞模式
	 * 
	 * @param skyBeacon
	 * @return 返回错误码
	 */
	public int connectDevice(SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		this.skyBeacon = new SKYBeacon();
		this.skyBeacon = (SKYBeacon) skyBeacon.clone();
		// 初始化成员变量
		isMultiIDsBeacon = false;
		isConfiging = true;
		isDeviceConnected = false;
		isGattServicesDiscover = false;
		readState = ConfigReadStates.READ_STATE_INIT;
		writeState = false;
		randomKey = 0x00;
		// 连接设备
		int flag = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		try {
			flag = connectDevice(skyBeacon.getDeviceAddress());
			// 连接成功的情况下继续进行配置
			if (flag == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				// 更新GattService
				getGattAttributes();
				// 读取随机密钥
				readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_1_INIT;
				flag = readBle(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID));
				synchronized (lockedRead) {
					try {
						lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
					} catch (InterruptedException e) {
						e.printStackTrace();
					} catch (Exception e) {
						// TODO: handle exception
						e.printStackTrace();
					}
				}
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}
				if (readState != ConfigReadStates.READ_STATE_RETURN_KEY_SUCCESS) {
					result = ConfigInfoCode.ErrorCode.CODE_READ_RANDOM_KEY_FAILED;
					return result;
				}
				// 读取characteristic 2
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_2_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}

				// 读取characteristic 3
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_3_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}
				// 读取characteristic 7
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_7_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}

				ArrayList<String> readChs = SKYBeaconVersionUtil
						.getReadCharacteristicSingleId(
								this.skyBeacon.getHardwareVersion(),
								this.skyBeacon.getFirmwareVersionMajor(),
								this.skyBeacon.getFirmwareVersionMinor());

				for (int i = 0; i < readChs.size(); i++) {
					flag = readCharacteristic(
							UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
							UUID.fromString(readChs.get(i)));
					if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						result = flag;
						return result;
					}
				}

				// DA14580处理
				if (SKYBeaconVersionUtil.isDA14580(
						this.skyBeacon.getHardwareVersion(),
						this.skyBeacon.getFirmwareVersionMajor(),
						this.skyBeacon.getFirmwareVersionMinor())) {
					// 确定配置过程中的密钥
					String keyDefault = "";
					String usedKeyThis = "";
					try {
						keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
								DefaultStaticValues.STRING_BYTES_CONVERT);
					} catch (UnsupportedEncodingException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					if (this.skyBeacon.isLocked() == 1) {
						usedKeyThis = skyBeacon.getKey();
					} else {
						usedKeyThis = keyDefault;
					}

					flag = configDA14580Pairing(usedKeyThis);
					if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						result = flag;
						return result;
					}

					flag = readConfigFinish();
					if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						result = flag;
						return result;
					}
				}

				this.skyBeacon.setSeekcyBeacon(1);
			} else {
				// interruptConfig();
				result = flag;
			}
		} catch (Exception e) {
			// TODO: handle exception
			flag = ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED;
		}
		return result;
	}

	/**
	 * 连接设备，包括数据读取后才算成功，多id的iBeacon 阻塞模式
	 * 
	 * @param skyBeacon
	 * @return 返回错误码
	 */
	public int connectDevice(SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		this.skyBeaconMultiIDs = new SKYBeaconMultiIDs();
		this.skyBeaconMultiIDs = (SKYBeaconMultiIDs) skyBeaconMultiIDs.clone();
		// 初始化成员变量
		isMultiIDsBeacon = true;
		isConfiging = true;
		isDeviceConnected = false;
		isGattServicesDiscover = false;
		readState = ConfigReadStates.READ_STATE_INIT;
		writeState = false;
		randomKey = 0x00;
		// 连接设备
		int flag = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (skyBeaconMultiIDs.getDeviceAddress() == null) {
			result = ConfigInfoCode.ErrorCode.CODE_FALSE_DEVICE_ADDRESS;
			return result;
		}
		if (skyBeaconMultiIDs.getDeviceAddress().equals("")) {
			result = ConfigInfoCode.ErrorCode.CODE_FALSE_DEVICE_ADDRESS;
			return result;
		}
		try {

			flag = connectDevice(skyBeaconMultiIDs.getDeviceAddress());
			// 连接成功的情况下继续进行配置
			if (flag == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				// 更新GattService
				getGattAttributes();
				// 读取随机密钥
				readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_1_INIT;
				flag = readBle(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID));
				synchronized (lockedRead) {
					try {
						lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
					} catch (InterruptedException e) {
						e.printStackTrace();
					} catch (Exception e) {
						// TODO: handle exception
						e.printStackTrace();
					}
				}
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}
				if (readState != ConfigReadStates.READ_STATE_RETURN_KEY_SUCCESS) {
					result = ConfigInfoCode.ErrorCode.CODE_READ_RANDOM_KEY_FAILED;
					return result;
				}
				// 读取characteristic 2
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_2_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}
				// 判断是否需要读取uuid major minor信息
				boolean[] reads = new boolean[] { false, false, false };
				for (int i = 0; i < 9; i++) {
					if (SKYBeaconBroadcastID.getInt(this.skyBeaconMultiIDs
							.getCellsSituation().getUsedCellsSituation()[i]) == 1) {
						reads[0] = true;
					}
					if (SKYBeaconBroadcastID.getInt(this.skyBeaconMultiIDs
							.getCellsSituation().getUsedCellsSituation()[i]) == 2) {
						reads[1] = true;
					}
					if (SKYBeaconBroadcastID.getInt(this.skyBeaconMultiIDs
							.getCellsSituation().getUsedCellsSituation()[i]) == 3) {
						reads[2] = true;
					}
				}
				// 读取characteristic 3
				// if (reads[0]) {
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_3_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}
				// }
				// 读取characteristic 4
				// if (reads[1]) {
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_4_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}
				// }
				// 读取characteristic 5
				// if (reads[2]) {
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_5_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}
				// }
				// 读取characteristic 7
				flag = readCharacteristic(
						UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
						UUID.fromString(GattAttributes.CHARACTERISTIC_7_READ_UUID));
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					return result;
				}

				ArrayList<String> readChs = SKYBeaconVersionUtil
						.getReadCharacteristicMultiId(this.skyBeaconMultiIDs
								.getHardwareVersion(), this.skyBeaconMultiIDs
								.getFirmwareVersionMajor(),
								this.skyBeaconMultiIDs
										.getFirmwareVersionMinor());

				for (int i = 0; i < readChs.size(); i++) {
					flag = readCharacteristic(
							UUID.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE),
							UUID.fromString(readChs.get(i)));
					if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						result = flag;
						return result;
					}
				}

				// DA14580处理
				if (SKYBeaconVersionUtil.isDA14580(
						this.skyBeaconMultiIDs.getHardwareVersion(),
						this.skyBeaconMultiIDs.getFirmwareVersionMajor(),
						this.skyBeaconMultiIDs.getFirmwareVersionMinor())) {
					// 确定配置过程中的密钥
					String keyDefault = "";
					String usedKeyThis = "";
					try {
						keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
								DefaultStaticValues.STRING_BYTES_CONVERT);
					} catch (UnsupportedEncodingException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					if (this.skyBeaconMultiIDs.isLocked() == 1) {
						usedKeyThis = skyBeaconMultiIDs.getKey();
					} else {
						usedKeyThis = keyDefault;
					}

					flag = configDA14580Pairing(usedKeyThis);
					if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						result = flag;
						return result;
					}

					flag = readConfigFinish();
					if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						result = flag;
						return result;
					}
				}

				this.skyBeaconMultiIDs.setSeekcyBeacon(1);
			} else {
				// interruptConfig();
				result = flag;
			}
		} catch (Exception e) {
			// TODO: handle exception
			flag = ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED;
		}
		return result;
	}

	/**
	 * SeekcyBeacon配置，单id
	 * 
	 * @param skyBeaconConfig
	 * @return 返回错误码
	 */
	public int configBeacon(SKYBeaconConfig skyBeaconConfig) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		boolean configState = false;
		if (skyBeaconConfig == null) {
			result = ConfigInfoCode.ErrorCode.CODE_INPUT_NULL;
			return result;
		}
		if (isMultiIDsBeacon) {
			result = ConfigInfoCode.ErrorCode.CODE_USE_MULTI_ID_BEACON_REPLACE;
			return result;
		}
		if (skyBeacon.isLocked() == 1) {
			if (skyBeaconConfig.getLockedKey() == null) {
				result = ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE;
				return result;
			}
			if (skyBeaconConfig.getLockedKey().equals(
					DefaultStaticValues.DEFAULT_SKY_BEACON_LOCKED_KEY_FALSE)) {
				result = ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE;
				return result;
			}
		}
		// 确定配置过程中的密钥
		String keyDefault = "";
		String usedKeyThis = "";
		try {
			keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
					DefaultStaticValues.STRING_BYTES_CONVERT);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (skyBeacon.isLocked() == 1) {
			usedKeyThis = skyBeaconConfig.getLockedKey();
		} else {
			usedKeyThis = keyDefault;
		}
		// 防篡改模式是否需要配置
		if (skyBeaconConfig.isLocked() != DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE) {
			if (skyBeacon.isLocked() == 1 && skyBeaconConfig.isLocked() == 1) {
				// result = ConfigInfoCode.ErrorCode.CODE_BEACON_ALREADY_LOCKED;
				// return result;

				result = configKey(skyBeaconConfig.getLockedKey(),
						skyBeaconConfig.getLockedKey());
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}
			if (skyBeacon.isLocked() != 1 && skyBeaconConfig.isLocked() == 1) {
				result = configKey(keyDefault, skyBeaconConfig.getLockedKey());
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}
			if (skyBeacon.isLocked() == 1 && skyBeaconConfig.isLocked() == 0) {
				result = configKey(skyBeaconConfig.getLockedKey(), keyDefault);
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}
		}
		// device name是否需要配置
		if (!skyBeaconConfig.getDeviceName().equals(
				DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME)) {
			result = configDeviceName(skyBeaconConfig.getDeviceName(),
					usedKeyThis);
			configState = true;
			if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				return result;
			}
		}
		// 广播信息是否需要配置（子函数中进行判断）
		if (skyBeaconConfig.getProximityUUID().equals(
				DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)
				&& skyBeaconConfig.getMajor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE
				&& skyBeaconConfig.getMinor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE
				&& SKYBeaconPower.getPower(skyBeaconConfig.getTxpower()) == DefaultStaticValues.DEFAULT_SKY_BEACON_TXPOWER_FALSE
				&& skyBeaconConfig.getMeasuredPower() == DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE
				&& skyBeaconConfig.getIntervalMillisecond() == DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE
				&& skyBeaconConfig.isEncrypted() == DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE) {
		} else {
			result = configSingleIDBroadcast(skyBeaconConfig, usedKeyThis);
			configState = true;
			if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				return result;
			}
		}

		// 版本区分的WriteFlags
		ArrayList<Integer> writeFlags = SKYBeaconVersionUtil
				.getWriteFlagsSingleId(skyBeacon.getHardwareVersion(),
						skyBeacon.getFirmwareVersionMajor(),
						skyBeacon.getFirmwareVersionMinor());

		// led state配置
		if (skyBeaconConfig.getLedState() != DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE) {
			if (writeFlags.contains(WriteFlags.WRITE_FLAG_LED)) {
				result = configLedState(skyBeaconConfig.getLedState(),
						usedKeyThis);
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_LED_STATE_CONFIG_FAILED_VERSION_LOW;
			}
			configState = true;
			if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				return result;
			}
		}

		// temperature sensor配置
		if (skyBeaconConfig.getTemperatureUpdateSecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_UPDATE_FALSE) {
			if (writeFlags.contains(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR)) {
				result = configTemperatureUpdate(
						skyBeaconConfig.getTemperatureUpdateSecond(),
						usedKeyThis);
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_TEMPERATURE_SENSOR_STATE_CONFIG_FAILED_VERSION_LOW;
			}
			configState = true;
			if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				return result;
			}
		}

		// light sensor配置
		if (skyBeaconConfig.getLightState() != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_STATE
				|| LightSensorThreshold.getThreshold(skyBeaconConfig
						.getLightThreshold()) != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_DARK_THRESHOLD
				|| skyBeaconConfig.getLightUpdateSecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND
				|| skyBeaconConfig.getDarkIntervalMillisencond() != DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND) {
			if (skyBeaconConfig.getLightState() == 0
					|| (skyBeaconConfig.getLightState() == 1
							&& LightSensorThreshold
									.getThreshold(skyBeaconConfig
											.getLightThreshold()) != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_DARK_THRESHOLD
							&& skyBeaconConfig.getLightUpdateSecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND && skyBeaconConfig
							.getDarkIntervalMillisencond() != DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND)) {
				if (writeFlags.contains(WriteFlags.WRITE_FLAG_LIGHT_SENSOR)) {
					result = configLightSensor(usedKeyThis,
							skyBeaconConfig.getLightState(),
							LightSensorThreshold.getThreshold(skyBeaconConfig
									.getLightThreshold()),
							skyBeaconConfig.getLightUpdateSecond(),
							skyBeaconConfig.getDarkIntervalMillisencond());
				} else {
					result = ConfigInfoCode.ErrorCode.CODE_LIGHT_SENSOR_STATE_CONFIG_FAILED_VERSION_LOW;
				}
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_LIGHT_SENSOR_STATE_CONFIG_FAILED_VALUE_INCOMPLETE;
			}
			configState = true;
			if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				return result;
			}
		}
		// DA14580处理
		if (SKYBeaconVersionUtil.isDA14580(this.skyBeacon.getHardwareVersion(),
				this.skyBeacon.getFirmwareVersionMajor(),
				this.skyBeacon.getFirmwareVersionMinor())) {
			try {
				keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
						DefaultStaticValues.STRING_BYTES_CONVERT);
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if (skyBeacon.isLocked() == 1) {
				usedKeyThis = skyBeaconConfig.getLockedKey();
			} else {
				usedKeyThis = keyDefault;
			}

			result = configDA14580FinishConfig(usedKeyThis);
			if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				return result;
			}
		}

		// 配置结束标识读取
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT
				&& configState == true) {
			result = readConfigFinish();
		}

		return result;
	}

	/**
	 * SeekcyBeacon配置，多id
	 * 
	 * @param skyBeaconConfigMultiIDs
	 * @return 返回错误码
	 */
	public int configBeacon(SKYBeaconConfigMultiIDs skyBeaconConfigMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		boolean configState = false;
		if (skyBeaconConfigMultiIDs == null) {
			result = ConfigInfoCode.ErrorCode.CODE_INPUT_NULL;
			return result;
		}
		if (!isMultiIDsBeacon) {
			result = ConfigInfoCode.ErrorCode.CODE_USE_SINGLE_ID_BEACON_REPLACE;
			return result;
		}
		if (this.skyBeaconMultiIDs.isLocked() == 1) {
			if (skyBeaconConfigMultiIDs.getLockedKey() == null) {
				result = ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE;
				return result;
			}
			if (skyBeaconConfigMultiIDs.getLockedKey().equals(
					DefaultStaticValues.DEFAULT_SKY_BEACON_LOCKED_KEY_FALSE)) {
				result = ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE;
				return result;
			}
		}

		if (skyBeaconConfigMultiIDs.isReset()) {
			String keyDefault = "";
			String keyRaw = "";
			try {
				keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
						DefaultStaticValues.STRING_BYTES_CONVERT);
				keyRaw = skyBeaconConfigMultiIDs.getLockedKey();
			} catch (Exception e) {
				// TODO: handle exception
			}
			// 恢复出产设置
			result = configReset(keyRaw, keyDefault);
			configState = true;
			if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				return result;
			}

			if (this.skyBeaconMultiIDs.getHardwareVersion() == 0
					&& this.skyBeaconMultiIDs.getFirmwareVersionMajor() == 2
					&& this.skyBeaconMultiIDs.getFirmwareVersionMinor() == 0) {
			} else {
				// 配置结束标识读取
				if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT
						&& configState == true) {
					result = readConfigFinish();
				}
			}
		} else {
			// 确定配置过程中的密钥
			String keyDefault = "";
			String usedKeyThis = "";
			try {
				keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
						DefaultStaticValues.STRING_BYTES_CONVERT);
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if (this.skyBeaconMultiIDs.isLocked() == 1) {
				usedKeyThis = skyBeaconConfigMultiIDs.getLockedKey();
			} else {
				usedKeyThis = keyDefault;
			}
			// 防篡改模式是否需要配置
			if (skyBeaconConfigMultiIDs.isLocked() != DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE) {
				if (this.skyBeaconMultiIDs.isLocked() == 1
						&& skyBeaconConfigMultiIDs.isLocked() == 1) {
					result = configKey(skyBeaconConfigMultiIDs.getLockedKey(),
							skyBeaconConfigMultiIDs.getLockedKey());
					configState = true;
					if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						return result;
					}
				}
				if (this.skyBeaconMultiIDs.isLocked() != 1
						&& skyBeaconConfigMultiIDs.isLocked() == 1) {
					result = configKey(keyDefault,
							skyBeaconConfigMultiIDs.getLockedKey());
					configState = true;
					if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						return result;
					}
				}
				if (this.skyBeaconMultiIDs.isLocked() == 1
						&& skyBeaconConfigMultiIDs.isLocked() == 0) {
					result = configKey(skyBeaconConfigMultiIDs.getLockedKey(),
							keyDefault);
					configState = true;
					if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						return result;
					}
				}
			}
			// device name是否需要配置
			if (!skyBeaconConfigMultiIDs.getDeviceName().equals(
					DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME)) {
				result = configDeviceName(
						skyBeaconConfigMultiIDs.getDeviceName(), usedKeyThis);
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}
			// general参数是否需要配置
			if (!(skyBeaconConfigMultiIDs.getCellsSituation() == null && skyBeaconConfigMultiIDs
					.getIntervalMillisecond() == DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE)) {
				result = configMultiIDsGeneral(skyBeaconConfigMultiIDs,
						usedKeyThis);
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}
			// 广播信息是否需要配置（子函数中进行判断）
			SKYBeaconConfig[] skyBeaconList = skyBeaconConfigMultiIDs
					.getConfigBeacons();
			for (int i = 0; i < 3; i++) {
				if (skyBeaconList[i].getProximityUUID().equals(
						DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)
						&& skyBeaconList[i].getMajor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE
						&& skyBeaconList[i].getMinor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE
						&& SKYBeaconPower.getPower(skyBeaconList[i]
								.getTxpower()) == DefaultStaticValues.DEFAULT_SKY_BEACON_TXPOWER_FALSE
						&& skyBeaconList[i].getMeasuredPower() == DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE
						&& skyBeaconList[i].isEncrypted() == DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE) {
				} else {
					result = configMultiIDsBroadcast(skyBeaconList[i],
							usedKeyThis, i);
					configState = true;
					if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
						break;
					}
				}
			}
			// 配置时间
			if (skyBeaconConfigMultiIDs.isSetTime()) {
				result = configTime(usedKeyThis);
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}

			// 版本区分的WriteFlags
			ArrayList<Integer> writeFlags = SKYBeaconVersionUtil
					.getWriteFlagsMultiId(this.skyBeaconMultiIDs.getHardwareVersion(),
							this.skyBeaconMultiIDs.getFirmwareVersionMajor(),
							this.skyBeaconMultiIDs.getFirmwareVersionMinor());

			// led state配置
			if (skyBeaconConfigMultiIDs.getLedState() != DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE) {
				if (writeFlags.contains(WriteFlags.WRITE_FLAG_LED)) {
					result = configLedState(
							skyBeaconConfigMultiIDs.getLedState(), usedKeyThis);
				} else {
					result = ConfigInfoCode.ErrorCode.CODE_LED_STATE_CONFIG_FAILED_VERSION_LOW;
				}
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}

			// temperature sensor配置
			if (skyBeaconConfigMultiIDs.getTemperatureUpdateSecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_UPDATE_FALSE) {
				if (writeFlags
						.contains(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR)) {
					result = configTemperatureUpdate(
							skyBeaconConfigMultiIDs
									.getTemperatureUpdateSecond(),
							usedKeyThis);
				} else {
					result = ConfigInfoCode.ErrorCode.CODE_TEMPERATURE_SENSOR_STATE_CONFIG_FAILED_VERSION_LOW;
				}
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}

			// light sensor配置
			if (skyBeaconConfigMultiIDs.getLightState() != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_STATE
					|| LightSensorThreshold
							.getThreshold(skyBeaconConfigMultiIDs
									.getLightThreshold()) != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_DARK_THRESHOLD
					|| skyBeaconConfigMultiIDs.getLightUpdateSecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND
					|| skyBeaconConfigMultiIDs.getDarkIntervalMillisencond() != DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND) {
				if (skyBeaconConfigMultiIDs.getLightState() == 0
						|| (skyBeaconConfigMultiIDs.getLightState() == 1
								&& LightSensorThreshold
										.getThreshold(skyBeaconConfigMultiIDs
												.getLightThreshold()) != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_DARK_THRESHOLD
								&& skyBeaconConfigMultiIDs
										.getLightUpdateSecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND && skyBeaconConfigMultiIDs
								.getDarkIntervalMillisencond() != DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND)) {
					if (writeFlags
							.contains(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR)) {
						result = configLightSensor(usedKeyThis,
								skyBeaconConfigMultiIDs.getLightState(),
								LightSensorThreshold
										.getThreshold(skyBeaconConfigMultiIDs
												.getLightThreshold()),
								skyBeaconConfigMultiIDs.getLightUpdateSecond(),
								skyBeaconConfigMultiIDs
										.getDarkIntervalMillisencond());
					} else {
						result = ConfigInfoCode.ErrorCode.CODE_LIGHT_SENSOR_STATE_CONFIG_FAILED_VERSION_LOW;
					}
				} else {
					result = ConfigInfoCode.ErrorCode.CODE_LIGHT_SENSOR_STATE_CONFIG_FAILED_VALUE_INCOMPLETE;
				}
				configState = true;
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}

			// DA14580处理
			if (SKYBeaconVersionUtil.isDA14580(
					this.skyBeaconMultiIDs.getHardwareVersion(),
					this.skyBeaconMultiIDs.getFirmwareVersionMajor(),
					this.skyBeaconMultiIDs.getFirmwareVersionMinor())) {
				try {
					keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
							DefaultStaticValues.STRING_BYTES_CONVERT);
				} catch (UnsupportedEncodingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				if (skyBeaconMultiIDs.isLocked() == 1) {
					usedKeyThis = skyBeaconConfigMultiIDs.getLockedKey();
				} else {
					usedKeyThis = keyDefault;
				}

				result = configDA14580FinishConfig(usedKeyThis);
				if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					return result;
				}
			}

			// 配置结束标识读取
			if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT
					&& configState == true) {
				result = readConfigFinish();
			}
		}
		return result;
	}

	public SKYBeacon getSkyBeacon() {
		return skyBeacon;
	}

	public SKYBeaconMultiIDs getSkyBeaconMultiIDs() {
		return skyBeaconMultiIDs;
	}

	// Handles various events fired by the Service.
	// ACTION_GATT_CONNECTED: connected to a GATT server.
	// ACTION_GATT_DISCONNECTED: disconnected from a GATT server.
	// ACTION_GATT_SERVICES_DISCOVERED: discovered GATT services.
	// ACTION_DATA_AVAILABLE: received data from the device. This can be
	// a
	// result of read
	// or notification operations.
	private final BroadcastReceiver gattUpdateReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			if (!isConfiging) {
				return;
			}
			final String action = intent.getAction();
			if (BluetoothLeService.ACTION_GATT_CONNECTED.equals(action)) {
				updateConnectionState(true);
				Log.i(TAG, "ble connected");
			} else if (BluetoothLeService.ACTION_GATT_DISCONNECTED
					.equals(action)) {
				updateConnectionState(false);
				Log.i(TAG, "ble disconnected");
			} else if (BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED
					.equals(action)) {
				bluetoothLeService = serviceConnection.getBleService();
				isGattServicesDiscover = true;
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
				Log.i(TAG, "ble displayserice");
			} else if (BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED_FAILED
					.equals(action)) {
				isGattServicesDiscover = false;
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
				Log.i(TAG, "ble displayserice failed");
			} else if (BluetoothLeService.ACTION_DATA_AVAILABLE.equals(action)) {
				// String readDatas = intent
				// .getStringExtra(BluetoothLeService.EXTRA_DATA);
				Log.i(TAG, "ble display data");
			} else if (BluetoothLeService.ACTION_DATA_WRITE_SUCCESS
					.equals(action)) {
				Log.i(TAG, "ble write data success");
				updateWriteDatas(true);
			} else if (BluetoothLeService.ACTION_DATA_WRITE_FAILED
					.equals(action)) {
				Log.i(TAG, "ble write data failed");
				updateWriteDatas(false);
			} else if (BluetoothLeService.ACTION_DATA_READ_SUCCESS
					.equals(action)) {
				String readDatas = intent
						.getStringExtra(BluetoothLeService.EXTRA_DATA);
				updateReadDatas(readDatas);
				Log.i(TAG, "ble read data success");
			} else if (BluetoothLeService.ACTION_DATA_READ_FAILED
					.equals(action)) {
				updateReadDatas("failed");
				Log.i(TAG, "ble read data failed");
			}
		}
	};

	private void updateConnectionState(final boolean state) {
		isDeviceConnected = state;
		Thread thread = new Thread(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if (state) {
					// isDeviceConnected = true;
				} else {
					// isDeviceConnected = false;
					synchronized (lockedRead) {
						lockedRead.notifyAll();
					}
					Message msg = new Message();
					msg.what = SKYBeaconCommunication.HandleValue.DISCONNECT;
					handler.sendMessage(msg);
				}
			}
		});
		thread.start();
	}

	private void getGattAttributes() {
		if (bluetoothLeService != null) {
			List<BluetoothGattService> tmpGattService = bluetoothLeService
					.getBluetoothGatt().getServices();
			for (int i = 0; i < tmpGattService.size(); i++) {
				String tmp = tmpGattService.get(i).getUuid().toString();
				if (tmp.equals(GattAttributesDialog.SEEYCY_BEACON_COMMUNICATE_SERVICE)) {
					GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE = GattAttributesDialog.SEEYCY_BEACON_COMMUNICATE_SERVICE;
					GattAttributes.CHARACTERISTIC_1_READ_UUID = GattAttributesDialog.CHARACTERISTIC_1_READ_UUID;
					GattAttributes.CHARACTERISTIC_2_READ_UUID = GattAttributesDialog.CHARACTERISTIC_2_READ_UUID;
					GattAttributes.CHARACTERISTIC_3_READ_UUID = GattAttributesDialog.CHARACTERISTIC_3_READ_UUID;
					GattAttributes.CHARACTERISTIC_4_READ_UUID = GattAttributesDialog.CHARACTERISTIC_4_READ_UUID;
					GattAttributes.CHARACTERISTIC_5_READ_UUID = GattAttributesDialog.CHARACTERISTIC_5_READ_UUID;
					GattAttributes.CHARACTERISTIC_6_READ_UUID = GattAttributesDialog.CHARACTERISTIC_6_READ_UUID;
					GattAttributes.CHARACTERISTIC_7_READ_UUID = GattAttributesDialog.CHARACTERISTIC_7_READ_UUID;
					GattAttributes.CHARACTERISTIC_9_READ_UUID = GattAttributesDialog.CHARACTERISTIC_9_READ_UUID;
					GattAttributes.CHARACTERISTIC_10_READ_UUID = GattAttributesDialog.CHARACTERISTIC_10_READ_UUID;
					GattAttributes.CHARACTERISTIC_12_READ_UUID = GattAttributesDialog.CHARACTERISTIC_12_READ_UUID;
				}
			}
		}
	}

	private void updateReadDatas(final String datas) {
		Thread thread = new Thread(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if (datas != null && datas.equals("failed")) {
					// 读取失败
					readState = ConfigReadStates.READ_STATE_FAILED;
					return;
				} else {
					byte[] readDatasTmp = null;
					try {
						readDatasTmp = datas.getBytes("ISO-8859-1");
					} catch (UnsupportedEncodingException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					if (readDatasTmp == null) {
						readState = ConfigReadStates.READ_STATE_EMPTY;
						return;
					}
					if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_1_INIT) {
						if (readDatasTmp.length == 2 && readDatasTmp[0] == 0x54) {
							readState = ConfigReadStates.READ_STATE_RETURN_KEY_SUCCESS;
							randomKey = readDatasTmp[1];
							// isRandomKeyEnable = true;
						} else if (readDatasTmp.length == 2
								&& readDatasTmp[0] == (byte) 0x86
								&& readDatasTmp[1] == (byte) 0x27) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_SUCCESS;
						} else if (readDatasTmp.length == 2
								&& readDatasTmp[0] == (byte) 0x86
								&& readDatasTmp[1] == (byte) 0x28) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_PAUSE;
						} else if (readDatasTmp.length == 2
								&& readDatasTmp[0] == (byte) 0x86
								&& readDatasTmp[1] == (byte) 0x29) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_PASSCODE_ERROR;
						} else if (readDatasTmp.length == 2
								&& readDatasTmp[0] == (byte) 0x86
								&& readDatasTmp[1] == (byte) 0x2A) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_HEADER_ERROR;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_2_INIT) {
						if (readDatas.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_2_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_3_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_3_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_4_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_4_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_5_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_5_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_6_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_6_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_7_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_7_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_9_INIT) {
						if (readDatasTmp.length == 1) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_9_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_10_INIT) {
						if (readDatasTmp.length == 6) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_10_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_12_INIT) {
						if (readDatasTmp.length == 2) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp,
									readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_12_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else {
						readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
					}
				}
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
			}
		});
		thread.start();
	}

	private void updateWriteDatas(final boolean isSuccess) {
		Thread thread = new Thread(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if (isSuccess) {
					writeState = true;
				} else {
					writeState = false;
				}
				synchronized (lockedWrite) {
					lockedWrite.notifyAll();
				}
			}
		});
		thread.start();
	}

	/**
	 * 读取特定service的特定characteristic的数据
	 * 
	 * @param bluetoothLeService
	 * @param suuid
	 * @param cuuid
	 * @return
	 */
	private int readBle(UUID suuid, UUID cuuid) {
		int ret = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (bluetoothLeService != null) {
			BluetoothGattService tmpGattService = null;
			tmpGattService = bluetoothLeService.getBluetoothGatt().getService(
					suuid);
			if (tmpGattService != null) {
				BluetoothGattCharacteristic gattCharacteristics = null;
				gattCharacteristics = tmpGattService.getCharacteristic(cuuid);
				if (gattCharacteristics != null) {
					boolean retTmp = false;
					retTmp = bluetoothLeService
							.readCharacteristic(gattCharacteristics);
					if (!retTmp) {
						ret = ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_FAILED;
					}
				} else {
					ret = ConfigInfoCode.ErrorCode.CODE_CHARACTERISTIC_NOT_FOUND;
				}
			} else {
				ret = ConfigInfoCode.ErrorCode.CODE_GATT_SERVICE_NOT_FOUND;
			}
		}
		return ret;
	}

	/**
	 * 向特定的service的特定的characteristic写入数据
	 * 
	 * @param bluetoothLeService
	 * @param suuid
	 * @param cuuid
	 * @param writeDatas
	 * @return
	 */
	private int writeBle(UUID suuid, UUID cuuid, byte[] writeDatas) {
		int ret = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (bluetoothLeService != null) {
			BluetoothGattService tmpGattService = null;
			tmpGattService = bluetoothLeService.getBluetoothGatt().getService(
					suuid);
			if (tmpGattService != null) {
				BluetoothGattCharacteristic gattCharacteristics = null;
				gattCharacteristics = tmpGattService.getCharacteristic(cuuid);
				if (gattCharacteristics != null) {
					bluetoothLeService.setCharacteristicNotification(
							gattCharacteristics, true);
					gattCharacteristics.setValue(writeDatas);
					boolean retTmp = bluetoothLeService
							.writeCharacteristic(gattCharacteristics);
					if (!retTmp) {
						ret = ConfigInfoCode.ErrorCode.CODE_WRITE_CHARACTERISTIC_FAILED;
					}
				} else {
					ret = ConfigInfoCode.ErrorCode.CODE_CHARACTERISTIC_NOT_FOUND;
				}
			} else {
				ret = ConfigInfoCode.ErrorCode.CODE_GATT_SERVICE_NOT_FOUND;
			}
		}
		return ret;
	}

	/**
	 * 设备连接
	 * 
	 * @param skyBeacon
	 * @return
	 */
	private int connectDevice(String deviceAddress) {
		if(!DatasCheck.isMac(deviceAddress)) {
			return ConfigInfoCode.ErrorCode.CODE_NOT_SEEKCY_BEACON;
		}
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		isDeviceConnected = false;
		skyBeacon = new SKYBeacon(deviceAddress);
		skyBeaconMultiIDs = new SKYBeaconMultiIDs(deviceAddress);
		serviceConnection = new BleServiceConnection(bluetoothLeService,
				deviceAddress);
		context.registerReceiver(gattUpdateReceiver,
				BluetoothLeService.makeGattUpdateIntentFilter());
		isRegistered = true;
		Intent gattServiceIntent = new Intent(context, BluetoothLeService.class);

		synchronized (lockedRead) {
			try {
				isBind = context.bindService(gattServiceIntent,
						serviceConnection, Context.BIND_AUTO_CREATE);
				lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
		if (!isDeviceConnected) {
			result = ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED;
		} else if (isDeviceConnected && !isGattServicesDiscover) {
			result = ConfigInfoCode.ErrorCode.CODE_DISCOVER_GATT_SERVICE_FAILED;
		}
		return result;
	}

	@SuppressWarnings("unused")
	private void updateStates(int state, String msgStr) {
		if (isConfiging) {
			Message msg = new Message();
			msg.what = state;
			msg.obj = msgStr;
			handlerMessageListener.handlerMessage(msg);
		}
	}

	private int getReadStateStart(String gattAttributes) {
		int ret = ConfigReadStates.READ_STATE_INIT;
		if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_1_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_1_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_2_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_2_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_3_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_3_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_4_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_4_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_5_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_5_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_6_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_6_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_7_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_7_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_9_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_9_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_10_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_10_INIT;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_12_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_12_INIT;
		}
		return ret;
	}

	private int getReadStateEnd(String gattAttributes) {
		int ret = ConfigReadStates.READ_STATE_INIT;
		if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_1_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_1_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_2_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_2_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_3_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_3_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_4_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_4_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_5_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_5_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_6_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_6_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_7_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_7_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_9_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_9_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_10_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_10_END;
		} else if (gattAttributes
				.equals(GattAttributes.CHARACTERISTIC_12_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_12_END;
		}
		return ret;
	}

	/**
	 * 读取特征值的信息，包括解析数据
	 * 
	 * @param suuid
	 * @param cuuid
	 * @return
	 */
	private int readCharacteristic(UUID suuid, UUID cuuid) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		readState = getReadStateStart(cuuid.toString());
		final int readStateEnd = getReadStateEnd(cuuid.toString());
		// 读取数据
		result = readBle(suuid, cuuid);
		synchronized (lockedRead) {
			try {
				lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT
				&& readState == readStateEnd) {
			if (isMultiIDsBeacon) {
				result = SKYBeaconCommunicateProtocol.solveCharacteristic(
						readDatas, readDatasLength, readStateEnd,
						skyBeaconMultiIDs);
			} else {
				result = SKYBeaconCommunicateProtocol.solveCharacteristic(
						readDatas, readDatasLength, readStateEnd, skyBeacon);
			}
		}
		return result;
	}

	/**
	 * 读取特征值的信息，包括解析数据
	 * 
	 * @param suuid
	 * @param cuuid
	 * @return
	 */
	private int writeCharacteristic(UUID suuid, UUID cuuid, byte[] writeDatas) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		writeState = false;
		// 读取数据
		result = writeBle(suuid, cuuid, writeDatas);
		synchronized (lockedWrite) {
			try {
				lockedWrite.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
			if (!writeState) {
				result = ConfigInfoCode.ErrorCode.CODE_WRITE_CHARACTERISTIC_FAILED;
			}
		}
		return result;
	}

	/**
	 * 读取结束配置标识符结果
	 * 
	 * @return
	 */
	private int readConfigFinish() {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		readState = getReadStateStart(cuuid.toString());
		// 读取数据
		result = readBle(suuid, cuuid);
		synchronized (lockedRead) {
			try {
				lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
			if (readState == ConfigReadStates.READ_STATE_RETURN_CONFIG_SUCCESS) {
				result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
			} else if (readState == ConfigReadStates.READ_STATE_RETURN_CONFIG_PAUSE) {
				result = ConfigInfoCode.ErrorCode.CODE_WRITE_PROTOCOL_PAUSE_FALSE;
			} else if (readState == ConfigReadStates.READ_STATE_RETURN_CONFIG_HEADER_ERROR) {
				result = ConfigInfoCode.ErrorCode.CODE_WRITE_PROTOCOL_HEADER_FALSE;
			} else if (readState == ConfigReadStates.READ_STATE_RETURN_CONFIG_PASSCODE_ERROR) {
				result = ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE;
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
			}
		}
		return result;
	}

	/**
	 * 多id beacon的通用配置项目， interval和格子使用情况
	 * 
	 * @param skyBeaconMultiIDs
	 * @return
	 */
	private int configMultiIDsGeneral(
			SKYBeaconConfigMultiIDs skyBeaconConfigMultiIDs, String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		if (SKYBeaconVersionUtil.isDA14580(
				skyBeaconMultiIDs.getHardwareVersion(),
				skyBeaconMultiIDs.getFirmwareVersionMajor(),
				skyBeaconMultiIDs.getFirmwareVersionMinor())) {
			byte[] frame = SKYBeaconCommunicateProtocol
					.getDialogMultiIDsGeneralProtocol(skyBeaconConfigMultiIDs,
							key);
			byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
					frame, randomKey);
			result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		} else {
			byte[] frame = SKYBeaconCommunicateProtocol
					.getMultiIDsGeneralProtocol(skyBeaconMultiIDs, skyBeaconConfigMultiIDs, key);
			byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
					frame, randomKey);
			result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		}
		return result;
	}

	/**
	 * 配置时间
	 * 
	 * @param key
	 * @return
	 */
	private int configTime(String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame = SKYBeaconCommunicateProtocol.getConfigTimeProtocol(key);
		byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame, randomKey);
		result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		return result;
	}

	/**
	 * 配置多id的广播
	 * 
	 * @param skyBeacon
	 * @param id
	 * @return
	 */
	private int configMultiIDsBroadcast(SKYBeaconConfig skyBeaconConfig,
			String key, int id) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;

		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame1 = SKYBeaconCommunicateProtocol
				.getMultiIDsBroadcast1Protocol(skyBeaconConfig, id);
		byte[] frame2 = SKYBeaconCommunicateProtocol
				.getMultiIDsBroadcast2Protocol(skyBeaconConfig, id, key);
		byte[] frame1Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame1, randomKey);
		byte[] frame2Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame2, randomKey);
		result = writeCharacteristic(suuid, cuuid, frame1Encrypt);
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
			result = writeCharacteristic(suuid, cuuid, frame2Encrypt);
		}
		return result;
	}

	/**
	 * 配置单id的广播
	 * 
	 * @param skyBeacon
	 * @param id
	 * @return
	 */
	private int configSingleIDBroadcast(SKYBeaconConfig skyBeaconConfig,
			String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;

		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		if (SKYBeaconVersionUtil.isDA14580(skyBeacon.getHardwareVersion(),
				skyBeacon.getFirmwareVersionMajor(),
				skyBeacon.getFirmwareVersionMinor())) {
			byte[] frame1 = SKYBeaconCommunicateProtocol
					.getSingleIDBroadcast1Protocol(skyBeaconConfig);
			byte[] frame2 = SKYBeaconCommunicateProtocol
					.getDialogSingleIDBroadcast2Protocol(skyBeaconConfig, key);
			byte[] frame1Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
					frame1, randomKey);
			byte[] frame2Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
					frame2, randomKey);
			result = writeCharacteristic(suuid, cuuid, frame1Encrypt);
			if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				result = writeCharacteristic(suuid, cuuid, frame2Encrypt);
			}
		} else {
			byte[] frame1 = SKYBeaconCommunicateProtocol
					.getSingleIDBroadcast1Protocol(skyBeaconConfig);
			byte[] frame2 = SKYBeaconCommunicateProtocol
					.getSingleIDBroadcast2Protocol(skyBeaconConfig, key);
			byte[] frame1Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
					frame1, randomKey);
			byte[] frame2Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
					frame2, randomKey);
			result = writeCharacteristic(suuid, cuuid, frame1Encrypt);
			if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				result = writeCharacteristic(suuid, cuuid, frame2Encrypt);
			}
		}
		return result;
	}

	/**
	 * 配置单id的led状态
	 * 
	 * @param skyBeacon
	 * @param key
	 * @return
	 */
	private int configLedState(int ledState, String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (ledState > 2 || ledState < 0) {
			result = ConfigInfoCode.ErrorCode.CODE_LED_STATE_CONFIG_FAILED_VALUE_FALSE;
		} else {
			UUID suuid = UUID
					.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
			UUID cuuid = UUID
					.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
			byte[] frame = SKYBeaconCommunicateProtocol.getLedStateProtocol(
					ledState, key);
			byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
					frame, randomKey);
			result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		}
		return result;
	}

	/**
	 * 配置密钥
	 * 
	 * @param keyOld
	 * @param keyNew
	 * @return
	 */
	private int configKey(String keyOld, String keyNew) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame = SKYBeaconCommunicateProtocol.getKeyProtocol(keyOld,
				keyNew);
		byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame, randomKey);
		result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		return result;
	}

	/**
	 * 恢复出产设置
	 * 
	 * @param keyRaw
	 * @param keyDefault
	 * @return
	 */
	private int configReset(String keyRaw, String keyDefault) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame = SKYBeaconCommunicateProtocol.getResetProtocol(keyRaw,
				keyDefault);
		byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame, randomKey);
		result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		return result;
	}

	/**
	 * 配置device name
	 * 
	 * @param deviceName
	 * @param key
	 * @return
	 */
	private int configDeviceName(String deviceName, String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (deviceName == null) {
			return result;
		}
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame1 = SKYBeaconCommunicateProtocol
				.getDeviceName1Protocol(deviceName);
		byte[] frame2 = SKYBeaconCommunicateProtocol.getDeviceName2Protocol(
				deviceName, key);
		byte[] frame1Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame1, randomKey);
		byte[] frame2Encrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame2, randomKey);
		result = writeCharacteristic(suuid, cuuid, frame1Encrypt);
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
			result = writeCharacteristic(suuid, cuuid, frame2Encrypt);
		}
		return result;
	}

	/**
	 * 配置温度的更新时间，单位秒
	 * 
	 * @param updateSecond
	 * @param key
	 * @return
	 */
	private int configTemperatureUpdate(int updateSecond, String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame = SKYBeaconCommunicateProtocol
				.getTemperatureUpdateProtocol(updateSecond, key);
		byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame, randomKey);
		result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		return result;
	}

	/**
	 * 配置光感
	 * 
	 * @param key
	 * @param state
	 * @param threshold
	 * @param updateSecond
	 * @param darkIntervalMillisecond
	 * @return
	 */
	private int configLightSensor(String key, int state, int threshold,
			int updateSecond, int darkIntervalMillisecond) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame = SKYBeaconCommunicateProtocol.getLightSensorProtocol(key,
				state, threshold, updateSecond, darkIntervalMillisecond);
		byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame, randomKey);
		result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		return result;
	}

	/**
	 * da14580配对
	 * 
	 * @param key
	 * @return
	 */
	private int configDA14580Pairing(String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame = SKYBeaconCommunicateProtocol
				.getDA14580PairingProtocol(key);
		byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame, randomKey);
		result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		return result;
	}

	/**
	 * da14580结束配置
	 * 
	 * @param key
	 * @return
	 */
	private int configDA14580FinishConfig(String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		UUID suuid = UUID
				.fromString(GattAttributes.SEEYCY_BEACON_COMMUNICATE_SERVICE);
		UUID cuuid = UUID.fromString(GattAttributes.CHARACTERISTIC_1_READ_UUID);
		byte[] frame = SKYBeaconCommunicateProtocol
				.getDA14580FinishConfigProtocol(key);
		byte[] frameEncrypt = SKYBeaconEncrypt.encryptCharacteristicDatas(
				frame, randomKey);
		result = writeCharacteristic(suuid, cuuid, frameEncrypt);
		return result;
	}

	// private int configBeaconV020(SKYBeaconConfigMultiIDs
	// skyBeaconConfigMultiIDs) {
	// int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
	// boolean configState = false;
	// if (skyBeaconConfigMultiIDs == null) {
	// result = ConfigInfoCode.ErrorCode.CODE_INPUT_NULL;
	// return result;
	// }
	// if (!isMultiIDsBeacon) {
	// result = ConfigInfoCode.ErrorCode.CODE_USE_SINGLE_ID_BEACON_REPLACE;
	// return result;
	// }
	// if (this.skyBeaconMultiIDs.isLocked() == 1
	// && skyBeaconConfigMultiIDs
	// .getLockedKey()
	// .equals(DefaultStaticValues.DEFAULT_SKY_BEACON_LOCKED_KEY_FALSE)) {
	// result = ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE;
	// return result;
	// }
	//
	// if (skyBeaconConfigMultiIDs.isReset()) {
	// String keyDefault = "";
	// String keyRaw = "";
	// try {
	// keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
	// DefaultStaticValues.STRING_BYTES_CONVERT);
	// keyRaw = skyBeaconConfigMultiIDs.getLockedKey();
	// } catch (Exception e) {
	// // TODO: handle exception
	// }
	// // 恢复出产设置
	// result = configReset(keyRaw, keyDefault);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// } else {
	// // 确定配置过程中的密钥
	// String keyDefault = "";
	// String usedKeyThis = "";
	// try {
	// keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
	// DefaultStaticValues.STRING_BYTES_CONVERT);
	// } catch (UnsupportedEncodingException e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// }
	// if (this.skyBeaconMultiIDs.isLocked() == 1) {
	// usedKeyThis = skyBeaconConfigMultiIDs.getLockedKey();
	// } else {
	// usedKeyThis = keyDefault;
	// }
	// // 防篡改模式是否需要配置
	// if (skyBeaconConfigMultiIDs.isLocked() !=
	// DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE) {
	// if (this.skyBeaconMultiIDs.isLocked() == 1
	// && skyBeaconConfigMultiIDs.isLocked() == 1) {
	// // result =
	// // ConfigInfoCode.ErrorCode.CODE_BEACON_ALREADY_LOCKED;
	// // return result;
	//
	// result = configKey(skyBeaconConfigMultiIDs.getLockedKey(),
	// skyBeaconConfigMultiIDs.getLockedKey());
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// if (this.skyBeaconMultiIDs.isLocked() != 1
	// && skyBeaconConfigMultiIDs.isLocked() == 1) {
	// result = configKey(keyDefault,
	// skyBeaconConfigMultiIDs.getLockedKey());
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// if (this.skyBeaconMultiIDs.isLocked() == 1
	// && skyBeaconConfigMultiIDs.isLocked() == 0) {
	// result = configKey(skyBeaconConfigMultiIDs.getLockedKey(),
	// keyDefault);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// }
	// // device name是否需要配置
	// if (!skyBeaconConfigMultiIDs.getDeviceName().equals(
	// DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME)) {
	// result = configDeviceName(
	// skyBeaconConfigMultiIDs.getDeviceName(), usedKeyThis);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// // general参数是否需要配置
	// if (!(skyBeaconConfigMultiIDs.getCellsSituation() == null &&
	// skyBeaconConfigMultiIDs
	// .getIntervalMillisecond() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE)) {
	// result = configMultiIDsGeneral(skyBeaconConfigMultiIDs,
	// usedKeyThis);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// SKYBeaconConfig tmpSKYBeaconConfig = new SKYBeaconConfig();
	// result = configMultiIDsBroadcast(tmpSKYBeaconConfig,
	// usedKeyThis, 0);
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// // 广播信息是否需要配置（子函数中进行判断）
	// SKYBeaconConfig[] skyBeaconList = skyBeaconConfigMultiIDs
	// .getConfigBeacons();
	// for (int i = 0; i < 3; i++) {
	// if (skyBeaconList[i].getProximityUUID().equals(
	// DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)
	// && skyBeaconList[i].getMajor() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE
	// && skyBeaconList[i].getMinor() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE
	// && SKYBeaconPower.getPower(skyBeaconList[i]
	// .getTxpower()) == DefaultStaticValues.DEFAULT_SKY_BEACON_TXPOWER_FALSE
	// && skyBeaconList[i].getMeasuredPower() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE
	// && skyBeaconList[i].isEncrypted() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE) {
	// } else {
	// result = configMultiIDsBroadcast(skyBeaconList[i],
	// usedKeyThis, i);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// break;
	// }
	// }
	// }
	// // 配置时间
	// if (skyBeaconConfigMultiIDs.isSetTime()) {
	// result = configTime(usedKeyThis);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// // 配置结束标识读取
	// if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT
	// && configState == true) {
	// result = readConfigFinish();
	// }
	// }
	// return result;
	// }
	//
	// private int configBeaconV021(SKYBeaconConfigMultiIDs
	// skyBeaconConfigMultiIDs) {
	// int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
	// boolean configState = false;
	// if (skyBeaconConfigMultiIDs == null) {
	// result = ConfigInfoCode.ErrorCode.CODE_INPUT_NULL;
	// return result;
	// }
	// if (!isMultiIDsBeacon) {
	// result = ConfigInfoCode.ErrorCode.CODE_USE_SINGLE_ID_BEACON_REPLACE;
	// return result;
	// }
	// if (this.skyBeaconMultiIDs.isLocked() == 1
	// && skyBeaconConfigMultiIDs
	// .getLockedKey()
	// .equals(DefaultStaticValues.DEFAULT_SKY_BEACON_LOCKED_KEY_FALSE)) {
	// result = ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE;
	// return result;
	// }
	//
	// if (skyBeaconConfigMultiIDs.isReset()) {
	// String keyDefault = "";
	// String keyRaw = "";
	// try {
	// keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
	// DefaultStaticValues.STRING_BYTES_CONVERT);
	// keyRaw = skyBeaconConfigMultiIDs.getLockedKey();
	// } catch (Exception e) {
	// // TODO: handle exception
	// }
	// // 恢复出产设置
	// result = configReset(keyRaw, keyDefault);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// } else {
	// // 确定配置过程中的密钥
	// String keyDefault = "";
	// String usedKeyThis = "";
	// try {
	// keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES,
	// DefaultStaticValues.STRING_BYTES_CONVERT);
	// } catch (UnsupportedEncodingException e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// }
	// if (this.skyBeaconMultiIDs.isLocked() == 1) {
	// usedKeyThis = skyBeaconConfigMultiIDs.getLockedKey();
	// } else {
	// usedKeyThis = keyDefault;
	// }
	// // 防篡改模式是否需要配置
	// if (skyBeaconConfigMultiIDs.isLocked() !=
	// DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE) {
	// if (this.skyBeaconMultiIDs.isLocked() == 1
	// && skyBeaconConfigMultiIDs.isLocked() == 1) {
	// result = configKey(skyBeaconConfigMultiIDs.getLockedKey(),
	// skyBeaconConfigMultiIDs.getLockedKey());
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// if (this.skyBeaconMultiIDs.isLocked() != 1
	// && skyBeaconConfigMultiIDs.isLocked() == 1) {
	// result = configKey(keyDefault,
	// skyBeaconConfigMultiIDs.getLockedKey());
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// if (this.skyBeaconMultiIDs.isLocked() == 1
	// && skyBeaconConfigMultiIDs.isLocked() == 0) {
	// result = configKey(skyBeaconConfigMultiIDs.getLockedKey(),
	// keyDefault);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// }
	// // device name是否需要配置
	// if (!skyBeaconConfigMultiIDs.getDeviceName().equals(
	// DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME)) {
	// result = configDeviceName(
	// skyBeaconConfigMultiIDs.getDeviceName(), usedKeyThis);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// // general参数是否需要配置
	// if (!(skyBeaconConfigMultiIDs.getCellsSituation() == null &&
	// skyBeaconConfigMultiIDs
	// .getIntervalMillisecond() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE)) {
	// result = configMultiIDsGeneral(skyBeaconConfigMultiIDs,
	// usedKeyThis);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// // 广播信息是否需要配置（子函数中进行判断）
	// SKYBeaconConfig[] skyBeaconList = skyBeaconConfigMultiIDs
	// .getConfigBeacons();
	// for (int i = 0; i < 3; i++) {
	// if (skyBeaconList[i].getProximityUUID().equals(
	// DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)
	// && skyBeaconList[i].getMajor() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE
	// && skyBeaconList[i].getMinor() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE
	// && SKYBeaconPower.getPower(skyBeaconList[i]
	// .getTxpower()) == DefaultStaticValues.DEFAULT_SKY_BEACON_TXPOWER_FALSE
	// && skyBeaconList[i].getMeasuredPower() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE
	// && skyBeaconList[i].isEncrypted() ==
	// DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE) {
	// } else {
	// result = configMultiIDsBroadcast(skyBeaconList[i],
	// usedKeyThis, i);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// break;
	// }
	// }
	// }
	// // 配置时间
	// if (skyBeaconConfigMultiIDs.isSetTime()) {
	// result = configTime(usedKeyThis);
	// configState = true;
	// if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
	// return result;
	// }
	// }
	// }
	// // 配置结束标识读取
	// if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT
	// && configState == true) {
	// result = readConfigFinish();
	// }
	// return result;
	// }
}
