package com.skybeacon.sdk.config;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import com.skybeacon.sdk.entity.VersionAdaption;
import com.skybeacon.sdk.locate.SKYBeacon;
import com.skybeacon.sdk.locate.SKYBeaconMultiIDs;
import com.skybeacon.sdk.utils.DataConvertUtils;
import com.skybeacon.sdk.utils.DefaultStaticValues;

/**
 * 通信协议解析，组包
 * 
 * @author frank
 * 
 */
public class SKYBeaconCommunicateProtocol {
	/**
	 * 解析读取的数据 单id的iBeacon
	 * 
	 * @param datas
	 * @param length
	 * @param readStateEnd
	 * @param skyBeacon
	 * @return
	 */
	public static int solveCharacteristic(byte[] datas, int length,
			int readStateEnd, SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		switch (readStateEnd) {
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_1_END:
			// solveCharacteristic1(datas, length, skyBeaconConfig);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_2_END:
			result = solveCharacteristic2(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_3_END:
			result = solveCharacteristic3(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_4_END:
			// solveCharacteristic4(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_5_END:
			// solveCharacteristic5(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_6_END:
			// solveCharacteristic6(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_7_END:
			result = solveCharacteristic7(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_9_END:
			result = solveCharacteristic9(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_10_END:
			result = solveCharacteristic10(datas, length, skyBeacon);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_12_END:
			result = solveCharacteristic12(datas, length, skyBeacon);
			break;
		default:
			result = ConfigInfoCode.ErrorCode.CODE_FALSE_CHARACTERISTIC;
			break;
		}
		return result;
	}

	/**
	 * 解析读取的数据，多id的iBeacon
	 * 
	 * @param datas
	 * @param length
	 * @param readStateEnd
	 * @param skyBeaconMultiIDs
	 * @return
	 */
	public static int solveCharacteristic(byte[] datas, int length,
			int readStateEnd, SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		switch (readStateEnd) {
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_1_END:
			// result = solveCharacteristic1(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_2_END:
			result = solveCharacteristic2(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_3_END:
			result = solveCharacteristic3(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_4_END:
			result = solveCharacteristic4(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_5_END:
			result = solveCharacteristic5(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_6_END:
			result = solveCharacteristic6(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_7_END:
			result = solveCharacteristic7(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_9_END:
			result = solveCharacteristic9(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_10_END:
			result = solveCharacteristic10(datas, length, skyBeaconMultiIDs);
			break;
		case SKYBeaconCommunicateProcess.ConfigReadStates.READ_STATE_CHARACTERISTIC_12_END:
			result = solveCharacteristic12(datas, length, skyBeaconMultiIDs);
			break;
		default:
			result = ConfigInfoCode.ErrorCode.CODE_FALSE_CHARACTERISTIC;
			break;
		}
		return result;
	}

	/**
	 * 获取时间配置的协议
	 * 
	 * @param key
	 * @return
	 */
	public static byte[] getConfigTimeProtocol(String key) {
		byte[] result = new byte[17];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x18;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.arraycopy(DataConvertUtils.getCurrentTimeBytes(), 0, result, 10,
				7);
		return result;
	}

	/**
	 * 多id beacon通用配置协议
	 * 
	 * @param skyBeaconMultiIDs
	 * @param key
	 * @return
	 */
	public static byte[] getMultiIDsGeneralProtocol(
			SKYBeaconMultiIDs skyBeaconMultiIDs,
			SKYBeaconConfigMultiIDs skyBeaconConfigMultiIDs, String key) {
		byte[] result = new byte[16];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x53;
		result[3] = 0x22;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (skyBeaconConfigMultiIDs.getIntervalMillisecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE) {
			if (SKYBeaconVersionUtil.isIntervalFiveSecondProtocol(
					skyBeaconMultiIDs.getHardwareVersion(),
					skyBeaconMultiIDs.getFirmwareVersionMajor(),
					skyBeaconMultiIDs.getFirmwareVersionMinor())) {
				if (skyBeaconConfigMultiIDs.getIntervalMillisecond() > 5000) {
					result[10] = (byte) ((((5000 - 1000) / 50) & 0xFF) + 100);
				} else if (skyBeaconConfigMultiIDs.getIntervalMillisecond() > 1000) {
					result[10] = (byte) ((((skyBeaconConfigMultiIDs
							.getIntervalMillisecond() - 1000) / 50) & 0xFF) + 100);
				} else {
					result[10] = (byte) ((skyBeaconConfigMultiIDs
							.getIntervalMillisecond() / 10) & 0xFF);
				}
			} else {
				if (skyBeaconConfigMultiIDs.getIntervalMillisecond() >= 2560) {
					result[10] = (byte) 0xFF;
				} else {
					result[10] = (byte) ((skyBeaconConfigMultiIDs
							.getIntervalMillisecond() / 10) & 0xFF);
				}
			}
		} else {
			result[10] = 0;
		}
		if (skyBeaconConfigMultiIDs.getCellsSituation() != null) {
			// 格子使用情况
			SKYBeaconBroadcastID[] tmpSituation = skyBeaconConfigMultiIDs
					.getCellsSituation().getUsedCellsSituation();
			result[11] = (byte) ((byte) ((skyBeaconConfigMultiIDs
					.getCellsSituation().getUsedCellsNum() & 0x000F) << 4));
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[0]) != -1) {
				result[11] = (byte) (result[11] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[0]) - 1) & 0x0F));
			} else {
				result[11] = (byte) (result[11] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[0])) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[1]) != -1) {
				result[12] = (byte) (result[12] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[1]) - 1) & 0x0F) << 4));
			} else {
				result[12] = (byte) (result[12] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[1])) & 0x0F) << 4));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[2]) != -1) {
				result[12] = (byte) (result[12] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[2]) - 1) & 0x0F));
			} else {
				result[12] = (byte) (result[12] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[2])) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[3]) != -1) {
				result[13] = (byte) (result[13] + (byte) ((((SKYBeaconBroadcastID
						.getInt(tmpSituation[3]) - 1)) & 0x0F) << 4));
			} else {
				result[13] = (byte) (result[13] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[3])) & 0x0F) << 4));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[4]) != -1) {
				result[13] = (byte) (result[13] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[4]) - 1) & 0x0F));
			} else {
				result[13] = (byte) (result[13] + (byte) (SKYBeaconBroadcastID
						.getInt(tmpSituation[4]) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[5]) != -1) {
				result[14] = (byte) (result[14] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[5]) - 1) & 0x0F) << 4));
			} else {
				result[14] = (byte) (result[14] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[5])) & 0x0F << 4));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[6]) != -1) {
				result[14] = (byte) (result[14] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[6]) - 1) & 0x0F));
			} else {
				result[14] = (byte) (result[14] + (byte) (SKYBeaconBroadcastID
						.getInt(tmpSituation[6]) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[7]) != -1) {
				result[15] = (byte) (result[15] + (byte) ((((SKYBeaconBroadcastID
						.getInt(tmpSituation[7]) - 1)) & 0x0F) << 4));
			} else {
				result[15] = (byte) (result[15]
						+ (byte) ((SKYBeaconBroadcastID.getInt(tmpSituation[7])) & 0x0F) << 4);
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[8]) != -1) {
				result[15] = (byte) (result[15] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[8]) - 1) & 0x0F));
			} else {
				result[15] = (byte) (result[15] + (byte) (SKYBeaconBroadcastID
						.getInt(tmpSituation[8]) & 0x0F));
			}
		} else {
			result[11] = 0x0F;
			result[12] = (byte) 0xFF;
			result[13] = (byte) 0xFF;
			result[14] = (byte) 0xFF;
			result[15] = (byte) 0xFF;
		}

		return result;
	}

	/**
	 * 多id beacon通用配置协议
	 * 
	 * @param skyBeaconMultiIDs
	 * @param key
	 * @return
	 */
	public static byte[] getDialogMultiIDsGeneralProtocol(
			SKYBeaconConfigMultiIDs skyBeaconConfigMultiIDs, String key) {
		byte[] result = new byte[16];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x53;
		result[3] = 0x22;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (skyBeaconConfigMultiIDs.getIntervalMillisecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE) {
			if (skyBeaconConfigMultiIDs.getIntervalMillisecond() > 5000) {
				result[10] = (byte) ((((5000 - 1000) / 50) & 0xFF) + 100);
			} else if (skyBeaconConfigMultiIDs.getIntervalMillisecond() > 1000) {
				result[10] = (byte) ((((skyBeaconConfigMultiIDs
						.getIntervalMillisecond() - 1000) / 50) & 0xFF) + 100);
			} else {
				result[10] = (byte) ((skyBeaconConfigMultiIDs
						.getIntervalMillisecond() / 10) & 0xFF);
			}
		} else {
			result[10] = 0;
		}
		if (skyBeaconConfigMultiIDs.getCellsSituation() != null) {
			// 格子使用情况
			SKYBeaconBroadcastID[] tmpSituation = skyBeaconConfigMultiIDs
					.getCellsSituation().getUsedCellsSituation();
			result[11] = (byte) ((byte) ((skyBeaconConfigMultiIDs
					.getCellsSituation().getUsedCellsNum() & 0x000F) << 4));
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[0]) != -1) {
				result[11] = (byte) (result[11] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[0]) - 1) & 0x0F));
			} else {
				result[11] = (byte) (result[11] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[0])) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[1]) != -1) {
				result[12] = (byte) (result[12] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[1]) - 1) & 0x0F) << 4));
			} else {
				result[12] = (byte) (result[12] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[1])) & 0x0F) << 4));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[2]) != -1) {
				result[12] = (byte) (result[12] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[2]) - 1) & 0x0F));
			} else {
				result[12] = (byte) (result[12] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[2])) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[3]) != -1) {
				result[13] = (byte) (result[13] + (byte) ((((SKYBeaconBroadcastID
						.getInt(tmpSituation[3]) - 1)) & 0x0F) << 4));
			} else {
				result[13] = (byte) (result[13] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[3])) & 0x0F) << 4));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[4]) != -1) {
				result[13] = (byte) (result[13] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[4]) - 1) & 0x0F));
			} else {
				result[13] = (byte) (result[13] + (byte) (SKYBeaconBroadcastID
						.getInt(tmpSituation[4]) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[5]) != -1) {
				result[14] = (byte) (result[14] + (byte) (((SKYBeaconBroadcastID
						.getInt(tmpSituation[5]) - 1) & 0x0F) << 4));
			} else {
				result[14] = (byte) (result[14] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[5])) & 0x0F << 4));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[6]) != -1) {
				result[14] = (byte) (result[14] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[6]) - 1) & 0x0F));
			} else {
				result[14] = (byte) (result[14] + (byte) (SKYBeaconBroadcastID
						.getInt(tmpSituation[6]) & 0x0F));
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[7]) != -1) {
				result[15] = (byte) (result[15] + (byte) ((((SKYBeaconBroadcastID
						.getInt(tmpSituation[7]) - 1)) & 0x0F) << 4));
			} else {
				result[15] = (byte) (result[15]
						+ (byte) ((SKYBeaconBroadcastID.getInt(tmpSituation[7])) & 0x0F) << 4);
			}
			if ((byte) SKYBeaconBroadcastID.getInt(tmpSituation[8]) != -1) {
				result[15] = (byte) (result[15] + (byte) ((SKYBeaconBroadcastID
						.getInt(tmpSituation[8]) - 1) & 0x0F));
			} else {
				result[15] = (byte) (result[15] + (byte) (SKYBeaconBroadcastID
						.getInt(tmpSituation[8]) & 0x0F));
			}
		} else {
			result[11] = 0x0F;
			result[12] = (byte) 0xFF;
			result[13] = (byte) 0xFF;
			result[14] = (byte) 0xFF;
			result[15] = (byte) 0xFF;
		}

		return result;
	}

	/**
	 * 广播id协议1获取
	 * 
	 * @param skyBeaconConfig
	 * @param id
	 * @return
	 */
	public static byte[] getMultiIDsBroadcast1Protocol(
			SKYBeaconConfig skyBeaconConfig, int id) {
		byte[] result = new byte[20];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x54;
		result[3] = (byte) id;
		if (!skyBeaconConfig.getProximityUUID().equals(
				DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)) {
			System.arraycopy(DataConvertUtils
					.proximityUuidToBytes(skyBeaconConfig.getProximityUUID()),
					0, result, 4, 16);
		}
		return result;
	}

	/**
	 * 广播id协议2获取
	 * 
	 * @param skyBeaconConfig
	 * @param id
	 * @param key
	 * @return
	 */
	public static byte[] getMultiIDsBroadcast2Protocol(
			SKYBeaconConfig skyBeaconConfig, int id, String key) {
		byte[] result = new byte[18];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x55;
		result[17] = 0;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 3, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 3, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		result[9] = (byte) id;
		if (skyBeaconConfig.getMajor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE) {
			System.arraycopy(DataConvertUtils.intToByteArrayTwo(skyBeaconConfig
					.getMajor()), 0, result, 10, 2);
		} else {
			result[17] = (byte) (result[17] | 0x02);
		}
		if (skyBeaconConfig.getMinor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE) {
			System.arraycopy(DataConvertUtils.intToByteArrayTwo(skyBeaconConfig
					.getMinor()), 0, result, 12, 2);
		} else {
			result[17] = (byte) (result[17] | 0x01);
		}
		if (skyBeaconConfig.getTxpower() != SKYBeaconPower
				.getPower(DefaultStaticValues.DEFAULT_SKY_BEACON_TXPOWER_FALSE)) {
			result[14] = (byte) SKYBeaconPower.getPower(skyBeaconConfig
					.getTxpower());
		} else {
			result[14] = 0x7F;
		}
		if (skyBeaconConfig.getMeasuredPower() != DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE) {
			result[15] = (byte) skyBeaconConfig.getMeasuredPower();
		} else {
			result[15] = 0x7F;
		}
		if (skyBeaconConfig.isEncrypted() != DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE) {
			result[16] = (byte) skyBeaconConfig.isEncrypted();
		} else {
			result[16] = (byte) 0xFF;
		}
		if (skyBeaconConfig.getProximityUUID().equals(
				DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)) {
			result[17] = (byte) (result[17] | 0x04);
		}
		return result;
	}

	/**
	 * 单id 广播配置协议1
	 * 
	 * @param skyBeaconConfig
	 * @return
	 */
	public static byte[] getSingleIDBroadcast1Protocol(
			SKYBeaconConfig skyBeaconConfig) {
		byte[] result = new byte[20];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x13;
		if (!skyBeaconConfig.getProximityUUID().equals(
				DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)) {
			System.arraycopy(DataConvertUtils
					.proximityUuidToBytes(skyBeaconConfig.getProximityUUID()),
					0, result, 4, 16);
		}
		return result;
	}

	/**
	 * 单id 广播配置协议2
	 * 
	 * @param skyBeaconConfig
	 * @param key
	 * @return
	 */
	public static byte[] getSingleIDBroadcast2Protocol(
			SKYBeaconConfig skyBeaconConfig, String key) {
		byte[] result = new byte[18];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x14;
		result[17] = 0;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 11, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 11, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (skyBeaconConfig.getMajor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE) {
			System.arraycopy(DataConvertUtils.intToByteArrayTwo(skyBeaconConfig
					.getMajor()), 0, result, 4, 2);
		} else {
			result[17] = (byte) (result[17] | 0x02);
		}
		if (skyBeaconConfig.getMinor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE) {
			System.arraycopy(DataConvertUtils.intToByteArrayTwo(skyBeaconConfig
					.getMinor()), 0, result, 6, 2);
		} else {
			result[17] = (byte) (result[17] | 0x01);
		}
		if (skyBeaconConfig.getIntervalMillisecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE) {
			if (skyBeaconConfig.getIntervalMillisecond() >= 2560) {
				result[8] = (byte) 0xFF;
			} else {
				result[8] = (byte) ((skyBeaconConfig.getIntervalMillisecond() / 10) & 0xFF);
			}
		} else {
			result[8] = 0;
		}
		if (skyBeaconConfig.getTxpower() != SKYBeaconPower
				.getPower(DefaultStaticValues.DEFAULT_SKY_BEACON_TXPOWER_FALSE)) {
			result[9] = (byte) SKYBeaconPower.getPower(skyBeaconConfig
					.getTxpower());
		} else {
			result[9] = 0x7F;
		}
		if (skyBeaconConfig.getMeasuredPower() != DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE) {
			result[10] = (byte) skyBeaconConfig.getMeasuredPower();
		} else {
			result[10] = 0x7F;
		}
		if (skyBeaconConfig.getProximityUUID().equals(
				DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)) {
			result[17] = (byte) (result[17] | 0x04);
		}
		return result;
	}

	/**
	 * 单id 广播配置协议2
	 * 
	 * @param skyBeaconConfig
	 * @param key
	 * @return
	 */
	public static byte[] getDialogSingleIDBroadcast2Protocol(
			SKYBeaconConfig skyBeaconConfig, String key) {
		byte[] result = new byte[18];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x14;
		result[17] = 0;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 11, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 11, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (skyBeaconConfig.getMajor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE) {
			System.arraycopy(DataConvertUtils.intToByteArrayTwo(skyBeaconConfig
					.getMajor()), 0, result, 4, 2);
		} else {
			result[17] = (byte) (result[17] | 0x02);
		}
		if (skyBeaconConfig.getMinor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE) {
			System.arraycopy(DataConvertUtils.intToByteArrayTwo(skyBeaconConfig
					.getMinor()), 0, result, 6, 2);
		} else {
			result[17] = (byte) (result[17] | 0x01);
		}
		if (skyBeaconConfig.getIntervalMillisecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE) {
			if (skyBeaconConfig.getIntervalMillisecond() > 5000) {
				result[8] = (byte) ((((5000 - 1000) / 50) & 0xFF) + 100);
			} else if (skyBeaconConfig.getIntervalMillisecond() > 1000) {
				result[8] = (byte) ((((skyBeaconConfig.getIntervalMillisecond() - 1000) / 50) & 0xFF) + 100);
			} else {
				result[8] = (byte) ((skyBeaconConfig.getIntervalMillisecond() / 10) & 0xFF);
			}
		} else {
			result[8] = 0;
		}
		if (skyBeaconConfig.getTxpower() != SKYBeaconPower
				.getPower(DefaultStaticValues.DEFAULT_SKY_BEACON_TXPOWER_FALSE)) {
			result[9] = (byte) SKYBeaconPower.getPower(skyBeaconConfig
					.getTxpower());
		} else {
			result[9] = 0x7F;
		}
		if (skyBeaconConfig.getMeasuredPower() != DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE) {
			result[10] = (byte) skyBeaconConfig.getMeasuredPower();
		} else {
			result[10] = 0x7F;
		}
		if (skyBeaconConfig.getProximityUUID().equals(
				DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)) {
			result[17] = (byte) (result[17] | 0x04);
		}
		return result;
	}

	/**
	 * 单id led状态配置协议
	 * 
	 * @param ledState
	 * @return
	 */
	public static byte[] getLedStateProtocol(int ledState, String key) {
		byte[] result = new byte[11];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x19;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (ledState != DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE
				&& ledState <= 2 && ledState >= 0) {
			result[10] = (byte) ledState;
		}
		return result;
	}

	/**
	 * 恢复出产设置的协议
	 * 
	 * @param keyRaw
	 * @param keyDefault
	 * @return
	 */
	public static byte[] getResetProtocol(String keyRaw, String keyDefault) {
		byte[] result = new byte[16];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = (byte) 0x99;
		result[3] = (byte) 0x99;
		try {
			if (keyRaw.length() < 6) {
				System.arraycopy(keyRaw
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 4, keyRaw.length());
			} else {
				System.arraycopy(keyRaw
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 4, 6);
			}
			if (keyDefault.length() < 6) {
				System.arraycopy(keyDefault
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 10, keyDefault.length());
			} else {
				System.arraycopy(keyDefault
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 10, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 放篡改密钥配置协议
	 * 
	 * @param keyOld
	 * @param keyNew
	 * @return
	 */
	public static byte[] getKeyProtocol(String keyOld, String keyNew) {
		byte[] result = new byte[16];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x15;
		try {
			if (keyOld.length() < 6) {
				System.arraycopy(keyOld
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 4, keyOld.length());
			} else {
				System.arraycopy(keyOld
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 4, 6);
			}
			if (keyNew.length() < 6) {
				System.arraycopy(keyNew
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 10, keyNew.length());
			} else {
				System.arraycopy(keyNew
						.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0,
						result, 10, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 设置名称协议1
	 * 
	 * @param deviceName
	 * @param key
	 * @return
	 */
	public static byte[] getDeviceName1Protocol(String deviceName) {
		byte[] result = new byte[20];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x16;

		try {
			byte[] tmpName = deviceName
					.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT);
			int lengthNameTmp = tmpName.length;
			int lengthName = lengthNameTmp;
			for (int i = lengthName - 1; i > 0; i--) {
				if (tmpName[i] == 0) {
					lengthName--;
				} else {
					break;
				}
			}
			if (lengthName >= 16) {
				System.arraycopy(tmpName, 0, result, 4, 16);
			} else {
				System.arraycopy(tmpName, 0, result, 4, lengthName);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 设置名称协议2
	 * 
	 * @param deviceName
	 * @param key
	 * @return
	 */
	public static byte[] getDeviceName2Protocol(String deviceName, String key) {
		byte[] result = new byte[14];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x17;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
			byte[] tmpName = deviceName
					.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT);
			int lengthNameTmp = tmpName.length;
			int lengthName = lengthNameTmp;
			for (int i = lengthName - 1; i > 0; i--) {
				if (tmpName[i] == 0) {
					lengthName--;
				} else {
					break;
				}
			}
			if (lengthName >= 20) {
				System.arraycopy(tmpName, 16, result, 10, 4);
			} else if (lengthName >= 16 && lengthName < 20) {
				System.arraycopy(tmpName, 16, result, 10, lengthName - 16);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 温度更新频率协议
	 * 
	 * @param updateSecond
	 * @param key
	 * @return
	 */
	public static byte[] getTemperatureUpdateProtocol(int updateSecond,
			String key) {
		byte[] result = new byte[12];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x51;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
			if (updateSecond < 0) {
				updateSecond = 0;
			}
			if (updateSecond > 1800) {
				updateSecond = 1800;
			}
			byte[] tmpSecond = DataConvertUtils.intToByteArrayTwo(updateSecond);
			System.arraycopy(tmpSecond, 0, result, 10, 2);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 光感数据协议
	 * 
	 * @param key
	 * @param state
	 * @param threshold
	 * @param updateSecond
	 * @param darkIntervalMillisecond
	 * @return
	 */
	public static byte[] getLightSensorProtocol(String key, int state,
			int threshold, int updateSecond, int darkIntervalMillisecond) {
		byte[] result = new byte[15];
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = 0x52;
		result[3] = 0x20;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
			if (updateSecond < 0) {
				updateSecond = 0;
			}
			if (updateSecond > 1800) {
				updateSecond = 1800;
			}
			if (darkIntervalMillisecond < 100) {
				darkIntervalMillisecond = 100;
			}
			if (darkIntervalMillisecond > 10000) {
				darkIntervalMillisecond = 10000;
			}
			result[10] = (byte) state;
			result[11] = (byte) threshold;
			result[12] = (byte) (darkIntervalMillisecond / 50);
			byte[] tmpSecond = DataConvertUtils.intToByteArrayTwo(updateSecond);
			System.arraycopy(tmpSecond, 0, result, 13, 2);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	public static byte[] getDA14580PairingProtocol(String key) {
		byte[] result = new byte[10];
		// 0x86 0x43 0x99 0x98
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = (byte) 0x99;
		result[3] = (byte) 0x98;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	public static byte[] getDA14580FinishConfigProtocol(String key) {
		byte[] result = new byte[10];
		// 0x86 0x43 0x99 0x97
		result[0] = (byte) 0x86;
		result[1] = 0x43;
		result[2] = (byte) 0x99;
		result[3] = (byte) 0x97;
		try {
			if (key.length() < 6) {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, key.length());
			} else {
				System.arraycopy(
						key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT),
						0, result, 4, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	private static int solveCharacteristic2(byte[] datas, int length,
			SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (datas[0] == (byte) 0x86 && datas[1] == (byte) 0x43) {
			skyBeacon.setHardwareVersion(datas[2] & 0xFF);
			skyBeacon.setFirmwareVersionMajor(datas[3] & 0xFF);
			skyBeacon.setFirmwareVersionMinor(datas[4] & 0xFF);
			if ((datas[3] & 0xFF) == 0x02) {
				result = ConfigInfoCode.ErrorCode.CODE_USE_MULTI_ID_BEACON_REPLACE;
				return result;
			}
			skyBeacon.setBattery(datas[5] & 0xFF);
			skyBeacon.setTemperature(datas[6]);
			if ((datas[7] & 0x10) > 0) {
				skyBeacon.setEncrypted(1);
			} else {
				skyBeacon.setEncrypted(0);
			}
			if ((datas[7] & 0x01) > 0) {
				skyBeacon.setLocked(1);
			} else {
				skyBeacon.setLocked(0);
			}
			skyBeacon.setTxpower(SKYBeaconPower.getPower(datas[13]));
			skyBeacon.setMeasuredPower(datas[14]);
			int tmpInterval = (int) (datas[19] & 0xFF);
			if (SKYBeaconVersionUtil.isIntervalFiveSecondProtocol(
					skyBeacon.getHardwareVersion(),
					skyBeacon.getFirmwareVersionMajor(),
					skyBeacon.getFirmwareVersionMinor())) {
				if (tmpInterval > 100) {
					skyBeacon
							.setIntervalMillisecond((tmpInterval - 100) * 50 + 1000);
				} else {
					skyBeacon.setIntervalMillisecond(tmpInterval * 10);
				}
			} else {
				skyBeacon.setIntervalMillisecond(tmpInterval * 10);
			}
		} else {
			result = ConfigInfoCode.ErrorCode.CODE_NOT_SEEKCY_BEACON;
		}
		return result;
	}

	private static int solveCharacteristic3(byte[] datas, int length,
			SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		skyBeacon.setProximityUUID(DataConvertUtils
				.byte16ToProximityUuidStr(datas));
		byte[] majorTmp = new byte[] { datas[16], datas[17] };
		byte[] minorTmp = new byte[] { datas[18], datas[19] };
		skyBeacon.setMajor(DataConvertUtils.byte2ToInt(majorTmp));
		skyBeacon.setMinor(DataConvertUtils.byte2ToInt(minorTmp));
		return result;
	}

	private static int solveCharacteristic7(byte[] datas, int length,
			SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		try {
			String tmpName = new String(datas, "ISO-8859-1");
			skyBeacon.setDeviceName(tmpName);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	private static int solveCharacteristic9(byte[] datas, int length,
			SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (length != 1) {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		} else {
			if (datas[0] == 0) {
				skyBeacon.setLedState(0);
			} else if (datas[0] == 1) {
				skyBeacon.setLedState(1);
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
			}
		}
		return result;
	}

	private static int solveCharacteristic10(byte[] datas, int length,
			SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (length != 6) {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		} else {
			if (datas[0] == 0) {
				skyBeacon.setLightState(0);
			} else if (datas[0] == 1) {
				skyBeacon.setLightState(1);
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
			}
			skyBeacon.setLightThreshold(LightSensorThreshold
					.getThreshold(datas[1]));
			skyBeacon.setDarkIntervalMillisencond(((int) datas[2]) * 50);
			skyBeacon.setLightValue(1000 * ((int) datas[3]) / 256);
			byte[] tmp = new byte[] { datas[4], datas[5] };
			skyBeacon.setLightUpdateSecond(DataConvertUtils.byte2ToInt(tmp));
		}
		return result;
	}

	private static int solveCharacteristic12(byte[] datas, int length,
			SKYBeacon skyBeacon) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (length != 2) {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		} else {
			skyBeacon.setTemperatureUpdateSecond(DataConvertUtils
					.byte2ToInt(datas));
		}
		return result;
	}

	private static int solveCharacteristic2(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (datas[0] == (byte) 0x86 && datas[1] == (byte) 0x43) {
			skyBeaconMultiIDs.setHardwareVersion(datas[2] & 0xFF);
			skyBeaconMultiIDs.setFirmwareVersionMajor(datas[3] & 0xFF);
			skyBeaconMultiIDs.setFirmwareVersionMinor(datas[4] & 0xFF);
			// if ((datas[3] & 0xFF) != 0x02 ) {
			if (!VersionAdaption.isMultiIDsVersion((datas[2] & 0xFF),
					(datas[3] & 0xFF))) {
				result = ConfigInfoCode.ErrorCode.CODE_USE_SINGLE_ID_BEACON_REPLACE;
				return result;
			}
			skyBeaconMultiIDs.setBattery(datas[5] & 0xFF);
			skyBeaconMultiIDs.setTemperature(datas[6]);
			if ((datas[7] & 0x0E) > 0) {
				skyBeaconMultiIDs.setEncrypted(1);
			} else {
				skyBeaconMultiIDs.setEncrypted(0);
			}
			if ((datas[7] & 0x01) > 0) {
				skyBeaconMultiIDs.setLocked(1);
			} else {
				skyBeaconMultiIDs.setLocked(0);
			}
			int tmpInterval = (int) (datas[19] & 0xFF);
			if (SKYBeaconVersionUtil.isIntervalFiveSecondProtocol(
					skyBeaconMultiIDs.getHardwareVersion(),
					skyBeaconMultiIDs.getFirmwareVersionMajor(),
					skyBeaconMultiIDs.getFirmwareVersionMinor())) {
				if (tmpInterval > 100) {
					skyBeaconMultiIDs
							.setIntervalMillisecond((tmpInterval - 100) * 50 + 1000);
				} else {
					skyBeaconMultiIDs.setIntervalMillisecond(tmpInterval * 10);
				}
			} else {
				skyBeaconMultiIDs.setIntervalMillisecond(tmpInterval * 10);
			}

			SKYBeaconCellsSituation skyBeaconCellsSituation = new SKYBeaconCellsSituation(
					SKYBeaconBroadcastID.getBroadcastID((datas[8] & 0x0F) + 1),
					SKYBeaconBroadcastID
							.getBroadcastID(((datas[9] >> 4) & 0x0F) + 1),
					SKYBeaconBroadcastID.getBroadcastID((datas[9] & 0x0F) + 1),
					SKYBeaconBroadcastID
							.getBroadcastID(((datas[10] >> 4) & 0x0F) + 1),
					SKYBeaconBroadcastID.getBroadcastID((datas[10] & 0x0F) + 1),
					SKYBeaconBroadcastID
							.getBroadcastID(((datas[11] >> 4) & 0x0F) + 1),
					SKYBeaconBroadcastID.getBroadcastID((datas[11] & 0x0F) + 1),
					SKYBeaconBroadcastID
							.getBroadcastID(((datas[12] >> 4) & 0x0F) + 1),
					SKYBeaconBroadcastID.getBroadcastID((datas[12] & 0x0F) + 1));
			skyBeaconCellsSituation.setUsedCellsNum((datas[8] >> 4) & 0x0F);
			skyBeaconMultiIDs.setCellsSituation(skyBeaconCellsSituation);
			// 判断是否需要读取uuid major minor信息
			boolean[] reads = new boolean[] { false, false, false };
			for (int i = 0; i < 9; i++) {
				if (SKYBeaconBroadcastID.getInt(skyBeaconMultiIDs
						.getCellsSituation().getUsedCellsSituation()[i]) == 1) {
					reads[0] = true;
				}
				if (SKYBeaconBroadcastID.getInt(skyBeaconMultiIDs
						.getCellsSituation().getUsedCellsSituation()[i]) == 2) {
					reads[1] = true;
				}
				if (SKYBeaconBroadcastID.getInt(skyBeaconMultiIDs
						.getCellsSituation().getUsedCellsSituation()[i]) == 3) {
					reads[2] = true;
				}
			}

			List<SKYBeacon> beaconList = new ArrayList<SKYBeacon>();
			for (int i = 0; i < 3; i++) {
				// if (reads[i]) {
				SKYBeacon skyBeacon = new SKYBeacon();
				skyBeacon
						.setTxpower(SKYBeaconPower.getPower(datas[13 + i * 2]));
				skyBeacon.setMeasuredPower(datas[14 + i * 2]);
				int tmp = (int) Math.pow(2, (i + 1));
				if ((datas[7] & (byte) tmp) > 0) {
					skyBeacon.setEncrypted(1);
				}

				beaconList.add(skyBeacon);
				// } else {
				// beaconList.add(new SKYBeacon());
				// }
			}
			skyBeaconMultiIDs.setBeaconList(beaconList);
		} else {
			result = ConfigInfoCode.ErrorCode.CODE_NOT_SEEKCY_BEACON;
		}
		return result;
	}

	private static int solveCharacteristic3(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		List<SKYBeacon> tmpList = skyBeaconMultiIDs.getBeaconList();
		if (tmpList != null && tmpList.size() > 0) {
			tmpList.get(0).setProximityUUID(
					DataConvertUtils.byte16ToProximityUuidStr(datas));
			byte[] majorTmp = new byte[] { datas[16], datas[17] };
			byte[] minorTmp = new byte[] { datas[18], datas[19] };
			tmpList.get(0).setMajor(DataConvertUtils.byte2ToInt(majorTmp));
			tmpList.get(0).setMinor(DataConvertUtils.byte2ToInt(minorTmp));
		} else {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		}
		return result;
	}

	private static int solveCharacteristic4(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		List<SKYBeacon> tmpList = skyBeaconMultiIDs.getBeaconList();
		if (tmpList != null && tmpList.size() > 0) {
			tmpList.get(1).setProximityUUID(
					DataConvertUtils.byte16ToProximityUuidStr(datas));
			byte[] majorTmp = new byte[] { datas[16], datas[17] };
			byte[] minorTmp = new byte[] { datas[18], datas[19] };
			tmpList.get(1).setMajor(DataConvertUtils.byte2ToInt(majorTmp));
			tmpList.get(1).setMinor(DataConvertUtils.byte2ToInt(minorTmp));
		} else {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		}
		return result;
	}

	private static int solveCharacteristic5(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		List<SKYBeacon> tmpList = skyBeaconMultiIDs.getBeaconList();
		if (tmpList != null && tmpList.size() > 0) {
			tmpList.get(2).setProximityUUID(
					DataConvertUtils.byte16ToProximityUuidStr(datas));
			byte[] majorTmp = new byte[] { datas[16], datas[17] };
			byte[] minorTmp = new byte[] { datas[18], datas[19] };
			tmpList.get(2).setMajor(DataConvertUtils.byte2ToInt(majorTmp));
			tmpList.get(2).setMinor(DataConvertUtils.byte2ToInt(minorTmp));
		} else {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		}
		return result;
	}

	private static int solveCharacteristic6(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;

		return result;
	}

	private static int solveCharacteristic7(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		try {
			String tmpName = new String(datas, "ISO-8859-1");
			skyBeaconMultiIDs.setDeviceName(tmpName);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	private static int solveCharacteristic9(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (length != 1) {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		} else {
			if (datas[0] == 0) {
				skyBeaconMultiIDs.setLedState(0);
			} else if (datas[0] == 1) {
				skyBeaconMultiIDs.setLedState(1);
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
			}
		}
		return result;
	}

	private static int solveCharacteristic10(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (length != 6) {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		} else {
			if (datas[0] == 0) {
				skyBeaconMultiIDs.setLightState(0);
			} else if (datas[0] == 1) {
				skyBeaconMultiIDs.setLightState(1);
			} else {
				result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
			}
			skyBeaconMultiIDs.setLightThreshold(LightSensorThreshold
					.getThreshold(datas[1]));
			skyBeaconMultiIDs
					.setDarkIntervalMillisencond(((int) datas[2]) * 50);
			skyBeaconMultiIDs.setLightValue(1000 * ((int) datas[3]) / 256);
			byte[] tmp = new byte[] { datas[4], datas[5] };
			skyBeaconMultiIDs.setLightUpdateSecond(DataConvertUtils
					.byte2ToInt(tmp));
		}
		return result;
	}

	private static int solveCharacteristic12(byte[] datas, int length,
			SKYBeaconMultiIDs skyBeaconMultiIDs) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (length != 2) {
			result = ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED;
		} else {
			skyBeaconMultiIDs.setTemperatureUpdateSecond(DataConvertUtils
					.byte2ToInt(datas));
		}
		return result;
	}
}
