package com.skybeacon.sdk.config;

import com.skybeacon.sdk.ConfigCallback;
import com.skybeacon.sdk.ConnectionStateCallback;
import com.skybeacon.sdk.locate.PublicStaticValues;
import com.skybeacon.sdk.locate.SKYBeacon;
import com.skybeacon.sdk.locate.SKYBeaconMultiIDs;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Handler;
import android.os.Message;

/**
 * SeekcyBeacon通信类：连接、读取、配置
 * 
 * @author frank
 * 
 */
@SuppressLint("HandlerLeak")
public class SKYBeaconCommunication {

	public class HandleValue {
		public static final int CONNECT_SUCCESS = 1;
		public static final int CONNECT_FAILED = 2;
		public static final int MULTI_ID_BEACON = 3;
		public static final int SINGLE_ID_BEACON = 4;
		public static final int CONFIG_SUCCESS = 5;
		public static final int CONFIG_FAILED = 6;
		public static final int DISCONNECT = 7;
	}

	@SuppressWarnings("unused")
	private Context context;
	private Object lock = new Object();
	private boolean isCommunicateFlag = false;
	private boolean isConnected = false;
	private SKYBeaconCommunicateProcess skyBeaconCommunicateProcess;
	@SuppressWarnings("unused")
	private SKYBeacon skyBeacon;
	@SuppressWarnings("unused")
	private SKYBeaconMultiIDs skyBeaconMultiIDs;
	private ConnectionStateCallback connectionStateCallback;
	private ConfigCallback configCallback;

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public SKYBeaconCommunication(Context context) {
		super();
		this.context = context;
		PublicStaticValues.init(context);
		skyBeaconCommunicateProcess = new SKYBeaconCommunicateProcess(context,
				mHandler);
	}

	/**
	 * 连接设备，只支持单设备连接
	 * 
	 * @param skyBeacon
	 * @param connectionStateCallback
	 * @return result
	 */
	public boolean connect(final SKYBeacon skyBeacon,
			ConnectionStateCallback connectionStateCallback) {
		boolean result = true;
		synchronized (lock) {
			if (isCommunicateFlag) {
				result = false;
			} else {
				this.skyBeacon = skyBeacon;
				this.connectionStateCallback = connectionStateCallback;
				isCommunicateFlag = true;
				Thread thread = new Thread(new Runnable() {

					@Override
					public void run() {
						// TODO Auto-generated method stub
						int result = skyBeaconCommunicateProcess
								.connectDevice(skyBeacon);
						if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
							isConnected = true;
							Message msg = new Message();
							msg.what = HandleValue.CONNECT_SUCCESS;
							msg.arg1 = HandleValue.SINGLE_ID_BEACON;
							if (isCommunicateFlag) {
								mHandler.sendMessage(msg);
							}
						} else {
							isConnected = false;
							Message msg = new Message();
							msg.what = HandleValue.CONNECT_FAILED;
							SKYBeaconConfigException skyBeaconConfigException = new SKYBeaconConfigException();
							skyBeaconConfigException.setCode(result);
							msg.obj = skyBeaconConfigException;
							if (isCommunicateFlag) {
								mHandler.sendMessage(msg);
							}
						}
					}
				});
				thread.start();
			}
		}

		return result;
	}

	/**
	 * 连接设备，只支持单设备连接
	 * 
	 * @param skyBeaconMultiIDs
	 * @param connectionStateCallback
	 * @return result
	 */
	public boolean connect(final SKYBeaconMultiIDs skyBeaconMultiIDs,
			ConnectionStateCallback connectionStateCallback) {
		boolean result = true;
		synchronized (lock) {
			if (isCommunicateFlag) {
				result = false;
			} else {
				this.skyBeaconMultiIDs = skyBeaconMultiIDs;
				this.connectionStateCallback = connectionStateCallback;
				isCommunicateFlag = true;
				Thread thread = new Thread(new Runnable() {

					@Override
					public void run() {
						// TODO Auto-generated method stub
						int result = skyBeaconCommunicateProcess
								.connectDevice(skyBeaconMultiIDs);
						if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
							isConnected = true;
							Message msg = new Message();
							msg.what = HandleValue.CONNECT_SUCCESS;
							msg.arg1 = HandleValue.MULTI_ID_BEACON;
							if (isCommunicateFlag) {
								mHandler.sendMessage(msg);
							}
						} else {
							isConnected = false;
							Message msg = new Message();
							msg.what = HandleValue.CONNECT_FAILED;
							SKYBeaconConfigException skyBeaconConfigException = new SKYBeaconConfigException();
							skyBeaconConfigException.setCode(result);
							msg.obj = skyBeaconConfigException;
							if (isCommunicateFlag) {
								mHandler.sendMessage(msg);
							}
						}
					}
				});
				thread.start();
			}
		}

		return result;
	}

	/**
	 * 断开连接
	 */
	public void disconnect() {
		isCommunicateFlag = false;
		if (isConnected) {
			isConnected = false;
		}
		skyBeaconCommunicateProcess.interruptConfig();
	}

	/**
	 * 配置单id的iBeacon
	 * 
	 * @param skyBeaconConfig
	 * @param configCallback
	 * @return result
	 */
	public boolean configSKYBeacon(final SKYBeaconConfig skyBeaconConfig,
			ConfigCallback configCallback) {
		boolean result = true;
		synchronized (lock) {
			if (!isConnected) {
				result = false;
			} else {
				this.configCallback = configCallback;
				Thread thread = new Thread(new Runnable() {

					@Override
					public void run() {
						// TODO Auto-generated method stub
						int result = skyBeaconCommunicateProcess
								.configBeacon(skyBeaconConfig);
						if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
							Message msg = new Message();
							msg.what = HandleValue.CONFIG_SUCCESS;
							mHandler.sendMessage(msg);
						} else {
							Message msg = new Message();
							msg.what = HandleValue.CONFIG_FAILED;
							SKYBeaconConfigException skyBeaconConfigException = new SKYBeaconConfigException();
							skyBeaconConfigException.setCode(result);
							msg.obj = skyBeaconConfigException;
							mHandler.sendMessage(msg);
						}
					}
				});
				thread.start();
			}
		}

		return result;
	}

	/**
	 * 配置多id的iBeacon
	 * 
	 * @param skyBeaconConfigMultiIDs
	 * @param configCallback
	 * @return result
	 */
	public boolean configSKYBeaconMultiIDs(
			final SKYBeaconConfigMultiIDs skyBeaconConfigMultiIDs,
			ConfigCallback configCallback) {
		boolean result = true;
		synchronized (lock) {
			if (!isConnected) {
				result = false;
			} else {
				this.configCallback = configCallback;
				Thread thread = new Thread(new Runnable() {

					@Override
					public void run() {
						// TODO Auto-generated method stub
						int result = skyBeaconCommunicateProcess
								.configBeacon(skyBeaconConfigMultiIDs);
						if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
							Message msg = new Message();
							msg.what = HandleValue.CONFIG_SUCCESS;
							mHandler.sendMessage(msg);
						} else {
							Message msg = new Message();
							msg.what = HandleValue.CONFIG_FAILED;
							SKYBeaconConfigException skyBeaconConfigException = new SKYBeaconConfigException();
							skyBeaconConfigException.setCode(result);
							msg.obj = skyBeaconConfigException;
							mHandler.sendMessage(msg);
						}
					}
				});
				thread.start();
			}
		}

		return result;
	}

	private Handler mHandler = new Handler() {
		public void handleMessage(Message msg) {
			if (!isCommunicateFlag) {
				return;
			}
			switch (msg.what) {
			case HandleValue.CONNECT_SUCCESS:
				if (msg.arg1 == HandleValue.MULTI_ID_BEACON) {
					SKYBeaconMultiIDs skyBeaconMultiIDs = null;
					skyBeaconMultiIDs = skyBeaconCommunicateProcess
							.getSkyBeaconMultiIDs();
					connectionStateCallback
							.onConnectedSuccess(skyBeaconMultiIDs);
				} else if (msg.arg1 == HandleValue.SINGLE_ID_BEACON) {
					SKYBeacon skyBeacon = null;
					skyBeacon = skyBeaconCommunicateProcess.getSkyBeacon();
					connectionStateCallback.onConnectedSuccess(skyBeacon);
				}
				break;
			case HandleValue.CONNECT_FAILED:
				isCommunicateFlag = false;
				SKYBeaconConfigException exception1 = new SKYBeaconConfigException();
				exception1 = (SKYBeaconConfigException) msg.obj;
				connectionStateCallback.onConnectedFailed(exception1);
				break;
			case HandleValue.CONFIG_SUCCESS:
				configCallback.onConfigSuccess();
				break;
			case HandleValue.CONFIG_FAILED:
				SKYBeaconConfigException exception2 = new SKYBeaconConfigException();
				exception2 = (SKYBeaconConfigException) msg.obj;
				configCallback.onConfigFailed(exception2);
				break;
			case HandleValue.DISCONNECT:
				isCommunicateFlag = false;
				connectionStateCallback.onDisconnected();
				break;
			}
			super.handleMessage(msg);
		}
	};
}
