package com.skybeacon.sdk.config;

import java.io.Serializable;

import com.skybeacon.sdk.utils.DefaultStaticValues;

/**
 * 单ID iBeacon的配置存储类
 * 
 * @author frank
 * 
 */
public class SKYBeaconConfig implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	// 设备名称，多id iBeacon配置不使用该字段
	private String deviceName = DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME;
	// uuid
	private String proximityUUID = DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID;
	// major
	private int major = DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE;
	// minor
	private int minor = DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE;
	// 测量功率
	private int measuredPower = DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE;
	// 发送间隔，多id iBeacon配置不使用该字段
	private int intervalMillisecond = DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE;
	// 发送功率
	private SKYBeaconPower txpower = SKYBeaconPower.POWER_LEVEL_FALSE;
	// 是否打开防篡改模式，多id iBeacon配置不使用该字段
	// 0表示关闭，1表示开启
	private int isLocked = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE;
	// 是否开启防蹭用模式
	// 0表示关闭，1表示开启
	private int isEncrypted = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE;
	// led状态更改
	// 0表示关闭，1表示开启，2表示改变led开关状态（switch）
	private int ledState = DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE;
	// 防篡改密钥，多id iBeacon配置不使用该字段
	private String lockedKey = DefaultStaticValues.DEFAULT_SKY_BEACON_LOCKED_KEY_FALSE;
	// 温度更新频率：秒
	private int temperatureUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_UPDATE_FALSE;
	// light sensor状态：0表示关闭，1表示开启
	private int lightState = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_STATE;
	// 光感黑暗门限值
	private LightSensorThreshold lightThreshold = LightSensorThreshold.DARK_LEVEL_FALSE;
	// 光感更新频率：秒
	private int lightUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND;
	// 黑暗广播频率：毫秒
	private int darkIntervalMillisencond = DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND;

	/**
	 * 构造函数
	 */
	public SKYBeaconConfig() {

	}

	/**
	 * 构造函数
	 * 
	 * @param deviceName
	 */
	public SKYBeaconConfig(String deviceName) {
		super();
		this.deviceName = deviceName;
	}

	/**
	 * 构造函数
	 * 
	 * @param isLocked
	 */
	public SKYBeaconConfig(int isLocked) {
		super();
		this.isLocked = isLocked;
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceName
	 * @param lockedKey
	 */
	public SKYBeaconConfig(String deviceName, String lockedKey) {
		super();
		this.deviceName = deviceName;
		this.lockedKey = lockedKey;
	}

	/**
	 * 构造函数
	 * 
	 * @param isLocked
	 * @param lockedKey
	 */
	public SKYBeaconConfig(int isLocked, String lockedKey) {
		super();
		this.isLocked = isLocked;
		this.lockedKey = lockedKey;
	}

	/**
	 * 构造函数
	 * 
	 * @param proximityUUID
	 * @param major
	 * @param minor
	 * @param measuredPower
	 * @param intervalMillisecond
	 * @param txpower
	 */
	public SKYBeaconConfig(String proximityUUID, int major, int minor,
			int measuredPower, int intervalMillisecond, SKYBeaconPower txpower) {
		super();
		this.proximityUUID = proximityUUID;
		this.major = major;
		this.minor = minor;
		this.measuredPower = measuredPower;
		this.intervalMillisecond = intervalMillisecond;
		this.txpower = txpower;
	}

	/**
	 * 构造函数
	 * 
	 * @param proximityUUID
	 * @param major
	 * @param minor
	 * @param measuredPower
	 * @param intervalMillisecond
	 * @param txpower
	 * @param lockedKey
	 */
	public SKYBeaconConfig(String proximityUUID, int major, int minor,
			int measuredPower, int intervalMillisecond, SKYBeaconPower txpower,
			String lockedKey) {
		super();
		this.proximityUUID = proximityUUID;
		this.major = major;
		this.minor = minor;
		this.measuredPower = measuredPower;
		this.intervalMillisecond = intervalMillisecond;
		this.txpower = txpower;
		this.lockedKey = lockedKey;
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceName
	 * @param proximityUUID
	 * @param major
	 * @param minor
	 * @param measuredPower
	 * @param intervalMillisecond
	 * @param txpower
	 * @param isLocked
	 * @param isEncrypted
	 * @param lockedKey
	 */
	public SKYBeaconConfig(String deviceName, String proximityUUID, int major,
			int minor, int measuredPower, int intervalMillisecond,
			SKYBeaconPower txpower, int isLocked, int isEncrypted,
			String lockedKey) {
		super();
		this.deviceName = deviceName;
		this.proximityUUID = proximityUUID;
		this.major = major;
		this.minor = minor;
		this.measuredPower = measuredPower;
		this.intervalMillisecond = intervalMillisecond;
		this.txpower = txpower;
		this.isLocked = isLocked;
		this.isEncrypted = isEncrypted;
		this.lockedKey = lockedKey;
	}

	public String getDeviceName() {
		return deviceName;
	}

	public void setDeviceName(String deviceName) {
		this.deviceName = deviceName;
	}

	public String getProximityUUID() {
		return proximityUUID;
	}

	public void setProximityUUID(String proximityUUID) {
		this.proximityUUID = proximityUUID;
	}

	public int getMajor() {
		return major;
	}

	public void setMajor(int major) {
		this.major = major;
	}

	public int getMinor() {
		return minor;
	}

	public void setMinor(int minor) {
		this.minor = minor;
	}

	public int getMeasuredPower() {
		return measuredPower;
	}

	public void setMeasuredPower(int measuredPower) {
		this.measuredPower = measuredPower;
	}

	public int getIntervalMillisecond() {
		return intervalMillisecond;
	}

	public void setIntervalMillisecond(int intervalMillisecond) {
		this.intervalMillisecond = intervalMillisecond;
	}

	public SKYBeaconPower getTxpower() {
		return txpower;
	}

	public void setTxpower(SKYBeaconPower txpower) {
		this.txpower = txpower;
	}

	public int isLocked() {
		return isLocked;
	}

	public void setLocked(int isLocked) {
		this.isLocked = isLocked;
	}

	public int isEncrypted() {
		return isEncrypted;
	}

	public void setEncrypted(int isEncrypted) {
		this.isEncrypted = isEncrypted;
	}

	public int getLedState() {
		return ledState;
	}

	public void setLedState(int ledState) {
		this.ledState = ledState;
	}

	public String getLockedKey() {
		return lockedKey;
	}

	public void setLockedKey(String lockedKey) {
		this.lockedKey = lockedKey;
	}

	public int getTemperatureUpdateSecond() {
		return temperatureUpdateSecond;
	}

	public void setTemperatureUpdateSecond(int temperatureUpdateSecond) {
		this.temperatureUpdateSecond = temperatureUpdateSecond;
	}

	public int getLightState() {
		return lightState;
	}

	public void setLightState(int lightState) {
		this.lightState = lightState;
	}

	public LightSensorThreshold getLightThreshold() {
		return lightThreshold;
	}

	public void setLightThreshold(LightSensorThreshold lightThreshold) {
		this.lightThreshold = lightThreshold;
	}

	public int getLightUpdateSecond() {
		return lightUpdateSecond;
	}

	public void setLightUpdateSecond(int lightUpdateSecond) {
		this.lightUpdateSecond = lightUpdateSecond;
	}

	public int getDarkIntervalMillisencond() {
		return darkIntervalMillisencond;
	}

	public void setDarkIntervalMillisencond(int darkIntervalMillisencond) {
		this.darkIntervalMillisencond = darkIntervalMillisencond;
	}
}
