package com.skybeacon.sdk.config;

import com.skybeacon.sdk.locate.PublicStaticValues;

/**
 * 连接、配置过程中的异常信息
 * 
 * @author frank
 * 
 */
public class SKYBeaconConfigException {
	private int code = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
	private String message = "";

	/**
	 * 构造函数
	 */
	public SKYBeaconConfigException() {
		super();
	}

	/**
	 * 构造函数
	 * 
	 * @param code
	 *            错误码
	 */
	public SKYBeaconConfigException(int code) {
		super();
		this.code = code;
		this.message = getMessage(code);
	}

	public int getCode() {
		return code;
	}

	public void setCode(int code) {
		this.code = code;
		this.message = getMessage(code);
	}

	public String getMessage() {
		return message;
	}

	public String getMessage(int code) {
		String result = "";
		switch (code) {
		case ConfigInfoCode.ErrorCode.CODE_DEFAULT:
			break;
		case ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED:
			result = PublicStaticValues.isCN ? "连接设备失败" : "Connect Device Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_CONNECT_TIMEOUT:
			result = PublicStaticValues.isCN ? "连接设备超时" : "Connect Device Timeout.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_DISCOVER_GATT_SERVICE_FAILED:
			result = PublicStaticValues.isCN ? "获取GATT Service失败" : "Get GATT Service Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_DISCOVER_GATT_SERVICE_TIMEOUT:
			result = PublicStaticValues.isCN ? "获取GATT Service超时" : "Get GATT Service Timeout.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_RANDOM_KEY_FAILED:
			result = PublicStaticValues.isCN ? "连接验证失败" : "Connection Verification Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_WRITE_PARAMS_1_FAILED:
			result = PublicStaticValues.isCN ? "写入失败" : "Write Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_WRITE_PARAMS_2_FAILED:
			result = PublicStaticValues.isCN ? "写入失败" : "Write Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_WRITE_PASSCODE_FAILED:
			result = PublicStaticValues.isCN ? "写入失败" : "Write Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_WRITE_PROTOCOL_HEADER_FALSE:
			result = PublicStaticValues.isCN ? "写入错误" : "Write False.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_WRITE_PROTOCOL_PAUSE_FALSE:
			result = PublicStaticValues.isCN ? "写入错误" : "Write False.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_PASSCODE_FALSE:
			result = PublicStaticValues.isCN ? "防篡改密钥错误" : "Passkey False.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_NOT_SEEKCY_BEACON:
			result = PublicStaticValues.isCN ? "非SeekcyBeacon" : "Is't SeekcyBeacon, can' config!";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_2_FAILED:
			result = PublicStaticValues.isCN ? "读取信息失败" : "Read Datas Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_3_FAILED:
			result = PublicStaticValues.isCN ? "读取信息失败" : "Read Datas Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_4_FAILED:
			result = PublicStaticValues.isCN ? "读取信息失败" : "Read Datas Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_5_FAILED:
			result = PublicStaticValues.isCN ? "读取信息失败" : "Read Datas Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_6_FAILED:
			result = PublicStaticValues.isCN ? "读取信息失败" : "Read Datas Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_7_FAILED:
			result = PublicStaticValues.isCN ? "读取信息失败" : "Read Datas Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_GATT_SERVICE_NOT_FOUND:
			result = PublicStaticValues.isCN ? "Gatt Service没有找到" : "Can not found Gatt Service.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_CHARACTERISTIC_NOT_FOUND:
			result = PublicStaticValues.isCN ? "Characteristic没有找到" : "Can not found Characteristic.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_WRITE_CHARACTERISTIC_FAILED:
			result = PublicStaticValues.isCN ? "写Characteristic失败" : "Write Characteristic Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_FAILED:
			result = PublicStaticValues.isCN ? "写Characteristic失败" : "Write Characteristic Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_READ_DATAS_ANALYSIS_FAILED:
			result = PublicStaticValues.isCN ? "读取的数据解析失败" : "Analyze Read Datas Failed.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_PASSCODE_EMPTY:
			result = PublicStaticValues.isCN ? "开启防篡改模式请填写防篡改密钥" : "Please Fill in Passkey First.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_BEACON_ALREADY_LOCKED:
			result = PublicStaticValues.isCN ? "SeekcyBeacon当前已经处于防篡改模式下" : "At Present Already in Passkey Mode.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_INPUT_NULL:
			result = PublicStaticValues.isCN ? "输入的数据为空" : "Input Datas Empty.";
			break;
		case ConfigInfoCode.ErrorCode.CODE_LED_STATE_CONFIG_FAILED_VERSION_LOW:
			result = PublicStaticValues.isCN ? "改变led状态失败，iBeacon版本不支持" : "Change Led Status Failed, this Version is not Supported";
			break;
		case ConfigInfoCode.ErrorCode.CODE_LED_STATE_CONFIG_FAILED_VALUE_FALSE:
			result = PublicStaticValues.isCN ? "改变led状态失败，设置的值有错" : "Change Led Status Failed, Setting Value Error.";
			break;
		}
		return result;
	}
}
