package com.skybeacon.sdk.config;

import java.io.Serializable;

import com.skybeacon.sdk.utils.DefaultStaticValues;

public class SKYBeaconConfigMultiIDs implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	// 设备名称
	private String deviceName = DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME;
	// 发送间隔
	private int intervalMillisecond = DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE;
	// 是否打开防篡改模式
	// 0表示关闭，1表示开启
	private int isLocked = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE;
	// 防篡改密钥
	private String lockedKey = DefaultStaticValues.DEFAULT_SKY_BEACON_LOCKED_KEY_FALSE;
	// 格子使用情况
	private SKYBeaconCellsSituation cellsSituation = null;
	private SKYBeaconConfig configBeacons[] = new SKYBeaconConfig[] {
			new SKYBeaconConfig(), new SKYBeaconConfig(), new SKYBeaconConfig() };
	// 是否配置时间
	private boolean isSetTime = false;
	// 是否恢复出产设置
	private boolean isReset = false;
	// 温度更新频率：秒
	private int temperatureUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_UPDATE_FALSE;
	// led状态更改
	// 0表示关闭，1表示开启，2表示改变led开关状态（switch）
	private int ledState = DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE;
	// light sensor状态：0表示关闭，1表示开启
	private int lightState = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_STATE;
	// 光感黑暗门限值
	private LightSensorThreshold lightThreshold = LightSensorThreshold.DARK_LEVEL_FALSE;
	// 光感更新频率：秒
	private int lightUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND;
	// 黑暗广播频率：毫秒
	private int darkIntervalMillisencond = DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND;

	/**
	 * 构造函数
	 */
	public SKYBeaconConfigMultiIDs() {
		super();
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceName
	 *            设备名称
	 */
	public SKYBeaconConfigMultiIDs(String deviceName) {
		super();
		this.deviceName = deviceName;
	}

	/**
	 * 构造函数
	 * 
	 * @param intervalMillisecond
	 *            发送间隔，ms
	 */
	public SKYBeaconConfigMultiIDs(int intervalMillisecond) {
		super();
		this.intervalMillisecond = intervalMillisecond;
	}

	/**
	 * 构造函数
	 * 
	 * @param cellsSituation
	 *            格子使用情况
	 */
	public SKYBeaconConfigMultiIDs(SKYBeaconCellsSituation cellsSituation) {
		super();
		if (this.cellsSituation == null) {
			this.cellsSituation = new SKYBeaconCellsSituation(cellsSituation);
		} else {
			this.cellsSituation = cellsSituation;
		}
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceName
	 *            设备名称
	 * @param intervalMillisecond
	 *            发送频率，ms
	 * @param isLocked
	 *            是否防蹭用模式开启
	 * @param cellsSituation
	 *            格子使用情况
	 */
	public SKYBeaconConfigMultiIDs(String deviceName, int intervalMillisecond,
			int isLocked, SKYBeaconCellsSituation cellsSituation) {
		super();
		this.deviceName = deviceName;
		this.intervalMillisecond = intervalMillisecond;
		this.isLocked = isLocked;
		if (this.cellsSituation == null) {
			this.cellsSituation = new SKYBeaconCellsSituation(cellsSituation);
		} else {
			this.cellsSituation = cellsSituation;
		}
	}

	/**
	 * 构造函数
	 * 
	 * @param lockedKey
	 *            防蹭用密钥
	 * @param intervalMillisecond
	 *            发送频率，ms
	 */
	public SKYBeaconConfigMultiIDs(String lockedKey, int intervalMillisecond) {
		super();
		this.lockedKey = lockedKey;
		this.intervalMillisecond = intervalMillisecond;
	}

	/**
	 * 构造函数
	 * 
	 * @param lockedKey
	 *            防蹭用密钥
	 * @param cellsSituation
	 *            格子使用情况
	 */
	public SKYBeaconConfigMultiIDs(String lockedKey,
			SKYBeaconCellsSituation cellsSituation) {
		super();
		this.lockedKey = lockedKey;
		if (this.cellsSituation == null) {
			this.cellsSituation = new SKYBeaconCellsSituation(cellsSituation);
		} else {
			this.cellsSituation = cellsSituation;
		}
	}

	/**
	 * 构造函数
	 * 
	 * @param lockedKey
	 *            防蹭用密钥
	 * @param deviceName
	 *            设备名称
	 * @param intervalMillisecond
	 *            发送频率，ms
	 * @param isLocked
	 *            是否防蹭用模式开启
	 * @param cellsSituation
	 *            格子使用情况
	 */
	public SKYBeaconConfigMultiIDs(String lockedKey, String deviceName,
			int intervalMillisecond, int isLocked,
			SKYBeaconCellsSituation cellsSituation) {
		super();
		this.lockedKey = lockedKey;
		this.deviceName = deviceName;
		this.intervalMillisecond = intervalMillisecond;
		this.isLocked = isLocked;
		if (this.cellsSituation == null) {
			this.cellsSituation = new SKYBeaconCellsSituation(cellsSituation);
		} else {
			this.cellsSituation = cellsSituation;
		}
	}

	public String getDeviceName() {
		return deviceName;
	}

	public void setDeviceName(String deviceName) {
		this.deviceName = deviceName;
	}

	public int getIntervalMillisecond() {
		return intervalMillisecond;
	}

	public void setIntervalMillisecond(int intervalMillisecond) {
		this.intervalMillisecond = intervalMillisecond;
	}

	public int isLocked() {
		return isLocked;
	}

	public void setLocked(int isLocked) {
		this.isLocked = isLocked;
	}

	public String getLockedKey() {
		return lockedKey;
	}

	public void setLockedKey(String lockedKey) {
		this.lockedKey = lockedKey;
	}

	public SKYBeaconCellsSituation getCellsSituation() {
		return cellsSituation;
	}

	public void setCellsSituation(SKYBeaconCellsSituation cellsSituation) {
		if (this.cellsSituation == null) {
			this.cellsSituation = new SKYBeaconCellsSituation(cellsSituation);
		} else {
			this.cellsSituation = cellsSituation;
		}
	}

	public SKYBeaconConfig[] getConfigBeacons() {
		return configBeacons;
	}

	public void setConfigBeaconsID1(SKYBeaconConfig configBeacon) {
		this.configBeacons[0] = configBeacon;
	}

	public void setConfigBeaconsID2(SKYBeaconConfig configBeacon) {
		this.configBeacons[1] = configBeacon;
	}

	public void setConfigBeaconsID3(SKYBeaconConfig configBeacon) {
		this.configBeacons[2] = configBeacon;
	}

	public int getIsLocked() {
		return isLocked;
	}

	public void setIsLocked(int isLocked) {
		this.isLocked = isLocked;
	}

	public boolean isSetTime() {
		return isSetTime;
	}

	public void setSetTime(boolean isSetTime) {
		this.isSetTime = isSetTime;
	}

	public boolean isReset() {
		return isReset;
	}

	public void setReset(boolean isReset) {
		this.isReset = isReset;
	}

	public int getLedState() {
		return ledState;
	}

	public void setLedState(int ledState) {
		this.ledState = ledState;
	}

	public int getTemperatureUpdateSecond() {
		return temperatureUpdateSecond;
	}

	public void setTemperatureUpdateSecond(int temperatureUpdateSecond) {
		this.temperatureUpdateSecond = temperatureUpdateSecond;
	}

	public int getLightState() {
		return lightState;
	}

	public void setLightState(int lightState) {
		this.lightState = lightState;
	}

	public LightSensorThreshold getLightThreshold() {
		return lightThreshold;
	}

	public void setLightThreshold(LightSensorThreshold lightThreshold) {
		this.lightThreshold = lightThreshold;
	}

	public int getLightUpdateSecond() {
		return lightUpdateSecond;
	}

	public void setLightUpdateSecond(int lightUpdateSecond) {
		this.lightUpdateSecond = lightUpdateSecond;
	}

	public int getDarkIntervalMillisencond() {
		return darkIntervalMillisencond;
	}

	public void setDarkIntervalMillisencond(int darkIntervalMillisencond) {
		this.darkIntervalMillisencond = darkIntervalMillisencond;
	}
}
