package com.skybeacon.sdk.config;

/**
 * Seekcy iBeacon的功率枚举类
 * 
 * @author frank
 * 
 */
public enum SKYBeaconPower {

	/*
	 * ! 配置时txPower的值 注意：不同版本的beacon对txPower有不同允许范围
	 * 版本号（hardwareVersion-firmwareVersionMajor-firmwareVersionMinor）：
	 * 5-1-1、5-2-0、3-1-0允许范围是:0,1,2,3,4,5,-3,-6,-9,-12,-15,-18,-21
	 * 7-1-0允许范围时：0，－20 其余的版本允许范围是：0,-2,-4,-6,-8,-10 (单位dBm)
	 */

	POWER_LEVEL_FALSE(100), POWER_LEVEL_DEFAULT(0), POWER_LEVEL_PLUS_5(5), POWER_LEVEL_PLUS_4(
			4), POWER_LEVEL_PLUS_3(3), POWER_LEVEL_PLUS_2(2), POWER_LEVEL_PLUS_1(
			1), POWER_LEVEL_PLUS_0(0), POWER_LEVEL_MINUS_2(-2), POWER_LEVEL_MINUS_3(
			-3), POWER_LEVEL_MINUS_4(-4), POWER_LEVEL_MINUS_6(-6), POWER_LEVEL_MINUS_8(
			-8), POWER_LEVEL_MINUS_9(-9), POWER_LEVEL_MINUS_10(-10), POWER_LEVEL_MINUS_12(
			-12), POWER_LEVEL_MINUS_15(-15), POWER_LEVEL_MINUS_18(-18), POWER_LEVEL_MINUS_20(
			-20), POWER_LEVEL_MINUS_21(-21);

	// 定义私有变量
	private int eCode;

	/**
	 * 构造函数，枚举类型只能为私有
	 * 
	 * @param eCode
	 */
	private SKYBeaconPower(int eCode) {
		this.eCode = eCode;
	}

	@Override
	public String toString() {
		return String.valueOf(this.eCode);
	}

	public static SKYBeaconPower getPower(int power) {
		SKYBeaconPower result = SKYBeaconPower.POWER_LEVEL_FALSE;
		switch (power) {
		case 5:
			result = SKYBeaconPower.POWER_LEVEL_PLUS_5;
			break;
		case 4:
			result = SKYBeaconPower.POWER_LEVEL_PLUS_4;
			break;
		case 3:
			result = SKYBeaconPower.POWER_LEVEL_PLUS_3;
			break;
		case 2:
			result = SKYBeaconPower.POWER_LEVEL_PLUS_2;
			break;
		case 1:
			result = SKYBeaconPower.POWER_LEVEL_PLUS_1;
			break;
		case 0:
			result = SKYBeaconPower.POWER_LEVEL_PLUS_0;
			break;
		case -2:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_2;
			break;
		case -3:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_3;
			break;
		case -4:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_4;
			break;
		case -6:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_6;
			break;
		case -8:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_8;
			break;
		case -9:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_9;
			break;
		case -10:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_10;
			break;
		case -12:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_12;
			break;
		case -15:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_15;
			break;
		case -18:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_18;
			break;
		case -20:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_20;
			break;
		case -21:
			result = SKYBeaconPower.POWER_LEVEL_MINUS_21;
			break;
		}
		return result;
	}

	public static int getPower(SKYBeaconPower skyBeaconPower) {
		int result = 0x7F;
		if (skyBeaconPower == POWER_LEVEL_PLUS_5) {
			result = 5;
		} else if (skyBeaconPower == POWER_LEVEL_PLUS_4) {
			result = 4;
		} else if (skyBeaconPower == POWER_LEVEL_PLUS_3) {
			result = 3;
		} else if (skyBeaconPower == POWER_LEVEL_PLUS_2) {
			result = 2;
		} else if (skyBeaconPower == POWER_LEVEL_PLUS_1) {
			result = 1;
		} else if (skyBeaconPower == POWER_LEVEL_PLUS_0) {
			result = 0;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_2) {
			result = -2;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_3) {
			result = -3;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_4) {
			result = -4;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_6) {
			result = -6;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_8) {
			result = -8;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_9) {
			result = -9;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_10) {
			result = -10;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_12) {
			result = -12;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_15) {
			result = -15;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_18) {
			result = -18;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_20) {
			result = -20;
		} else if (skyBeaconPower == POWER_LEVEL_MINUS_21) {
			result = -21;
		}
		return result;
	}
}
