package com.skybeacon.sdk.config;

import java.util.ArrayList;

import com.skybeacon.sdk.config.SKYBeaconCommunicateProcess.GattAttributes;

public class SKYBeaconVersionUtil {

	/**
	 * 硬件版本号
	 * 
	 * @author frank
	 * 
	 */
	public class HardwareVersion {
		static public final int HARDWARE_S1_VERSION = 0;
		static public final int HARDWARE_M0_VERSION = 1;
		static public final int HARDWARE_S1P_VERSION = 2;
		static public final int HARDWARE_M0L_VERSION = 3;
		static public final int HARDWARE_M1_VERSION = 4;
		static public final int HARDWARE_S1L_VERSION = 5;
		static public final int HARDWARE_K1L_VERSION = 6;
		static public final int HARDWARE_S1U_VERSION = 7;
		static public final int HARDWARE_M0U_VERSION = 8;
		static public final int HARDWARE_S2U_VERSION = 9;
		static public final int HARDWARE_T1U_VERSION = 18;
	}

	/**
	 * 写入数据标识
	 * 
	 * @author frank
	 * 
	 */
	public class WriteFlags {
		static public final int WRITE_FLAG_LED = 1;
		static public final int WRITE_FLAG_TEMPERATURE_SENSOR = 2;
		static public final int WRITE_FLAG_LIGHT_SENSOR = 3;
	}

	/**
	 * 获取Single ID beacon需要读取的GattAttributes
	 * 
	 * @param hardwareVersion
	 * @param softwareMajorVersion
	 * @param softwareMinorVersion
	 * @return
	 */
	static public ArrayList<String> getReadCharacteristicSingleId(int hardwareVersion, int softwareMajorVersion, int softwareMinorVersion) {
		ArrayList<String> result = new ArrayList<String>();
		switch (hardwareVersion) {
		case HardwareVersion.HARDWARE_S1_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 0) {
				result.add(GattAttributes.CHARACTERISTIC_9_READ_UUID);
			}
			break;
		case HardwareVersion.HARDWARE_S1P_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0L_VERSION:
			if (softwareMajorVersion == 1) {
				if (softwareMinorVersion == 0 || softwareMinorVersion == 1) {
					result.add(GattAttributes.CHARACTERISTIC_9_READ_UUID);
					result.add(GattAttributes.CHARACTERISTIC_10_READ_UUID);
					result.add(GattAttributes.CHARACTERISTIC_12_READ_UUID);
				}
			}
			break;
		case HardwareVersion.HARDWARE_M1_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_S1L_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 1) {
				result.add(GattAttributes.CHARACTERISTIC_12_READ_UUID);
			}
			break;
		case HardwareVersion.HARDWARE_K1L_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 0) {
				result.add(GattAttributes.CHARACTERISTIC_12_READ_UUID);
			}
			break;
		case HardwareVersion.HARDWARE_S1U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_M0U_VERSION:
			result.add(GattAttributes.CHARACTERISTIC_9_READ_UUID);
			break;
		case HardwareVersion.HARDWARE_S2U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_T1U_VERSION:
			break;
		default:
			break;
		}
		return result;
	}

	/**
	 * 获取Multi ID beacon需要读取的GattAttributes
	 * 
	 * @param hardwareVersion
	 * @param softwareMajorVersion
	 * @param softwareMinorVersion
	 * @return
	 */
	static public ArrayList<String> getReadCharacteristicMultiId(int hardwareVersion, int softwareMajorVersion, int softwareMinorVersion) {
		ArrayList<String> result = new ArrayList<String>();
		result.add(GattAttributes.CHARACTERISTIC_7_READ_UUID);
		switch (hardwareVersion) {
		case HardwareVersion.HARDWARE_S1_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1P_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0L_VERSION:
			if (softwareMajorVersion == 3) {
				result.add(GattAttributes.CHARACTERISTIC_9_READ_UUID);
				result.add(GattAttributes.CHARACTERISTIC_12_READ_UUID);
			}
			break;
		case HardwareVersion.HARDWARE_M1_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_S1L_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 1) {
				result.add(GattAttributes.CHARACTERISTIC_12_READ_UUID);
			}
			break;
		case HardwareVersion.HARDWARE_K1L_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_M0U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_S2U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_T1U_VERSION:
			// TODO
			break;
		default:
			break;
		}
		return result;
	}

	/**
	 * 获取Single ID beacon写入数据
	 * 
	 * @param hardwareVersion
	 * @param softwareMajorVersion
	 * @param softwareMinorVersion
	 * @return
	 */
	static public ArrayList<Integer> getWriteFlagsSingleId(int hardwareVersion, int softwareMajorVersion, int softwareMinorVersion) {
		ArrayList<Integer> result = new ArrayList<Integer>();
		switch (hardwareVersion) {
		case HardwareVersion.HARDWARE_S1_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 0) {
				result.add(WriteFlags.WRITE_FLAG_LED);
			}
			break;
		case HardwareVersion.HARDWARE_S1P_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0L_VERSION:
			if (softwareMajorVersion == 1) {
				if (softwareMinorVersion == 0 || softwareMinorVersion == 1) {
					result.add(WriteFlags.WRITE_FLAG_LED);
					result.add(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR);
					// result.add(WriteFlags.WRITE_FLAG_LIGHT_SENSOR);
				}
			}
			break;
		case HardwareVersion.HARDWARE_M1_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_S1L_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 1) {
				result.add(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR);
			}
			break;
		case HardwareVersion.HARDWARE_K1L_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 0) {
				result.add(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR);
			}
			break;
		case HardwareVersion.HARDWARE_S1U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_M0U_VERSION:
			if (softwareMajorVersion == 1) {
				result.add(WriteFlags.WRITE_FLAG_LED);
			}
			break;
		case HardwareVersion.HARDWARE_S2U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_T1U_VERSION:
			// TODO
			break;
		default:
			break;
		}
		return result;
	}

	/**
	 * 获取Multi ID beacon写入数据
	 * 
	 * @param hardwareVersion
	 * @param softwareMajorVersion
	 * @param softwareMinorVersion
	 * @return
	 */
	static public ArrayList<Integer> getWriteFlagsMultiId(int hardwareVersion, int softwareMajorVersion, int softwareMinorVersion) {
		ArrayList<Integer> result = new ArrayList<Integer>();
		switch (hardwareVersion) {
		case HardwareVersion.HARDWARE_S1_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1P_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0L_VERSION:
			if (softwareMajorVersion == 3) {
				result.add(WriteFlags.WRITE_FLAG_LED);
				result.add(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR);
				// result.add(WriteFlags.WRITE_FLAG_LIGHT_SENSOR);
			}
			break;
		case HardwareVersion.HARDWARE_M1_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1L_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 1) {
				result.add(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR);
			}
			break;
		case HardwareVersion.HARDWARE_K1L_VERSION:
			if (softwareMajorVersion == 1 && softwareMinorVersion == 0) {
				result.add(WriteFlags.WRITE_FLAG_TEMPERATURE_SENSOR);
			}
			break;
		case HardwareVersion.HARDWARE_S1U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_M0U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_S2U_VERSION:
			// TODO
			break;
		case HardwareVersion.HARDWARE_T1U_VERSION:
			// TODO
			break;
		default:
			break;
		}
		return result;
	}

	static public boolean isDA14580(int hardwareVersion, int softwareMajorVersion, int softwareMinorVersion) {
		boolean result = false;

		if (hardwareVersion == HardwareVersion.HARDWARE_S1U_VERSION || hardwareVersion == HardwareVersion.HARDWARE_M0U_VERSION || hardwareVersion == HardwareVersion.HARDWARE_S2U_VERSION
				|| hardwareVersion == HardwareVersion.HARDWARE_T1U_VERSION) {
			result = true;
		}

		return result;
	}

	static public boolean isIntervalFiveSecondProtocol(int hardwareVersion, int softwareMajorVersion, int softwareMinorVersion) {
		boolean result = false;

		switch (hardwareVersion) {
		case HardwareVersion.HARDWARE_S1_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1P_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0L_VERSION:
			if (softwareMajorVersion == 3) {
				result = true;
			}
			break;
		case HardwareVersion.HARDWARE_M1_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1L_VERSION:
			if (softwareMajorVersion == 3) {
				result = true;
			}
			break;
		case HardwareVersion.HARDWARE_K1L_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1U_VERSION:
			result = true;
			break;
		case HardwareVersion.HARDWARE_M0U_VERSION:
			result = true;
			break;
		case HardwareVersion.HARDWARE_S2U_VERSION:
			result = true;
			break;
		case HardwareVersion.HARDWARE_T1U_VERSION:
			result = true;
			break;
		default:
			break;
		}
		return result;
	}

	static public boolean isEncryptedVersion2(int hardwareVersion, int softwareMajorVersion, int softwareMinorVersion) {
		boolean result = false;

		switch (hardwareVersion) {
		case HardwareVersion.HARDWARE_S1_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1P_VERSION:
			break;
		case HardwareVersion.HARDWARE_M0L_VERSION:
			if (softwareMajorVersion == 3) {
				result = true;
			}
			break;
		case HardwareVersion.HARDWARE_M1_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1L_VERSION:
			if (softwareMajorVersion == 3) {
				result = true;
			}
			break;
		case HardwareVersion.HARDWARE_K1L_VERSION:
			break;
		case HardwareVersion.HARDWARE_S1U_VERSION:
			if (softwareMajorVersion == 3) {
				result = true;
			}
			break;
		case HardwareVersion.HARDWARE_M0U_VERSION:
			break;
		case HardwareVersion.HARDWARE_S2U_VERSION:
			break;
		case HardwareVersion.HARDWARE_T1U_VERSION:
			break;
		default:
			break;
		}
		return result;
	}
}
