package com.skybeacon.sdk.entity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.net.wifi.ScanResult;

import com.skybeacon.sdk.utils.DefaultStaticValues;

public class WifiDatasEntity {
	private static WifiDatasEntity instance = null;
	private Map<String, WifiData> wifiDatas = new HashMap<String, WifiDatasEntity.WifiData>();
	private Object lock = new Object();

	public synchronized static WifiDatasEntity getInstance() {
		if (instance == null) {
			instance = new WifiDatasEntity();
		}
		return instance;
	}

	public void addWifiDatas(List<ScanResult> scanInfo) {
		synchronized (lock) {
			if (scanInfo != null) {
				for (ScanResult result : scanInfo) {
					String mac = result.BSSID.replace(":", "").toUpperCase();
					int rssiTmp = result.level;
					// Log.e("Test", String.valueOf(rssiTmp));
					if (wifiDatas.containsKey(mac)) {
						// Log.e("raw", String.valueOf(rssiTmp));
						int rssiInput = wifiDatas.get(mac).getRssi() + rssiTmp;
						int countInput = wifiDatas.get(mac).getCount() + 1;
						// Log.e("input", String.valueOf(rssiInput) + "\t" +
						// String.valueOf(countInput));
						wifiDatas.get(mac).setRssi(rssiInput);
						wifiDatas.get(mac).setCount(countInput);
					} else {
						wifiDatas.put(mac, new WifiData(mac, rssiTmp));
						wifiDatas.get(mac).setCount(1);
					}
				}
			}
		}
	}

	public ArrayList<WifiDatasEntity.WifiData> getWifiDatas() {
		ArrayList<WifiDatasEntity.WifiData> result = null;
		synchronized (lock) {
			result = new ArrayList<WifiDatasEntity.WifiData>(wifiDatas.values());
			wifiDatas.clear();
		}
		return result;
	}

	public class WifiData {
		private int rssi = DefaultStaticValues.DEFAULT_SKY_BEACON_RSSI_FALSE;
		private String deviceMacString = DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS;
		// private long deviceMacLong =
		// DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS_LONG;
		private int count = 0;

		public WifiData(String deviceMac, int rssi) {
			deviceMacString = deviceMac;
			this.rssi = rssi;
		}

		public int getRssi() {
			return rssi;
		}

		public void setRssi(int rssi) {
			this.rssi = rssi;
		}

		public String getDeviceMacString() {
			return deviceMacString;
		}

		public void setDeviceMacString(String deviceMacString) {
			this.deviceMacString = deviceMacString;
		}

		// public long getDeviceMacLong() {
		// return deviceMacLong;
		// }
		//
		// public void setDeviceMacLong(long deviceMacLong) {
		// this.deviceMacLong = deviceMacLong;
		// }

		public int getCount() {
			return count;
		}

		public void setCount(int count) {
			this.count = count;
		}

		public int getRssiFiltered() {
			// Log.e("rssi-count", String.valueOf(rssi) + "\t" +
			// String.valueOf(count));
			if (count == 0) {
				return rssi;
			} else {
				return (rssi / count);
			}
		}
	}
}
