package com.skybeacon.sdk.locate;

import java.io.IOException;

import com.skybeacon.sdk.utils.DataConvertUtils;
import com.skybeacon.sdk.utils.FileAdapter;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Environment;
import android.telephony.TelephonyManager;

@SuppressLint("DefaultLocale")
public class LocateConfigurator {
	private static final String META_DATA_KEY = "SEEKCY_SDK_KEY";
	private static LocateConfigurator instance;

	private Context context;
	private String deviceAddress = "";
	private int versionCode = 0;
	private String versionName = "";
	private String appKey = "";
	private String deviceType = "";
	private String deviceIMEI = "";
	private String deviceSoftwareVersion = "";
	private String deviceIMSI = "";

	public synchronized static LocateConfigurator getInstance() {
		if (instance == null) {
			instance = new LocateConfigurator();
		}
		return instance;
	}

	public void init(Context context) {
		this.context = context;
		SharedPreferences sharedPreferences = context.getSharedPreferences(
				"SeekcySDK", Activity.MODE_PRIVATE);
		deviceAddress = sharedPreferences.getString("DeviceAddress", "");
		String macTmp = getWifiMac();
		if (!macTmp.equals("")) {
			if (deviceAddress.equals("")) {
				deviceAddress = macTmp;
			} else if (!deviceAddress.equals(macTmp)) {
				// 发现mac不同的处理，纪录log日志
				String strLog = DataConvertUtils.getCurrentTimestampSecond()
						+ "\t" + "设备的mac地址变化问题：" + deviceAddress + "->"
						+ macTmp;
				deviceAddress = macTmp;
				sharedPreferences.edit().putString("mac", deviceAddress)
						.commit();
				try {
					FileAdapter.writeSDFileAppend(PublicStaticValues.FILE_ROOT
							+ PublicStaticValues.FILE_LOGS_DERECTORY,
							DataConvertUtils.getCurrentTimestampDay() + ".txt",
							strLog);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		try {
			PackageManager pm = context.getPackageManager();
			PackageInfo pinfo = null;
			pinfo = pm.getPackageInfo(context.getPackageName(),
					PackageManager.GET_CONFIGURATIONS);
			versionCode = pinfo.versionCode;
			versionName = pinfo.versionName;
		} catch (NameNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		TelephonyManager tm = (TelephonyManager) context
				.getSystemService(Context.TELEPHONY_SERVICE);
		deviceIMEI = tm.getDeviceId();
		deviceSoftwareVersion = tm.getDeviceSoftwareVersion();
		deviceIMSI = tm.getSubscriberId();

		appKey = getMetaData(META_DATA_KEY);
		deviceType = getPhoneType();
	}

	public String getDeviceAddress() {
		if (deviceAddress.equals("")) {
			deviceAddress = getWifiMac();
			if (!deviceAddress.equals("")) {
				SharedPreferences sharedPreferences = context
						.getSharedPreferences("SeekcySDK",
								Activity.MODE_PRIVATE);
				sharedPreferences.edit().putString("mac", deviceAddress)
						.commit();
			}
		}
		return deviceAddress.toUpperCase();
	}

	/**
	 * 检查sd卡是否有效
	 * 
	 * @return
	 */
	public static boolean isSdcardValid() {
		return Environment.getExternalStorageState().equals(
				android.os.Environment.MEDIA_MOUNTED);
	}

	/**
	 * 获取sdk的版本号
	 * 
	 * @return
	 */
	public int getVersionCode() {
		return versionCode;
	}

	/**
	 * 获取sdk的版本名称
	 * 
	 * @return
	 */
	public String getVersionName() {
		return versionName;
	}

	/**
	 * app key
	 * 
	 * @return
	 */
	public String getAppKey() {
		return appKey;
	}

	/**
	 * 手机型号
	 * 
	 * @return
	 */
	public String getDeviceType() {
		return deviceType;
	}

	public String getDeviceIMEI() {
		return deviceIMEI;
	}

	public String getDeviceIMSI() {
		return deviceIMSI;
	}

	public String getDeviceSoftwareVersion() {
		return deviceSoftwareVersion;
	}

	private String getWifiMac() {
		String result = "";
		try {
			WifiManager wifiManager = (WifiManager) context
					.getSystemService(Context.WIFI_SERVICE);
			WifiInfo wifiInfo = wifiManager.getConnectionInfo();
			if (wifiInfo != null) {
				String mac = wifiInfo.getMacAddress();
				if (mac != null) {
					result = mac.replace(":", "");
				}
			}
		} catch (Exception e) {

		}
		return result;
	}

	private String getMetaData(String key) {
		Object msg = new Object();
		try {
			ApplicationInfo info = context.getPackageManager()
					.getApplicationInfo(context.getPackageName(),
							PackageManager.GET_META_DATA);
//			msg = info.metaData.getString(key);
			msg = (Object)info.metaData.get(key);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return msg.toString();
	}

	private String getPhoneType() {
		return android.os.Build.MODEL;
	}

	private LocateConfigurator() {

	}
}
