package com.skybeacon.sdk.locate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.skybeacon.sdk.utils.DefaultStaticValues;

/**
 * 监听处理类
 */
public class MonitorBeaconsProcess {
	private final String LOG_TAG = "MonitorLog:";

	private static int RSSI_MIN_THRESHOLD = -90;
	private static int STATE_READY = 0;
	public static int STATE_EXITED_INIT = 1;
	public static int STATE_ENTERED_INIT = 2;
	private static int STATE_EXITED = 11;
	private static int STATE_ENTERED = 22;

	class BeaconStates {
		public int state = STATE_READY;
		public boolean isScan = false;
		public SKYRegion skyRegion;
	}

	private ArrayList<SKYRegion> skyBeaconRegions;
	private Map<String, BeaconStates> beaconsStates;

	public MonitorBeaconsProcess() {
		skyBeaconRegions = new ArrayList<SKYRegion>();
		beaconsStates = new HashMap<String, BeaconStates>();
	}

	/**
	 * 添加监听区域
	 * 
	 * @param skyBeaconRegion
	 */
	public void addSKYBeaconRegion(SKYRegion skyBeaconRegion) {
		synchronized (skyBeaconRegions) {
			skyBeaconRegions.add(skyBeaconRegion);
		}
	}

	/**
	 * 删除监听区域
	 * 
	 * @param skyBeaconRegion
	 */
	public void removeSKYBeaconRegion(SKYRegion skyBeaconRegion) {
		synchronized (skyBeaconRegions) {
			skyBeaconRegions.remove(skyBeaconRegion);
		}
	}

	/**
	 * 返回监听区域个数
	 * 
	 * @return
	 */
	public int getSKYBeaconRegionCount() {
		return skyBeaconRegions.size();
	}

	/**
	 * 获取监听状态
	 * 
	 * @param singleID
	 * @param multiID
	 * @return
	 */
	public List<Integer> getBeaconsStates(List<SKYBeacon> singleID,
			List<SKYBeaconMultiIDs> multiID, List<SKYRegion> skyRegions) {
		List<Integer> result = new ArrayList<Integer>();
		try {

			for (int j = 0; j < skyBeaconRegions.size(); ++j) {
				for (int i = 0; i < singleID.size(); ++i) {
					if (isUsed(singleID.get(i), skyBeaconRegions.get(j))) {
						if (beaconsStates.containsKey(singleID.get(i)
								.getDeviceAddress())) {
							System.out.println(LOG_TAG + "single id:"
									+ singleID.get(i).getDeviceAddress());
							System.out.println(LOG_TAG + "containsKey");
							BeaconStates stateTmp = new BeaconStates();
							stateTmp = beaconsStates.get(singleID.get(i)
									.getDeviceAddress());
							stateTmp.isScan = true;
							System.out.println(LOG_TAG + "lastState:"
									+ String.valueOf(stateTmp.state));
							// try {
							// stateTmp.skyRegion = (SKYRegion) skyBeaconRegions
							// .get(j).clone();
							// } catch (CloneNotSupportedException e) {
							// // TODO Auto-generated catch block
							// e.printStackTrace();
							// }
							stateTmp.skyRegion = new SKYRegion(
									skyBeaconRegions.get(j));
							if (stateTmp.state == STATE_READY) {
								stateTmp.state = STATE_ENTERED_INIT;
								beaconsStates.put(singleID.get(i)
										.getDeviceAddress(), stateTmp);
								result.add(STATE_ENTERED_INIT);
								skyRegions.add(skyBeaconRegions.get(j));
							} else if (stateTmp.state == STATE_ENTERED_INIT) {
								stateTmp.state = STATE_ENTERED;
								beaconsStates.put(singleID.get(i)
										.getDeviceAddress(), stateTmp);
							} else if (stateTmp.state == STATE_EXITED_INIT) {
								stateTmp.state = STATE_READY;
								beaconsStates.put(singleID.get(i)
										.getDeviceAddress(), stateTmp);
							} else if (stateTmp.state == STATE_EXITED) {
								stateTmp.state = STATE_READY;
								beaconsStates.put(singleID.get(i)
										.getDeviceAddress(), stateTmp);
							} else if (stateTmp.state == STATE_ENTERED) {
								stateTmp.state = STATE_ENTERED;
								beaconsStates.put(singleID.get(i)
										.getDeviceAddress(), stateTmp);
							}
							System.out.println(LOG_TAG + "nowState:"
									+ String.valueOf(stateTmp.state));
						} else {
							System.out.println(LOG_TAG
									+ "single not containsKey");
							System.out
									.println(LOG_TAG
											+ "device:"
											+ singleID.get(i)
													.getDeviceAddress()
											+ ","
											+ singleID.get(i)
													.getProximityUUID()
											+ ","
											+ String.valueOf(singleID.get(i)
													.getMajor())
											+ ","
											+ String.valueOf(singleID.get(i)
													.getMinor()));
							System.out.println(LOG_TAG
									+ "region:"
									+ skyBeaconRegions.get(j)
											.getDeviceAddress()
									+ ","
									+ skyBeaconRegions.get(j)
											.getProximityUUID()
									+ ","
									+ String.valueOf(skyBeaconRegions.get(j)
											.getMajor())
									+ ","
									+ String.valueOf(skyBeaconRegions.get(j)
											.getMinor()));
							BeaconStates stateTmp = new BeaconStates();
							stateTmp.isScan = true;
							beaconsStates.put(singleID.get(i)
									.getDeviceAddress(), stateTmp);
							System.out.println(LOG_TAG + "nowState:"
									+ String.valueOf(stateTmp.state));
						}
					}
				}

				for (int i = 0; i < multiID.size(); ++i) {
					if (isUsed(multiID.get(i), skyBeaconRegions.get(j))) {
						if (beaconsStates.containsKey(multiID.get(i)
								.getDeviceAddress())) {
							System.out.println(LOG_TAG + "multi id:"
									+ multiID.get(i).getDeviceAddress());
							System.out.println(LOG_TAG + "containsKey");
							BeaconStates stateTmp = new BeaconStates();
							stateTmp = beaconsStates.get(multiID.get(i)
									.getDeviceAddress());
							stateTmp.isScan = true;
							System.out.println(LOG_TAG + "lastState:"
									+ String.valueOf(stateTmp.state));
							// try {
							// stateTmp.skyRegion = (SKYRegion) skyBeaconRegions
							// .get(j).clone();
							// } catch (CloneNotSupportedException e) {
							// // TODO Auto-generated catch block
							// e.printStackTrace();
							// }
							stateTmp.skyRegion = new SKYRegion(
									skyBeaconRegions.get(j));
							if (stateTmp.state == STATE_READY) {
								stateTmp.state = STATE_ENTERED_INIT;
								beaconsStates.put(multiID.get(i)
										.getDeviceAddress(), stateTmp);
							} else if (stateTmp.state == STATE_ENTERED_INIT) {
								stateTmp.state = STATE_ENTERED;
								beaconsStates.put(multiID.get(i)
										.getDeviceAddress(), stateTmp);
								result.add(STATE_ENTERED_INIT);
								skyRegions.add(skyBeaconRegions.get(j));
							} else if (stateTmp.state == STATE_EXITED_INIT) {
								stateTmp.state = STATE_READY;
								beaconsStates.put(multiID.get(i)
										.getDeviceAddress(), stateTmp);
							} else if (stateTmp.state == STATE_EXITED) {
								stateTmp.state = STATE_READY;
								beaconsStates.put(multiID.get(i)
										.getDeviceAddress(), stateTmp);
							} else if (stateTmp.state == STATE_ENTERED) {
								stateTmp.state = STATE_ENTERED;
								beaconsStates.put(multiID.get(i)
										.getDeviceAddress(), stateTmp);
							}
							System.out.println(LOG_TAG + "nowState:"
									+ String.valueOf(stateTmp.state));
						} else {
							System.out.println(LOG_TAG
									+ "multi not containsKey");
							System.out.println(LOG_TAG + "device:"
									+ multiID.get(i).getDeviceAddress());
							System.out.println(LOG_TAG
									+ "region:"
									+ skyBeaconRegions.get(j)
											.getDeviceAddress()
									+ ","
									+ skyBeaconRegions.get(j)
											.getProximityUUID()
									+ ","
									+ String.valueOf(skyBeaconRegions.get(j)
											.getMajor())
									+ ","
									+ String.valueOf(skyBeaconRegions.get(j)
											.getMinor()));
							BeaconStates stateTmp = new BeaconStates();
							stateTmp.isScan = true;
							beaconsStates
									.put(multiID.get(i).getDeviceAddress(),
											stateTmp);
							System.out.println(LOG_TAG + "nowState:"
									+ String.valueOf(stateTmp.state));
						}
					}
				}
			}

			// 遍历数据
			Iterator<Map.Entry<String, BeaconStates>> it = beaconsStates
					.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<String, BeaconStates> entry = it.next();
				BeaconStates content = new BeaconStates();
				content = entry.getValue();
				// for (BeaconStates content : beaconsStates.values()) {
				if (!content.isScan) {
					System.out.println(LOG_TAG + "not scan:" + entry.getKey());
					System.out.println(LOG_TAG + "lastState:"
							+ String.valueOf(content.state));
					if (content.state == STATE_READY) {
						content.state = STATE_EXITED_INIT;
					} else if (content.state == STATE_ENTERED_INIT) {
						content.state = STATE_EXITED_INIT;
					} else if (content.state == STATE_ENTERED) {
						content.state = STATE_EXITED_INIT;
					} else if (content.state == STATE_EXITED) {
						content.state = STATE_EXITED;
					} else if (content.state == STATE_EXITED_INIT) {
						content.state = STATE_EXITED;
					}
					System.out.println(LOG_TAG + "nowState:"
							+ String.valueOf(content.state));
					if (content.state == STATE_EXITED_INIT) {
						for (int i = 0; i < skyBeaconRegions.size(); ++i) {
							if (skyBeaconRegions.get(i).equals(
									content.skyRegion)) {
								System.out.println(LOG_TAG + "add region:"
										+ content.skyRegion.getIdentifier());
								result.add(STATE_EXITED_INIT);
								skyRegions.add(skyBeaconRegions.get(i));
							}
						}
					}
				}
				content.isScan = false;
			}
		} catch (Exception e) {
			// TODO: handle exception
		}

		return result;
	}

	private boolean isUsed(final SKYBeacon skyBeacon,
			final SKYRegion skyBeaconRegion) {
		boolean result = false;
		// if (skyBeaconRegion != null) {
		// if (!skyBeaconRegion.getDeviceAddress().equals(
		// DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS)
		// && !skyBeaconRegion.getDeviceAddress().equals(
		// skyBeacon.getDeviceAddress())
		// && skyBeacon.getRssi() >= RSSI_MIN_THRESHOLD) {
		// return true;
		// }
		// if (!skyBeaconRegion.getProximityUUID().equals(
		// DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)
		// && !skyBeaconRegion.getProximityUUID().equals(
		// skyBeacon.getProximityUUID())
		// && skyBeacon.getRssi() >= RSSI_MIN_THRESHOLD) {
		// return true;
		// }
		// if (skyBeaconRegion.getMajor() !=
		// DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE
		// && skyBeaconRegion.getMajor() != skyBeacon.getMajor()
		// && skyBeacon.getRssi() >= RSSI_MIN_THRESHOLD) {
		// return true;
		// }
		// if (skyBeaconRegion.getMinor() !=
		// DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE
		// && skyBeaconRegion.getMinor() != skyBeacon.getMinor()
		// && skyBeacon.getRssi() >= RSSI_MIN_THRESHOLD) {
		// return true;
		// }
		// }
		if (skyBeacon.getDeviceAddress().equals("19:18:FC:00:18:6C")) {

			result = false;
		}
		if (skyBeaconRegion != null) {
			if ((skyBeaconRegion.getDeviceAddress()
					.equals(DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS))
					|| ((!skyBeaconRegion
							.getDeviceAddress()
							.equals(DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS)) && skyBeaconRegion
							.getDeviceAddress().equals(
									skyBeacon.getDeviceAddress()))) {

				if ((skyBeaconRegion.getProximityUUID()
						.equals(DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID))
						|| ((!skyBeaconRegion
								.getProximityUUID()
								.equals(DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)) && skyBeaconRegion
								.getProximityUUID().equals(
										skyBeacon.getProximityUUID()))) {
					if ((skyBeaconRegion.getMajor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE)
							|| ((skyBeaconRegion.getMajor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE) && (skyBeaconRegion
									.getMajor() == skyBeacon.getMajor()))) {
						if ((skyBeaconRegion.getMinor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE)
								|| ((skyBeaconRegion.getMinor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE) && (skyBeaconRegion
										.getMinor() == skyBeacon.getMinor()))) {
							if (skyBeacon.getRssi() >= RSSI_MIN_THRESHOLD) {
								result = true;
							}
						}
					}
				}
			}
		}

		return result;
	}

	private boolean isUsed(final SKYBeaconMultiIDs skyBeaconMultiIDs,
			final SKYRegion skyBeaconRegion) {
		boolean result = false;
		if (skyBeaconRegion != null) {
			for (int i = 0; i < skyBeaconMultiIDs.getBeaconList().size(); ++i) {
				if (isUsed(skyBeaconMultiIDs.getBeaconList().get(i),
						skyBeaconRegion)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}

	@SuppressWarnings("unused")
	private boolean isFilter(String deviceAddress, SKYRegion skyBeaconRegion) {
		boolean result = false;
		if (skyBeaconRegion != null) {
			if (!skyBeaconRegion.getDeviceAddress().equals(
					DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS)
					&& !skyBeaconRegion.getDeviceAddress()
							.equals(deviceAddress)) {
				return true;
			}
		}
		return result;
	}
}
