package com.skybeacon.sdk.locate;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.skybeacon.sdk.utils.DataConvertUtils;

public class NearbyLogicProcess {
	@SuppressWarnings("unused")
	private final String TAG = "NearbyLogicProcess";
	private final int TIME_WINDOWS_THRESHOLD_MILLISECOND = 7000;
	private final int NUMBER_WINDOWS_THRESHOLD_MILLISECOND = 4;
	private final int NUMBER_CONTINUTE_GROWTH_TIMES_THRESHOLD = 2;
	private final int NUMBER_DIFF_WITH_MEX_THRESHOLD_RSSI_1_THRESHOLD = 2;
	private final int DEFAULT_PHONE_MEASURED_POWER = -59;
	private final int DIFF_WITH_MEX_THRESHOLD_RSSI_1 = -3;
	private final int DIFF_WITH_MEX_THRESHOLD_RSSI_2 = 1;
	private final int DIFF_NEAR_DEVICE_RSSI_THRESHOLD = 5;
	private final int WEIGHT_CLASS_1 = 3;
	private final int WEIGHT_CLASS_2 = 1;
	private int phoneMeasuredPower = DEFAULT_PHONE_MEASURED_POWER;
	private SKYBeaconNearbyThreshold skyBeaconNearbyThreshold = SKYBeaconNearbyThreshold.DEFAULT;
	private int nearbyRssiThreshold = DEFAULT_PHONE_MEASURED_POWER;

	public NearbyLogicProcess() {

	}

	/**
	 * 推送返回处理
	 * 
	 * @param rawDatas
	 * @return
	 */
	public List<SKYBeacon> processNearbyLogic(Map<String, LinkedList<SKYBeacon>> rawDatas) {
		// Log.d(TAG,
		// "*******************************************************");
		// Log.d(TAG,
		// "*******************************************************");
		// Log.d(TAG,
		// "*******************************************************");
		// Log.d(TAG, "Start Process!");
		List<SKYBeacon> result = new ArrayList<SKYBeacon>();
		if (skyBeaconNearbyThreshold == SKYBeaconNearbyThreshold.DEFAULT) {
			return result;
		}
		// 按照时间窗和次数窗过滤
		Map<String, LinkedList<SKYBeacon>> filter1 = new HashMap<String, LinkedList<SKYBeacon>>();
		Iterator<Map.Entry<String, LinkedList<SKYBeacon>>> it1 = rawDatas.entrySet().iterator();
		long timestampNow = DataConvertUtils.getCurrentTimestampMillisecond();
		long difftime = 0;
		boolean isRssiEffective = false;
		// Log.d(TAG, "rawDatas Device Number: " +
		// String.valueOf(rawDatas.size()));
		while (it1.hasNext()) {
			Map.Entry<String, LinkedList<SKYBeacon>> entry = it1.next();
			LinkedList<SKYBeacon> value = entry.getValue();
			LinkedList<SKYBeacon> newValue = new LinkedList<SKYBeacon>();
			if (entry.getKey().equals("19:18:FC:00:7D:63")) {
				// Log.d(TAG, "Device Address: " + entry.getKey() +
				// ", List Number: " + value.size());
			}
			isRssiEffective = false;
			for (int i = 0; i < value.size(); i++) {
				if (i < NUMBER_WINDOWS_THRESHOLD_MILLISECOND) {
					difftime = timestampNow - value.get(i).getTimestampMillisecond();
					if (difftime < TIME_WINDOWS_THRESHOLD_MILLISECOND) {
						newValue.addLast(value.get(i));
						if ((!isRssiEffective) && (value.get(i).getRssi() > (nearbyRssiThreshold + DIFF_WITH_MEX_THRESHOLD_RSSI_1))) {
							isRssiEffective = true;
						}
					} else {
						break;
					}
				} else {
					break;
				}
			}
			if (newValue.size() > 0 && isRssiEffective) {
				filter1.put(newValue.get(0).getDeviceAddress(), newValue);
				if (entry.getKey().equals("19:18:FC:00:7D:63")) {
					// Log.d(TAG, "Effective，Size: " +
					// String.valueOf(newValue.size()));
				}
			} else {
				if (entry.getKey().equals("19:18:FC:00:7D:63")) {
					// Log.d(TAG, "Not Effective，Timeout!");
				}
			}
		}
		// Log.d(TAG, "Time and window filter, filter1 size: " +
		// String.valueOf(filter1.size()));
		// Log.d(TAG, "##################################################");
		// 分类
		// Log.d(TAG, "Start set class!");
		ArrayList<SKYBeaconNearby> filter2 = new ArrayList<NearbyLogicProcess.SKYBeaconNearby>();
		it1 = filter1.entrySet().iterator();
		while (it1.hasNext()) {
			Map.Entry<String, LinkedList<SKYBeacon>> entry = it1.next();
			LinkedList<SKYBeacon> value = entry.getValue();
			// Log.d(TAG, "Device Address: " + entry.getKey() +
			// ", List Number: " + value.size());
			SKYBeaconNearby skyBeaconNearby = new SKYBeaconNearby(value.get(0));
			boolean continuteGrow = true;
			int growTimes = 0;
			int rMinus3 = 0;
			boolean isRssiNewestPlus1 = false;
			for (int i = 0; i < value.size(); i++) {
				if ((i == 0) && (value.get(i).getRssi() > (nearbyRssiThreshold + DIFF_WITH_MEX_THRESHOLD_RSSI_2))) {
					isRssiNewestPlus1 = true;
				}
				if (value.get(i).getRssi() > (nearbyRssiThreshold + DIFF_WITH_MEX_THRESHOLD_RSSI_1)) {
					rMinus3++;
				}
				if (i != 0) {
					if (continuteGrow && (value.get(i - 1).getRssi() - value.get(i).getRssi() >= 0)) {
						growTimes++;
					} else {
						continuteGrow = false;
					}
				}
			}
			if (continuteGrow && (growTimes >= NUMBER_CONTINUTE_GROWTH_TIMES_THRESHOLD)) {
				skyBeaconNearby.setClassNumber(1);
				filter2.add(skyBeaconNearby);
			} else if (isRssiNewestPlus1 && (rMinus3 >= NUMBER_DIFF_WITH_MEX_THRESHOLD_RSSI_1_THRESHOLD)) {
				skyBeaconNearby.setClassNumber(2);
				filter2.add(skyBeaconNearby);
			}
			// Log.d(TAG, "continuteGrow: " + String.valueOf(continuteGrow) +
			// ", growTimes: " + String.valueOf(growTimes) +
			// ", isRssiNewestPlus1: " + String.valueOf(isRssiNewestPlus1) +
			// ", rMinus3: "
			// + String.valueOf(rMinus3) + ", classNumber: " +
			// String.valueOf(skyBeaconNearby.getClassNumber()));
		}
		// Log.d(TAG, "##################################################");
		// Log.d(TAG, "filter2 size: " + String.valueOf(filter2.size()));
		// 排序求权重
		Collections.sort(filter2, new Comparator<SKYBeaconNearby>() {
			@Override
			public int compare(SKYBeaconNearby s1, SKYBeaconNearby s2) {
				return s2.getRssi() - s1.getRssi();
			}

		});
		//
		ArrayList<SKYBeaconNearby> filter3 = new ArrayList<NearbyLogicProcess.SKYBeaconNearby>();
		int rssiLast = 0;
		for (int i = 0; i < filter2.size(); i++) {
			// Log.d(TAG, "Device Address: " + filter2.get(i).getDeviceAddress()
			// + ", Rssi: " + String.valueOf(filter2.get(i).getRssi()));
			if (i == 0) {
				filter3.add(filter2.get(i));
			} else {
				if ((filter2.get(i - 1).getRssi() - filter2.get(i).getRssi()) > DIFF_NEAR_DEVICE_RSSI_THRESHOLD) {
					break;
				} else {
					filter3.add(filter2.get(i));
				}
			}
			rssiLast = filter2.get(i).getRssi();
		}
		// Log.d(TAG, "##################################################");
		// Log.d(TAG, "filter3 size: " + String.valueOf(filter3.size()));
		if (rssiLast != 0) {
			int maxWeight = 0;
			for (int i = 0; i < filter3.size(); i++) {
				if (filter3.get(i).getClassNumber() == 1) {
					filter3.get(i).setWeightTmp(filter3.get(i).getRssi() - rssiLast + WEIGHT_CLASS_1);
				} else {
					filter3.get(i).setWeightTmp(filter3.get(i).getRssi() - rssiLast + WEIGHT_CLASS_2);
				}
				if (filter3.get(i).getWeightTmp() > maxWeight) {
					maxWeight = filter3.get(i).getWeightTmp();
				}
			}
			if (maxWeight != 0) {
				for (int i = 0; i < filter3.size(); i++) {
					float weightTmp1 = filter3.get(i).getWeightTmp();
					float weightTmp = ((float) Math.round((weightTmp1 / maxWeight) * 100) / 100);
					filter3.get(i).setPushWeight(weightTmp);
					SKYBeacon tmp = new SKYBeacon(filter3.get(i).getSKYBeacon());
					result.add(tmp);
					// Log.d(TAG, "Device Address: " + tmp.getDeviceAddress() +
					// ", Rssi: " + String.valueOf(tmp.getRssi()) + ", weight: "
					// + String.valueOf(tmp.getPushWeight()));
				}
				Collections.sort(result, new Comparator<SKYBeacon>() {
					@Override
					public int compare(SKYBeacon s1, SKYBeacon s2) {
						return (int) ((s2.getPushWeight() - s1.getPushWeight()) * 100);
					}

				});
			}
		}

		return result;
	}

	public int getPhoneMeasuredPower() {
		return phoneMeasuredPower;
	}

	public void setPhoneMeasuredPower(int phoneMeasuredPower) {
		this.phoneMeasuredPower = phoneMeasuredPower;
		setNearbyRssiThreshold();
	}

	public SKYBeaconNearbyThreshold getSkyBeaconNearbyThreshold() {
		return skyBeaconNearbyThreshold;
	}

	public void setSkyBeaconNearbyThreshold(SKYBeaconNearbyThreshold skyBeaconNearbyThreshold) {
		this.skyBeaconNearbyThreshold = skyBeaconNearbyThreshold;
		setNearbyRssiThreshold();
	}

	private void setNearbyRssiThreshold() {
		if (skyBeaconNearbyThreshold == SKYBeaconNearbyThreshold.NEAR_1_METER) {
			nearbyRssiThreshold = phoneMeasuredPower;
		} else if (skyBeaconNearbyThreshold == SKYBeaconNearbyThreshold.NEAR_3_METER) {
			nearbyRssiThreshold = phoneMeasuredPower - 10;
		} else if (skyBeaconNearbyThreshold == SKYBeaconNearbyThreshold.MIDDLE) {
			nearbyRssiThreshold = phoneMeasuredPower - 18;
		} else if (skyBeaconNearbyThreshold == SKYBeaconNearbyThreshold.FAR) {
			nearbyRssiThreshold = -100;
		}
	}

	public class SKYBeaconNearby extends SKYBeacon implements Serializable {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private int classNumber = -1;
		private int weightTmp = 0;

		public SKYBeaconNearby() {
			super();
		}

		public SKYBeaconNearby(SKYBeacon skyBeacon) {
			super(skyBeacon);
		}

		public SKYBeaconNearby(SKYBeaconNearby skyBeaconNearby) {
			if (skyBeaconNearby != null) {
				setClassNumber(skyBeaconNearby.getClassNumber());
				setWeightTmp(skyBeaconNearby.getWeightTmp());
				setUuidReplaceName(skyBeaconNearby.getUuidReplaceName());

				setDeviceAddress(skyBeaconNearby.getDeviceAddress());
				setDeviceName(skyBeaconNearby.getDeviceName());
				setHardwareVersion(skyBeaconNearby.getHardwareVersion());
				setFirmwareVersionMajor(skyBeaconNearby.getFirmwareVersionMajor());
				setFirmwareVersionMinor(skyBeaconNearby.getFirmwareVersionMinor());
				setProximityUUID(skyBeaconNearby.getProximityUUID());
				setMajor(skyBeaconNearby.getMajor());
				setMinor(skyBeaconNearby.getMinor());
				setMeasuredPower(skyBeaconNearby.getMeasuredPower());
				setIntervalMillisecond(skyBeaconNearby.getIntervalMillisecond());
				setTxpower(skyBeaconNearby.getTxpower());
				setRssi(skyBeaconNearby.getRssi());
				setDistance(skyBeaconNearby.getDistance());
				setBattery(skyBeaconNearby.getBattery());
				setTemperature(skyBeaconNearby.getTemperature());
				setTemperatureUpdateSecond(skyBeaconNearby.getTemperatureUpdateSecond());
				setLocked(skyBeaconNearby.isLocked());
				setEncrypted(skyBeaconNearby.isEncrypted());
				setLedState(skyBeaconNearby.getLedState());
				setSeekcyBeacon(skyBeaconNearby.isSeekcyBeacon());
				setTimestampMillisecond(skyBeaconNearby.getTimestampMillisecond());
				setLightState(skyBeaconNearby.getLightState());
				setLightThreshold(skyBeaconNearby.getLightThreshold());
				setLightValue(skyBeaconNearby.getLightValue());
				setLightUpdateSecond(skyBeaconNearby.getLightUpdateSecond());
				setDarkIntervalMillisencond(skyBeaconNearby.getDarkIntervalMillisencond());
				setPushWeight(skyBeaconNearby.getPushWeight());
			}
		}

		public void setSKYBeacon(SKYBeacon skyBeacon) {
			setDeviceAddress(skyBeacon.getDeviceAddress());
			setDeviceName(skyBeacon.getDeviceName());
			setHardwareVersion(skyBeacon.getHardwareVersion());
			setFirmwareVersionMajor(skyBeacon.getFirmwareVersionMajor());
			setFirmwareVersionMinor(skyBeacon.getFirmwareVersionMinor());
			setProximityUUID(skyBeacon.getProximityUUID());
			setMajor(skyBeacon.getMajor());
			setMinor(skyBeacon.getMinor());
			setMeasuredPower(skyBeacon.getMeasuredPower());
			setIntervalMillisecond(skyBeacon.getIntervalMillisecond());
			setTxpower(skyBeacon.getTxpower());
			setRssi(skyBeacon.getRssi());
			setDistance(skyBeacon.getDistance());
			setBattery(skyBeacon.getBattery());
			setTemperature(skyBeacon.getTemperature());
			setTemperatureUpdateSecond(skyBeacon.getTemperatureUpdateSecond());
			setLocked(skyBeacon.isLocked());
			setEncrypted(skyBeacon.isEncrypted());
			setLedState(skyBeacon.getLedState());
			setSeekcyBeacon(skyBeacon.isSeekcyBeacon());
			setTimestampMillisecond(skyBeacon.getTimestampMillisecond());
			setLightState(skyBeacon.getLightState());
			setLightThreshold(skyBeacon.getLightThreshold());
			setLightValue(skyBeacon.getLightValue());
			setLightUpdateSecond(skyBeacon.getLightUpdateSecond());
			setDarkIntervalMillisencond(skyBeacon.getDarkIntervalMillisencond());
			setPushWeight(skyBeacon.getPushWeight());
		}

		public SKYBeacon getSKYBeacon() {
			SKYBeacon skyBeacon = new SKYBeacon();
			skyBeacon.setDeviceAddress(getDeviceAddress());
			skyBeacon.setDeviceName(getDeviceName());
			skyBeacon.setHardwareVersion(getHardwareVersion());
			skyBeacon.setFirmwareVersionMajor(getFirmwareVersionMajor());
			skyBeacon.setFirmwareVersionMinor(getFirmwareVersionMinor());
			skyBeacon.setProximityUUID(getProximityUUID());
			skyBeacon.setMajor(getMajor());
			skyBeacon.setMinor(getMinor());
			skyBeacon.setMeasuredPower(getMeasuredPower());
			skyBeacon.setIntervalMillisecond(getIntervalMillisecond());
			skyBeacon.setTxpower(getTxpower());
			skyBeacon.setRssi(getRssi());
			skyBeacon.setDistance(getDistance());
			skyBeacon.setBattery(getBattery());
			skyBeacon.setTemperature(getTemperature());
			skyBeacon.setTemperatureUpdateSecond(getTemperatureUpdateSecond());
			skyBeacon.setLocked(isLocked());
			skyBeacon.setEncrypted(isEncrypted());
			skyBeacon.setLedState(getLedState());
			skyBeacon.setSeekcyBeacon(isSeekcyBeacon());
			skyBeacon.setTimestampMillisecond(getTimestampMillisecond());
			skyBeacon.setLightState(getLightState());
			skyBeacon.setLightThreshold(getLightThreshold());
			skyBeacon.setLightValue(getLightValue());
			skyBeacon.setLightUpdateSecond(getLightUpdateSecond());
			skyBeacon.setDarkIntervalMillisencond(getDarkIntervalMillisencond());
			skyBeacon.setPushWeight(getPushWeight());
			return skyBeacon;
		}

		public int getClassNumber() {
			return classNumber;
		}

		public void setClassNumber(int classNumber) {
			this.classNumber = classNumber;
		}

		public int getWeightTmp() {
			return weightTmp;
		}

		public void setWeightTmp(int weightTmp) {
			this.weightTmp = weightTmp;
		}
	}
}
