package com.skybeacon.sdk.locate;

import java.io.Serializable;

import com.skybeacon.sdk.config.LightSensorThreshold;
import com.skybeacon.sdk.config.SKYBeaconPower;
import com.skybeacon.sdk.utils.DefaultStaticValues;

public class SKYBeacon implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String deviceAddress = DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS; // 设备地址
	private String deviceName = DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME; // 设备名称
	private int hardwareVersion = DefaultStaticValues.DEFAULT_SKY_BEACON_HARDWARE_VERSION; // 硬件版本号
	private int firmwareVersionMajor = DefaultStaticValues.DEFAULT_SKY_BEACON_FIRMWARE_VERSION_MAJOR; // 主软件版本号
	private int firmwareVersionMinor = DefaultStaticValues.DEFAULT_SKY_BEACON_FIRMWARE_VERSION_MINOR; // 次软件版本号
	private String uuidReplaceName = DefaultStaticValues.DEFAULT_SKY_BEACON_UUID_REPLACE_NAME; // uuid的替代名称
	private String proximityUUID = DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID; // uuid
	private int major = DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE; // major
	private int minor = DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE; // minor
	private int measuredPower = DefaultStaticValues.DEFAULT_SKY_BEACON_MEASURED_POWER_FALSE; // 测量功率
	private int intervalMillisecond = DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE; // 发送间隔
	private SKYBeaconPower txpower = SKYBeaconPower.POWER_LEVEL_FALSE; // 发送功率
	private int rssi = DefaultStaticValues.DEFAULT_SKY_BEACON_RSSI_FALSE; // 场强
	private double distance = DefaultStaticValues.DEFAULT_SKY_BEACON_DISTANCE_FALSE; // 距离
	private int battery = DefaultStaticValues.DEFAULT_SKY_BEACON_BATTERY_FALSE; // 电量：％
	private int temperature = DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_FALSE; // 温度：摄氏度
	private int temperatureUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_UPDATE_FALSE; // 温度更新频率：秒
	private int isLocked = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE; // 是否打开防篡改模式，0表示关闭，1表示开启
	private int isEncrypted = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE; // 是否开启防蹭用模式，0表示关闭，1表示开启
	private int ledState = DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE; // led的状态值，0表示关闭，1表示开启
	private int isSeekcyBeacon = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_SEEKCY_IBEACON; // 是否是寻息iBeacon，0表示关闭，1表示开启
	private long timestampMillisecond = DefaultStaticValues.DEFAULT_SKY_BEACON_TIMESTAMP_MILLISECOND; // iBeacon的扫描时间戳
	// 光感
	private int lightState = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_STATE; // light
																						// sensor状态：0表示关闭，1表示开启
	private LightSensorThreshold lightThreshold = LightSensorThreshold.DARK_LEVEL_FALSE; // 光感黑暗门限值
	private int lightValue = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_VALUE; // 光感值Lex
	private int lightUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND; // 光感更新频率：秒
	private int darkIntervalMillisencond = DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND; // 黑暗广播频率：毫秒

	private String key = null;

	// 推送权重
	private float pushWeight = DefaultStaticValues.DEFAULT_SKY_BEACON_PUSH_WEIGHT;

	/**
	 * 构造函数
	 */
	public SKYBeacon() {
		super();
	}

	/**
	 * 构造函数
	 */
	public SKYBeacon(SKYBeacon skyBeacon) {
		super();
		this.deviceAddress = skyBeacon.deviceAddress;
		this.deviceName = skyBeacon.deviceName;
		this.hardwareVersion = skyBeacon.hardwareVersion;
		this.firmwareVersionMajor = skyBeacon.firmwareVersionMajor;
		this.firmwareVersionMinor = skyBeacon.firmwareVersionMinor;
		this.uuidReplaceName = skyBeacon.uuidReplaceName;
		this.proximityUUID = skyBeacon.proximityUUID;
		this.major = skyBeacon.major;
		this.minor = skyBeacon.minor;
		this.measuredPower = skyBeacon.measuredPower;
		this.intervalMillisecond = skyBeacon.intervalMillisecond;
		this.txpower = skyBeacon.txpower;
		this.rssi = skyBeacon.rssi;
		this.distance = skyBeacon.distance;
		this.battery = skyBeacon.battery;
		this.temperature = skyBeacon.temperature;
		this.temperatureUpdateSecond = skyBeacon.temperatureUpdateSecond;
		this.isLocked = skyBeacon.isLocked;
		this.isEncrypted = skyBeacon.isEncrypted;
		this.ledState = skyBeacon.ledState;
		this.isSeekcyBeacon = skyBeacon.isSeekcyBeacon;
		this.timestampMillisecond = skyBeacon.timestampMillisecond;
		this.lightState = skyBeacon.lightState;
		this.lightThreshold = skyBeacon.lightThreshold;
		this.lightValue = skyBeacon.lightValue;
		this.lightUpdateSecond = skyBeacon.lightUpdateSecond;
		this.darkIntervalMillisencond = skyBeacon.darkIntervalMillisencond;
		this.pushWeight = skyBeacon.pushWeight;
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceAddress
	 *            ：mac地址
	 */
	public SKYBeacon(String deviceAddress) {
		super();
		this.deviceAddress = deviceAddress;
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceAddress
	 *            ：mac地址
	 * @param proximityUUID
	 *            ：uuid
	 * @param major
	 *            ：major
	 * @param minor
	 *            ：minor
	 * @param measuredPower
	 *            ：测量功率
	 * @param rssi
	 *            ：场强
	 */
	public SKYBeacon(String deviceAddress, String proximityUUID, int major, int minor, int measuredPower, int rssi) {
		super();
		this.deviceAddress = deviceAddress;
		this.proximityUUID = proximityUUID;
		this.major = major;
		this.minor = minor;
		this.measuredPower = measuredPower;
		this.rssi = rssi;
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceAddress
	 *            ：mac地址
	 * @param hardwareVersion
	 *            ：硬件版本号
	 * @param firmwareVersionMajor
	 *            ：主固件版本号
	 * @param firmwareVersionMinor
	 *            ：次固件版本号
	 * @param proximityUUID
	 *            ：uuid
	 * @param major
	 *            ：major
	 * @param minor
	 *            ：minor
	 * @param measuredPower
	 *            ：测量功率
	 * @param intervalMillisecond
	 *            ：发送频率，ms
	 * @param txpower
	 *            ：发送功率，dBm
	 * @param rssi
	 *            ：场强，dBm
	 * @param battery
	 *            ：电量，百分比
	 * @param isLocked
	 *            ：是否防篡改模式
	 * @param isSeekcyBeacon
	 *            ：是否寻息iBeacon
	 */
	public SKYBeacon(String deviceAddress, int hardwareVersion, int firmwareVersionMajor, int firmwareVersionMinor, String proximityUUID, int major, int minor, int measuredPower,
			int intervalMillisecond, SKYBeaconPower txpower, int rssi, int battery, int isLocked, int isSeekcyBeacon) {
		super();
		this.deviceAddress = deviceAddress;
		this.hardwareVersion = hardwareVersion;
		this.firmwareVersionMajor = firmwareVersionMajor;
		this.firmwareVersionMinor = firmwareVersionMinor;
		this.proximityUUID = proximityUUID;
		this.major = major;
		this.minor = minor;
		this.measuredPower = measuredPower;
		this.intervalMillisecond = intervalMillisecond;
		this.txpower = txpower;
		this.rssi = rssi;
		this.battery = battery;
		this.isLocked = isLocked;
		this.isSeekcyBeacon = isSeekcyBeacon;
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceAddress
	 *            ：mac地址
	 * @param deviceName
	 *            ：设备名称
	 * @param hardwareVersion
	 *            ：硬件版本号
	 * @param firmwareVersionMajor
	 *            ：主固件版本号
	 * @param firmwareVersionMinor
	 *            ：次固件版本号
	 * @param uuidReplaceName
	 *            ：uuid替代名称
	 * @param proximityUUID
	 *            ：uuid
	 * @param major
	 *            ：major
	 * @param minor
	 *            ：minor
	 * @param measuredPower
	 *            ：测量功率
	 * @param intervalMillisecond
	 *            ：发送频率，ms
	 * @param txpower
	 *            ：发送功率，dBm
	 * @param rssi
	 *            ：场强，dBm
	 * @param distance
	 *            ：距离，m
	 * @param battery
	 *            ：电量，百分比
	 * @param temperature
	 *            ：温度，摄氏度
	 * @param isLocked
	 *            ：是否防篡改模式
	 * @param isEncrypted
	 *            ：是否防蹭用模式
	 * @param isSeekcyBeacon
	 *            ：是否寻息iBeacon
	 */
	public SKYBeacon(String deviceAddress, String deviceName, int hardwareVersion, int firmwareVersionMajor, int firmwareVersionMinor, String uuidReplaceName, String proximityUUID, int major,
			int minor, int measuredPower, int intervalMillisecond, SKYBeaconPower txpower, int rssi, double distance, int battery, int temperature, int isLocked, int isEncrypted, int isSeekcyBeacon) {
		super();
		this.deviceAddress = deviceAddress;
		this.deviceName = deviceName;
		this.hardwareVersion = hardwareVersion;
		this.firmwareVersionMajor = firmwareVersionMajor;
		this.firmwareVersionMinor = firmwareVersionMinor;
		this.uuidReplaceName = uuidReplaceName;
		this.proximityUUID = proximityUUID;
		this.major = major;
		this.minor = minor;
		this.measuredPower = measuredPower;
		this.intervalMillisecond = intervalMillisecond;
		this.txpower = txpower;
		this.rssi = rssi;
		this.distance = distance;
		this.battery = battery;
		this.temperature = temperature;
		this.isLocked = isLocked;
		this.isEncrypted = isEncrypted;
		this.isSeekcyBeacon = isSeekcyBeacon;
	}

	public String getDeviceAddress() {
		return deviceAddress;
	}

	public void setDeviceAddress(String deviceAddress) {
		this.deviceAddress = deviceAddress;
	}

	public String getDeviceName() {
		return deviceName;
	}

	public void setDeviceName(String deviceName) {
		this.deviceName = deviceName;
	}

	public int getHardwareVersion() {
		return hardwareVersion;
	}

	public void setHardwareVersion(int hardwareVersion) {
		this.hardwareVersion = hardwareVersion;
	}

	public int getFirmwareVersionMajor() {
		return firmwareVersionMajor;
	}

	public void setFirmwareVersionMajor(int firmwareVersionMajor) {
		this.firmwareVersionMajor = firmwareVersionMajor;
	}

	public int getFirmwareVersionMinor() {
		return firmwareVersionMinor;
	}

	public void setFirmwareVersionMinor(int firmwareVersionMinor) {
		this.firmwareVersionMinor = firmwareVersionMinor;
	}

	public String getUuidReplaceName() {
		return uuidReplaceName;
	}

	public void setUuidReplaceName(String uuidReplaceName) {
		this.uuidReplaceName = uuidReplaceName;
	}

	public String getProximityUUID() {
		return proximityUUID;
	}

	public void setProximityUUID(String proximityUUID) {
		this.proximityUUID = proximityUUID;
	}

	public int getMajor() {
		return major;
	}

	public void setMajor(int major) {
		this.major = major;
	}

	public int getMinor() {
		return minor;
	}

	public void setMinor(int minor) {
		this.minor = minor;
	}

	public int getMeasuredPower() {
		return measuredPower;
	}

	public void setMeasuredPower(int measuredPower) {
		this.measuredPower = measuredPower;
	}

	public int getIntervalMillisecond() {
		return intervalMillisecond;
	}

	public void setIntervalMillisecond(int intervalMillisecond) {
		this.intervalMillisecond = intervalMillisecond;
	}

	public SKYBeaconPower getTxpower() {
		return txpower;
	}

	public void setTxpower(SKYBeaconPower txpower) {
		this.txpower = txpower;
	}

	public int getRssi() {
		return rssi;
	}

	public void setRssi(int rssi) {
		this.rssi = rssi;
	}

	public double getDistance() {
		return distance;
	}

	public void setDistance(double distance) {
		this.distance = distance;
	}

	public int getBattery() {
		return battery;
	}

	public void setBattery(int battery) {
		this.battery = battery;
	}

	public int getTemperature() {
		return temperature;
	}

	public void setTemperature(int temperature) {
		this.temperature = temperature;
	}

	public int isLocked() {
		return isLocked;
	}

	public void setLocked(int isLocked) {
		this.isLocked = isLocked;
	}

	public int isEncrypted() {
		return isEncrypted;
	}

	public void setEncrypted(int isEncrypted) {
		this.isEncrypted = isEncrypted;
	}

	public int getLedState() {
		return ledState;
	}

	public void setLedState(int ledState) {
		this.ledState = ledState;
	}

	public int isSeekcyBeacon() {
		return isSeekcyBeacon;
	}

	public void setSeekcyBeacon(int isSeekcyBeacon) {
		this.isSeekcyBeacon = isSeekcyBeacon;
	}

	public long getTimestampMillisecond() {
		return timestampMillisecond;
	}

	public void setTimestampMillisecond(long timestampMillisecond) {
		this.timestampMillisecond = timestampMillisecond;
	}

	public int getTemperatureUpdateSecond() {
		return temperatureUpdateSecond;
	}

	public void setTemperatureUpdateSecond(int temperatureUpdateSecond) {
		this.temperatureUpdateSecond = temperatureUpdateSecond;
	}

	public int getLightState() {
		return lightState;
	}

	public void setLightState(int lightState) {
		this.lightState = lightState;
	}

	public LightSensorThreshold getLightThreshold() {
		return lightThreshold;
	}

	public void setLightThreshold(LightSensorThreshold lightThreshold) {
		this.lightThreshold = lightThreshold;
	}

	public int getLightValue() {
		return lightValue;
	}

	public void setLightValue(int lightValue) {
		this.lightValue = lightValue;
	}

	public int getLightUpdateSecond() {
		return lightUpdateSecond;
	}

	public void setLightUpdateSecond(int lightUpdateSecond) {
		this.lightUpdateSecond = lightUpdateSecond;
	}

	public int getDarkIntervalMillisencond() {
		return darkIntervalMillisencond;
	}

	public void setDarkIntervalMillisencond(int darkIntervalMillisencond) {
		this.darkIntervalMillisencond = darkIntervalMillisencond;
	}

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public float getPushWeight() {
		return pushWeight;
	}

	public void setPushWeight(float pushWeight) {
		this.pushWeight = pushWeight;
	}

	@Override
	public Object clone() {
		// TODO Auto-generated method stub
		SKYBeacon skyBeacon = null;
		try {
			skyBeacon = (SKYBeacon) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return skyBeacon;
	}

	@Override
	public boolean equals(Object o) {
		// TODO Auto-generated method stub
		return super.equals(o);
	}
}
