package com.skybeacon.sdk.locate;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.skybeacon.sdk.config.LightSensorThreshold;
import com.skybeacon.sdk.config.SKYBeaconCellsSituation;
import com.skybeacon.sdk.utils.DefaultStaticValues;

/**
 * 多ID iBeacons存储对象
 * 
 * @author frank
 * 
 */
public class SKYBeaconMultiIDs implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Object lock = new Object();
	private String deviceAddress = DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS; // 设备地址
	private String deviceName = DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME; // 设备名称
	private int hardwareVersion = DefaultStaticValues.DEFAULT_SKY_BEACON_HARDWARE_VERSION; // 硬件版本号
	private int firmwareVersionMajor = DefaultStaticValues.DEFAULT_SKY_BEACON_FIRMWARE_VERSION_MAJOR; // 主软件版本号
	private int firmwareVersionMinor = DefaultStaticValues.DEFAULT_SKY_BEACON_FIRMWARE_VERSION_MINOR; // 次软件版本号
	private int intervalMillisecond = DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE; // 发送间隔
	private int battery = DefaultStaticValues.DEFAULT_SKY_BEACON_BATTERY_FALSE; // 电量：％
	private int temperature = DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_FALSE; // 温度：摄氏度
	private int temperatureUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_UPDATE_FALSE; // 温度更新频率：秒
	private int isLocked = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE; // 是否打开防篡改模式，0表示关闭，1表示开启
	private int isEncrypted = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE; // 是否开启防蹭用模式，0表示关闭，1表示开启
	private int isSeekcyBeacon = DefaultStaticValues.DEFAULT_SKY_BEACON_IS_SEEKCY_IBEACON; // 是否是寻息iBeacon，0表示关闭，1表示开启
	private SKYBeaconCellsSituation cellsSituation = null; // 格子使用情况
	private long timestampMillisecond = DefaultStaticValues.DEFAULT_SKY_BEACON_TIMESTAMP_MILLISECOND; // iBeacon的扫描时间戳
	private List<SKYBeacon> beaconList = new ArrayList<SKYBeacon>(); // 多id的ibeaconList
	private int ledState = DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE; // led的状态值，0表示关闭，1表示开启
	// 光感
	private int lightState = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_STATE; // light
																						// sensor状态：0表示关闭，1表示开启
	private LightSensorThreshold lightThreshold = LightSensorThreshold.DARK_LEVEL_FALSE; // 光感黑暗门限值
	private int lightValue = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_VALUE; // 光感值Lex
	private int lightUpdateSecond = DefaultStaticValues.DEFAULT_SKY_BEACON_LIGHT_SENSOR_UPDATE_SENCOND; // 光感更新频率：秒
	private int darkIntervalMillisencond = DefaultStaticValues.DEFAULT_SKY_BEACON_DARK_INTERVAL_MILLISECOND; // 黑暗广播频率：毫秒

	private String key = "";

	/**
	 * 构造函数
	 */
	public SKYBeaconMultiIDs() {

	}

	/**
	 * 构造函数
	 * 
	 * @param deviceAddress
	 */
	public SKYBeaconMultiIDs(String deviceAddress) {
		super();
		this.deviceAddress = deviceAddress;
	}

	/**
	 * 构造函数
	 * 
	 * @param deviceAddress
	 * @param hardwareVersion
	 * @param firmwareVersionMajor
	 * @param firmwareVersionMinor
	 * @param intervalMillisecond
	 * @param battery
	 * @param isLocked
	 * @param isSeekcyBeacon
	 */
	public SKYBeaconMultiIDs(String deviceAddress, int hardwareVersion,
			int firmwareVersionMajor, int firmwareVersionMinor,
			int intervalMillisecond, int battery, int isLocked,
			int isSeekcyBeacon) {
		super();
		this.deviceAddress = deviceAddress;
		this.hardwareVersion = hardwareVersion;
		this.firmwareVersionMajor = firmwareVersionMajor;
		this.firmwareVersionMinor = firmwareVersionMinor;
		this.intervalMillisecond = intervalMillisecond;
		this.battery = battery;
		this.isLocked = isLocked;
		this.isSeekcyBeacon = isSeekcyBeacon;
	}

	public String getDeviceAddress() {
		return deviceAddress;
	}

	public void setDeviceAddress(String deviceAddress) {
		this.deviceAddress = deviceAddress;
	}

	public String getDeviceName() {
		return deviceName;
	}

	public void setDeviceName(String deviceName) {
		this.deviceName = deviceName;
	}

	public int getHardwareVersion() {
		return hardwareVersion;
	}

	public void setHardwareVersion(int hardwareVersion) {
		this.hardwareVersion = hardwareVersion;
	}

	public int getFirmwareVersionMajor() {
		return firmwareVersionMajor;
	}

	public void setFirmwareVersionMajor(int firmwareVersionMajor) {
		this.firmwareVersionMajor = firmwareVersionMajor;
	}

	public int getFirmwareVersionMinor() {
		return firmwareVersionMinor;
	}

	public void setFirmwareVersionMinor(int firmwareVersionMinor) {
		this.firmwareVersionMinor = firmwareVersionMinor;
	}

	public int getIntervalMillisecond() {
		return intervalMillisecond;
	}

	public void setIntervalMillisecond(int intervalMillisecond) {
		this.intervalMillisecond = intervalMillisecond;
	}

	public int getBattery() {
		return battery;
	}

	public void setBattery(int battery) {
		this.battery = battery;
	}

	public int getTemperature() {
		return temperature;
	}

	public void setTemperature(int temperature) {
		this.temperature = temperature;
	}

	public int getTemperatureUpdateSecond() {
		return temperatureUpdateSecond;
	}

	public void setTemperatureUpdateSecond(int temperatureUpdateSecond) {
		this.temperatureUpdateSecond = temperatureUpdateSecond;
	}

	public int isLocked() {
		return isLocked;
	}

	public void setLocked(int isLocked) {
		this.isLocked = isLocked;
	}

	public int isEncrypted() {
		return isEncrypted;
	}

	public void setEncrypted(int isEncrypted) {
		this.isEncrypted = isEncrypted;
	}

	public int isSeekcyBeacon() {
		return isSeekcyBeacon;
	}

	public void setSeekcyBeacon(int isSeekcyBeacon) {
		this.isSeekcyBeacon = isSeekcyBeacon;
	}

	public SKYBeaconCellsSituation getCellsSituation() {
		return cellsSituation;
	}

	public void setCellsSituation(SKYBeaconCellsSituation cellsSituation) {
		this.cellsSituation = cellsSituation;
	}

	public long getTimestampMillisecond() {
		return timestampMillisecond;
	}

	public void setTimestampMillisecond(long timestampMillisecond) {
		this.timestampMillisecond = timestampMillisecond;
	}

	public int getLedState() {
		return ledState;
	}

	public void setLedState(int ledState) {
		this.ledState = ledState;
	}

	public int getLightState() {
		return lightState;
	}

	public void setLightState(int lightState) {
		this.lightState = lightState;
	}

	public LightSensorThreshold getLightThreshold() {
		return lightThreshold;
	}

	public void setLightThreshold(LightSensorThreshold lightThreshold) {
		this.lightThreshold = lightThreshold;
	}

	public int getLightValue() {
		return lightValue;
	}

	public void setLightValue(int lightValue) {
		this.lightValue = lightValue;
	}

	public int getLightUpdateSecond() {
		return lightUpdateSecond;
	}

	public void setLightUpdateSecond(int lightUpdateSecond) {
		this.lightUpdateSecond = lightUpdateSecond;
	}

	public int getDarkIntervalMillisencond() {
		return darkIntervalMillisencond;
	}

	public void setDarkIntervalMillisencond(int darkIntervalMillisencond) {
		this.darkIntervalMillisencond = darkIntervalMillisencond;
	}

	public List<SKYBeacon> getBeaconList() {
		return beaconList;
	}

	public void setBeaconList(List<SKYBeacon> beaconList) {
		this.beaconList.clear();
		this.beaconList.addAll(beaconList);
	}

	public void addBeaconList(SKYBeacon skybeacon) {
		synchronized (lock) {
			for (int i = 0; i < beaconList.size(); i++) {
				if (beaconList.get(i).getDeviceAddress()
						.equals(skybeacon.getDeviceAddress())) {
					beaconList.remove(i);
					break;
				}
			}
			beaconList.add(skybeacon);
		}
	}

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	@Override
	public Object clone() {
		// TODO Auto-generated method stub
		SKYBeaconMultiIDs skyBeaconMultiIDs = null;
		try {
			skyBeaconMultiIDs = (SKYBeaconMultiIDs) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return skyBeaconMultiIDs;
	}

	@Override
	public boolean equals(Object o) {
		// TODO Auto-generated method stub
		return super.equals(o);
	}
}
