package com.skybeacon.sdk.locate;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.content.pm.PackageManager;
import android.util.Log;

/**
 * iBeacon扫描单例，可以开启或者关闭扫描
 * 
 * @author frank
 * 
 */
public class SKYBeaconScanUnit {
	private static SKYBeaconScanUnit instance = null;
	private static final String TAG = "SKYBeaconScanUnit";
	private Context context;
	private BluetoothAdapter bluetoothAdapter = null;
	private boolean isStart = false;

	/**
	 * 单例获取接口
	 * 
	 * @return
	 */
	public synchronized static SKYBeaconScanUnit getInstance() {
		if (instance == null) {
			instance = new SKYBeaconScanUnit();
		}
		return instance;
	}

	/**
	 * 初始化接口
	 * 
	 * @param context
	 * @return
	 */
	public boolean init(Context context) {
		boolean ret = false;
		synchronized (SKYBeaconScanUnit.this) {
			this.context = context;
			if (isSuportBLE()) {
				final BluetoothManager bluetoothManager = (BluetoothManager) context
						.getSystemService(Context.BLUETOOTH_SERVICE);
				bluetoothAdapter = null;
				bluetoothAdapter = bluetoothManager.getAdapter();
				if (bluetoothAdapter != null) {
					ret = bluetoothAdapter.isEnabled();
				}
			}
		}
		return ret;
	}

	/**
	 * 判断是否支持BLE4.0及以上
	 * 
	 * @return boolean 是否
	 */
	public boolean isSuportBLE() {
		boolean ret = false;
		try {
			ret = context.getPackageManager().hasSystemFeature(
					PackageManager.FEATURE_BLUETOOTH_LE);
		} catch (Exception e) {
			Log.e(TAG, "isSuportBeacon", e);
		}
		return ret;
	}

	/**
	 * 判断蓝牙是否打开
	 * 
	 * @return boolean 是否
	 */
	public boolean isBluetoothEnabled() {
		boolean ret = false;
		if (bluetoothAdapter != null) {
			ret = bluetoothAdapter.isEnabled();
		}
		return ret;
	}

	/**
	 * 启动iBeacon扫描
	 */
	public void start() {
		if (isSuportBLE() && isBluetoothEnabled() && (bluetoothAdapter != null)) {
			bluetoothAdapter.startLeScan(leScanCallback);
			isStart = true;
		}
	}

	/**
	 * 停止iBeacon扫描
	 */
	public void stop() {
		if (isSuportBLE() && (bluetoothAdapter != null)) {
			try {
				bluetoothAdapter.stopLeScan(leScanCallback);
				isStart = false;
			} catch (Exception e) {
				// TODO: handle exception
			}
		}
	}

	public boolean isStart() {
		return isStart;
	}

	/**
	 * 关闭结束扫描类
	 */
	public void destory() {
		if (bluetoothAdapter != null) {
			bluetoothAdapter = null;
		}
	}

	private SKYBeaconScanUnit() {

	}

	/**
	 * 扫描回调函数
	 */
	private BluetoothAdapter.LeScanCallback leScanCallback = new BluetoothAdapter.LeScanCallback() {

		@Override
		public void onLeScan(final BluetoothDevice device, final int rssi,
				final byte[] scanRecord) {
			if (device != null) {
				SKYBeaconFilter.getInstance().addDevice(device, rssi,
						scanRecord);
			}
		}
	};
}
