package com.skybeacon.sdk.sensor;

import java.util.List;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

/**
 * 方向、姿态传感器数据获取
 * 
 * @author frank
 * 
 */
public class OrientationSensor {

	@SuppressWarnings("unused")
	private static final String TAG = "OrientationSensor";

	private SensorManager sensorMagager = null;
	private Sensor accSensor = null;
	private Sensor magSensor = null;

	boolean isOrientationSensorSupport = false;

	float[] accelerationValues = new float[3];
	float[] magneticValues = new float[3];
	float[] values = new float[3];
	float[] rotate = new float[9];

	private Context context = null;
	private static OrientationSensor instance = null;

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static OrientationSensor getInstance() {
		if (instance == null) {
			instance = new OrientationSensor();
		}
		return instance;
	}

	/**
	 * 初始化
	 * 
	 * @param c
	 * @param isValid
	 */
	public boolean init(Context c, boolean isValid) {
		context = c;
		boolean isAccelerometerSupport = false;
		boolean isMagnetometerSupport = false;
		sensorMagager = (SensorManager) context
				.getSystemService(Context.SENSOR_SERVICE);
		// 得到手机上所有的传感器
		List<Sensor> listSensor = sensorMagager.getSensorList(Sensor.TYPE_ALL);
		for (Sensor sensor : listSensor) {
			if (sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				isAccelerometerSupport = true;
			} else if (sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD) {
				isMagnetometerSupport = true;
			}
		}
		if (isAccelerometerSupport && isMagnetometerSupport && isValid) {
			isOrientationSensorSupport = true;
		}
		return isOrientationSensorSupport;
	}

	/**
	 * 开始获取传感器数据
	 */
	public void start() {
		if (isOrientationSensorSupport && sensorMagager != null) {
			accSensor = sensorMagager
					.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
			magSensor = sensorMagager
					.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
			sensorMagager.registerListener(sensorEventListener, accSensor,
					SensorManager.SENSOR_DELAY_GAME);
			sensorMagager.registerListener(sensorEventListener, magSensor,
					SensorManager.SENSOR_DELAY_GAME);
		}
	}

	/**
	 * 停止方向传感器数据获取
	 */
	public void stop() {
		if (isOrientationSensorSupport && sensorMagager != null) {
			sensorMagager.unregisterListener(sensorEventListener, accSensor);
			sensorMagager.unregisterListener(sensorEventListener, magSensor);
		}
	}

	final SensorEventListener sensorEventListener = new SensorEventListener() {

		@Override
		public void onSensorChanged(SensorEvent event) {
			if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				accelerationValues = event.values;
				AccelerationEntity.getInstance().put(event.values);
			}
			if (event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD) {
				magneticValues = event.values;
				MagneticEntity.getInstance().put(event.values);
			}

			SensorManager.getRotationMatrix(rotate, null, accelerationValues,
					magneticValues);
			SensorManager.getOrientation(rotate, values);
			// 经过SensorManager.getOrientation(rotate, values);
			// 得到的values值为弧度转换为角度
			values[0] = (float) Math.toDegrees(values[0]);
			if (values[0] < 0) {
				values[0] += 360;
			}
			OrientationEntity.getInstance().put(values);
		}

		@Override
		public void onAccuracyChanged(Sensor arg0, int arg1) {

		}

	};

	private OrientationSensor() {

	}

}
