package com.skybeacon.sdk.sensor;

import java.util.List;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

public class PressureSensor {

	private Context context = null;
	private boolean isPressureSensorValid = false;
	private SensorManager sensorMagager;

	private static PressureSensor preSensor = null;

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public synchronized static PressureSensor getInstance() {
		if (preSensor == null) {
			preSensor = new PressureSensor();
		}
		return preSensor;
	}

	public boolean init(Context c, boolean isValid) {
		context = c;
		boolean isPreValid = false;
		sensorMagager = (SensorManager) context
				.getSystemService(Context.SENSOR_SERVICE);
		// 得到手机上所有的传感器
		List<Sensor> listSensor = sensorMagager.getSensorList(Sensor.TYPE_ALL);
		for (Sensor sensor : listSensor) {
			if (sensor.getType() == Sensor.TYPE_PRESSURE) {
				isPreValid = true;
				break;
			}
		}
		if (isPreValid && isValid) {
			isPressureSensorValid = true;
		}
		return isPressureSensorValid;
	}

	/**
	 * 开始读取传感器数据
	 */
	public void start() {
		if (isPressureSensorValid && sensorMagager != null) {
			sensorMagager.registerListener(pressureListener,
					sensorMagager.getDefaultSensor(Sensor.TYPE_PRESSURE),
					SensorManager.SENSOR_DELAY_GAME);
		}
	}

	/**
	 * 停止读取传感器数据
	 */
	public void stop() {
		if (isPressureSensorValid && sensorMagager != null) {
			sensorMagager.unregisterListener(pressureListener);
		}
	}

	private SensorEventListener pressureListener = new SensorEventListener() {

		@Override
		public void onSensorChanged(SensorEvent event) {
			if (event.sensor.getType() == Sensor.TYPE_PRESSURE) {
				PressureEntity.getInstance().put(event.values[0]);
			}
		}

		@Override
		public void onAccuracyChanged(Sensor sensor, int accuracy) {

		}
	};
}
