package com.skybeacon.sdk.sensor;

import android.content.Context;

public class SensorAPI {

	private static SensorAPI instance = null;
	private Context context;
	private boolean isOrientationValid = false;
	private boolean isPressureValid = false;

	/**
	 * 传感器类型枚举，被封装过，方向传感器由：磁场、加速度计传感器合并
	 * 
	 * @author frank
	 * 
	 */
	public enum SensorType {
		ORIENTATION, PRESSURE
	};

	/**
	 * 单例获取
	 * 
	 * @return
	 */
	public synchronized static SensorAPI getInstance() {
		if (instance == null) {
			instance = new SensorAPI();
		}
		return instance;
	}

	/**
	 * 初始化
	 * 
	 * @param context
	 */
	public void init(Context context) {
		this.context = context;
	}

	/**
	 * 开始搜集数据
	 */
	public void start() {
		OrientationSensor.getInstance().start();
		PressureSensor.getInstance().start();
	}

	/**
	 * 停止搜集数据
	 */
	public void stop() {
		OrientationSensor.getInstance().stop();
		PressureSensor.getInstance().stop();
	}

	/**
	 * 设置传感器
	 * 
	 * @param sensorType
	 * @param isValid
	 * @return
	 */
	public boolean setSensor(SensorType sensorType, boolean isValid) {
		boolean result = false;
		switch (sensorType) {
		case ORIENTATION:
			isOrientationValid = isValid;
			result = OrientationSensor.getInstance().init(context,
					isOrientationValid);
			break;
		case PRESSURE:
			isPressureValid = isValid;
			result = PressureSensor.getInstance()
					.init(context, isPressureValid);
			break;
		default:
			break;
		}
		return result;
	}

	private SensorAPI() {

	}
}
