package com.skybeacon.sdk.utils;

import android.annotation.SuppressLint;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

public class DataConvertUtils {

	/**
	 * byte数组转十六进制String
	 * 
	 * @param src
	 *            输入转换的数组
	 * @return 结果
	 */
	public static String bytesToHexString(byte[] src) {
		StringBuilder stringBuilder = new StringBuilder("");
		if (src == null || src.length <= 0) {
			return null;
		}
		for (int i = 0; i < src.length; i++) {
			int v = src[i] & 0xFF;
			String hv = Integer.toHexString(v).toUpperCase();
			if (hv.length() < 2) {
				stringBuilder.append(0);
			}
			stringBuilder.append(hv);
		}
		return stringBuilder.toString();
	}

	/**
	 * byte数组转MacString，格式：00:00:00:00:00:00
	 * 
	 * @param src
	 * @return 结果
	 */
	public static String bytesToMacString(byte[] src) {
		String hexString = bytesToHexString(src);
		StringBuilder result = new StringBuilder();
		result.append(hexString.substring(0, 2));
		result.append(":");
		result.append(hexString.substring(2, 4));
		result.append(":");
		result.append(hexString.substring(4, 6));
		result.append(":");
		result.append(hexString.substring(6, 8));
		result.append(":");
		result.append(hexString.substring(8, 10));
		result.append(":");
		result.append(hexString.substring(10, 12));
		return result.toString();
	}

	/**
	 * 十六进制String转byte数组
	 * 
	 * @param hex
	 *            十六进制String
	 * @return 结果
	 */
	public static byte[] hexStringToByte(String hex) {
		int len = (hex.length() / 2);
		byte[] result = new byte[len];
		char[] achar = hex.toCharArray();
		for (int i = 0; i < len; i++) {
			int pos = i * 2;
			result[i] = (byte) (toByte(achar[pos]) << 4 | toByte(achar[pos + 1]));
		}
		return result;
	}

	private static byte toByte(char c) {
		byte b = (byte) "0123456789ABCDEF".indexOf(c);
		if (b == -1) {
			b = (byte) "0123456789abcdef".indexOf(c);
		}
		return b;
	}

	/**
	 * byte[2]转int32
	 * 
	 * @param b
	 * @return 结果
	 */
	public static int byte2ToInt(byte[] b) {
		int value = 0;
		for (int i = 0; i < 2; i++) {
			int shift = (2 - 1 - i) * 8;
			value += (b[i] & 0x000000FF) << shift;
		}
		return value;
	}

	/**
	 * byte[4]转int32
	 * 
	 * @param b
	 * @return 结果
	 */
	public static int byte4ToInt(byte[] b) {
		int value = 0;
		for (int i = 0; i < 4; i++) {
			int shift = (4 - 1 - i) * 8;
			value += (b[i] & 0x000000FF) << shift;
		}
		return value;
	}

	/**
	 * proximityUuid转byte数组
	 * 
	 * @param proximityUuid
	 * @return 结果
	 */
	public static byte[] proximityUuidToBytes(String proximityUuid) {
		byte[] result = new byte[16];

		byte[] tmp0 = new byte[32];
		byte[] tmp1 = null;
		try {
			tmp1 = proximityUuid.getBytes("ISO-8859-1");
			int i = 0, j = 0;
			for (; i < tmp1.length; i++) {
				if (i != 8 && i != 13 && i != 18 && i != 23) {
					if (j < 32) {
						tmp0[j] = tmp1[i];
						j++;
					}
				}
			}
			String tmp2 = new String(tmp0, "ISO-8859-1");
			result = hexStringToByte(tmp2);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return result;
	}

	/**
	 * mac String转byte数组
	 * 
	 * @param macString
	 * @return 结果
	 */
	public static byte[] macStringToBytes(String macString) {
		byte[] result = new byte[6];

		byte[] tmp0 = new byte[12];
		byte[] tmp1 = null;
		try {
			tmp1 = macString.getBytes("ISO-8859-1");
			int i = 0, j = 0;
			for (; i < tmp1.length; i++) {
				if (i != 2 && i != 5 && i != 8 && i != 11 && i != 14) {
					if (j < 12) {
						tmp0[j] = tmp1[i];
						j++;
					}
				}
			}
			String tmp2 = new String(tmp0, "ISO-8859-1");
			result = hexStringToByte(tmp2);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return result;
	}

	/**
	 * byte数组转proximityUuid String
	 * 
	 * @param uuid
	 * @return 结果
	 */
	public static String byte16ToProximityUuidStr(byte[] uuid) {
		String hexString = bytesToHexString(uuid);
		StringBuilder sb = new StringBuilder();
		sb.append(hexString.substring(0, 8));
		sb.append("-");
		sb.append(hexString.substring(8, 12));
		sb.append("-");
		sb.append(hexString.substring(12, 16));
		sb.append("-");
		sb.append(hexString.substring(16, 20));
		sb.append("-");
		sb.append(hexString.substring(20, 32));
		return sb.toString();
	}

	/**
	 * int转byte[2]
	 * 
	 * @param in
	 * @return 结果
	 */
	public static byte[] intToByteArrayTwo(int in) {
		byte[] result = new byte[2];
		result[0] = (byte) ((in >> 8) & 0xFF);
		result[1] = (byte) (in & 0xFF);
		return result;
	}

	/**
	 * 获取系统时间
	 * 
	 * @return 结果
	 */
	public static long getCurrentTimestampMillisecond() {
		return System.currentTimeMillis();
	}

	/**
	 * 获取系统时间，精确到天
	 * 
	 * @return
	 */
	@SuppressLint("SimpleDateFormat")
	public static String getCurrentTimestampDay() {
		String result = "";
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
		result = formatter.format(new java.util.Date());
		return result;
	}

	/**
	 * 获取系统时间，精确到秒
	 * 
	 * @return
	 */
	@SuppressLint("SimpleDateFormat")
	public static String getCurrentTimestampSecond() {
		String result = "";
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		result = formatter.format(new java.util.Date());
		return result;
	}

	/**
	 * 7个字节分别为：0:秒 1:分 2:时 3:星期 4:日 5:月 6:年
	 * 
	 * @return 结果
	 */
	public static byte[] getCurrentTimeBytes() {
		Calendar c = Calendar.getInstance();
		int year = c.get(Calendar.YEAR) - 2000;
		int month = c.get(Calendar.MONTH) + 1;
		int day = c.get(Calendar.DAY_OF_MONTH);
		int week = c.get(Calendar.DAY_OF_WEEK);
		int hour = c.get(Calendar.HOUR_OF_DAY);
		int minute = c.get(Calendar.MINUTE);
		int second = c.get(Calendar.SECOND);

		byte[] re = new byte[7];
		re[6] = digitTo2SegmentHex((byte) (year));
		re[5] = digitTo2SegmentHex((byte) month);
		re[4] = digitTo2SegmentHex((byte) day);
		re[3] = digitTo2SegmentHex((byte) week);
		re[2] = digitTo2SegmentHex((byte) hour);
		re[1] = digitTo2SegmentHex((byte) minute);
		re[0] = digitTo2SegmentHex((byte) second);

		return re;
	}

	private static byte digitTo2SegmentHex(byte b) {
		byte r = (byte) ((b / 10) << 4);
		r |= (byte) ((0xff & b) % 10);
		return r;
	}
}
