package com.skybeacon.sdk.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.util.ArrayList;

import org.apache.http.util.EncodingUtils;

public class FileAdapter {
	/**
	 * read file
	 * 
	 * @param fileName
	 * @return
	 * @throws IOException
	 */
	static public String readSDFile(String fileName) throws IOException {

		File file = new File(fileName);

		FileInputStream fis = new FileInputStream(file);

		int length = fis.available();

		byte[] buffer = new byte[length];
		fis.read(buffer);

		String res = EncodingUtils.getString(buffer, "UTF-8");

		fis.close();
		return res;
	}

	/**
	 * write file
	 * 
	 * @param filePath
	 * @param fileName
	 * @param write_str
	 * @throws IOException
	 */
	static public void writeSDFile(String filePath, String fileName,
			String write_str) throws IOException {

		makeRootDirectory(filePath);
		File file = new File(filePath + "/" + fileName);
		if (!file.exists()) {
			file.createNewFile();
		}
		FileOutputStream fos = new FileOutputStream(file);

		byte[] bytes = write_str.getBytes();

		fos.write(bytes);

		fos.close();
	}

	/**
	 * write file append
	 * 
	 * @param filePath
	 * @param fileName
	 * @param write_str
	 * @throws IOException
	 */
	static public void writeSDFileAppend(String filePath, String fileName,
			String write_str) throws IOException {

		makeRootDirectory(filePath);
		File file = new File(filePath + "/" + fileName);
		if (!file.exists()) {
			file.createNewFile();
		}
		FileOutputStream fos = new FileOutputStream(file, true);

		byte[] bytes = write_str.getBytes();

		fos.write(bytes);

		fos.close();
	}

	/**
	 * make derectory
	 * 
	 * @param filePath
	 */
	public static void makeRootDirectory(String filePath) {
		File file = null;
		try {
			file = new File(filePath);
			if (!file.exists()) {
				file.mkdirs();
			}
		} catch (Exception e) {

		}
	}

	/**
	 * 获取某目录下的包含关键词keyword的所有文件目录
	 * 
	 * @param filePath
	 * @param keyword
	 * @return
	 */
	public static ArrayList<String> searchFile(String filePath, String keyword) {
		ArrayList<String> result = new ArrayList<String>();
		File[] files = new File(filePath).listFiles();
		if (files != null) {
			for (File file : files) {
				if (keyword == null) {
					result.add(file.getName());
				} else if (keyword.equals("")) {
					result.add(file.getName());
				} else if (file.getName().indexOf(keyword) >= 0) {
					result.add(file.getName());
				} else {
					result.add(file.getName());
				}
			}
		}

		return result;
	}

	/**
	 * 递归删除文件夹
	 * 
	 * @param file
	 */
	public static void deleteFiles(File filePath) {
		if (filePath.isFile()) {
			filePath.delete();
			return;
		}
		if (filePath.isDirectory()) {
			File[] childFile = filePath.listFiles();
			if (childFile == null || childFile.length == 0) {
				filePath.delete();
				return;
			}
			for (File f : childFile) {
				deleteFiles(f);
			}
			filePath.delete();
		}
	}

	/**
	 * 获取文件的md5
	 * 
	 * @param filename
	 * @return
	 */
	public static String getFileMd5(String filename) {
		InputStream fis;
		byte[] buffer = new byte[1024];
		int numRead = 0;
		MessageDigest md5;
		try {
			fis = new FileInputStream(filename);
			md5 = MessageDigest.getInstance("MD5");
			while ((numRead = fis.read(buffer)) > 0) {
				md5.update(buffer, 0, numRead);
			}
			fis.close();
			return DataConvertUtils.bytesToHexString(md5.digest());
		} catch (Exception e) {
			System.out.println("error");
			return null;
		}
	}

	/**
	 * 递归删除文件夹
	 * 
	 * @param file
	 */
	public static boolean deleteDirectory(String filePath) {
		boolean flag = false;
		// 如果filePath不以文件分隔符结尾，自动添加文件分隔符
		if (!filePath.endsWith(File.separator)) {
			filePath = filePath + File.separator;
		}
		File dirFile = new File(filePath);
		if (!dirFile.exists() || !dirFile.isDirectory()) {
			return false;
		}
		flag = true;
		File[] files = dirFile.listFiles();
		// 遍历删除文件夹下的所有文件(包括子目录)
		for (int i = 0; i < files.length; i++) {
			if (files[i].isFile()) {
				// 删除子文件
				flag = deleteFile(files[i].getAbsolutePath());
				if (!flag)
					break;
			} else {
				// 删除子目录
				flag = deleteDirectory(files[i].getAbsolutePath());
				if (!flag)
					break;
			}
		}
		if (!flag)
			return false;
		// 删除当前空目录
		return dirFile.delete();
	}

	/**
	 * 根据路径删除指定的目录或文件，无论存在与否
	 * 
	 * @param filePath
	 *            要删除的目录或文件
	 * @return 删除成功返回 true，否则返回 false。
	 */
	public static boolean deleteFolder(String filePath) {
		File file = new File(filePath);
		if (!file.exists()) {
			return false;
		} else {
			if (file.isFile()) {
				// 为文件时调用删除文件方法
				return deleteFile(filePath);
			} else {
				// 为目录时调用删除目录方法
				return deleteDirectory(filePath);
			}
		}
	}

	/**
	 * 删除单个文件
	 * 
	 * @param filePath
	 *            被删除文件的文件名
	 * @return 文件删除成功返回true，否则返回false
	 */
	public static boolean deleteFile(String filePath) {
		File file = new File(filePath);
		if (file.isFile() && file.exists()) {
			return file.delete();
		}
		return false;
	}

	/**
	 * 删除目录下所有文件和目录
	 * 
	 * @param filePath
	 * @return 文件删除成功返回true，否则返回false
	 */
	public static boolean deleteAllInFolder(String filePath) {
		boolean flag = false;
		// 如果filePath不以文件分隔符结尾，自动添加文件分隔符
		if (!filePath.endsWith(File.separator)) {
			filePath = filePath + File.separator;
		}
		File dirFile = new File(filePath);
		if (!dirFile.exists() || !dirFile.isDirectory()) {
			return false;
		}
		flag = true;
		File[] files = dirFile.listFiles();
		// 遍历删除文件夹下的所有文件(包括子目录)
		for (int i = 0; i < files.length; i++) {
			if (files[i].isFile()) {
				// 删除子文件
				flag = deleteFile(files[i].getAbsolutePath());
				if (!flag)
					break;
			} else {
				// 删除子目录
				flag = deleteDirectory(files[i].getAbsolutePath());
				if (!flag)
					break;
			}
		}
		return flag;
	}

	/**
	 * 复制单个文件
	 * 
	 * @param oldPath
	 *            String 原文件路径 如：c:/fqf.txt
	 * @param newPath
	 *            String 复制后路径 如：f:/fqf.txt
	 * @return boolean
	 */
	public static void copyFile(String oldPath, String newPath) {
		try {
			int bytesum = 0;
			int byteread = 0;
			File oldfile = new File(oldPath);
			if (!oldfile.exists()) { // 文件不存在时
				InputStream inStream = new FileInputStream(oldPath); // 读入原文件
				@SuppressWarnings("resource")
				FileOutputStream fs = new FileOutputStream(newPath);
				byte[] buffer = new byte[1444];
				while ((byteread = inStream.read(buffer)) != -1) {
					bytesum += byteread; // 字节数 文件大小
					System.out.println(bytesum);
					fs.write(buffer, 0, byteread);
				}
				inStream.close();
			}
		} catch (Exception e) {
			System.out.println("复制单个文件操作出错");
			e.printStackTrace();
		}
	}
}
